@@ -19,8 +19,6 @@
 
 #include "info.h"
 
-#include <unistd.h> /* R_OK access() */
-
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <stddef.h> /* NULL size_t */
@@ -28,13 +26,17 @@
 #include <stdlib.h> /* abs() free() realloc() */
 #include <string.h> /* memset() strtol() strcmp() strchr() strlen() */
 
+#include "../compat/os.h"
 #include "../engine/cmds.h"
+#include "../engine/options.h"
+#include "../ui/ui.h"
 #include "../utils/file_streams.h"
 #include "../utils/filter.h"
 #include "../utils/fs.h"
 #include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
+#include "../utils/matcher.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
@@ -44,11 +46,11 @@
 #include "../dir_stack.h"
 #include "../filelist.h"
 #include "../filetype.h"
+#include "../fileview.h"
 #include "../opt_handlers.h"
 #include "../registers.h"
 #include "../status.h"
 #include "../trash.h"
-#include "../ui.h"
 #include "config.h"
 #include "hist.h"
 #include "info_chars.h"
@@ -63,7 +65,11 @@ static void set_view_property(FileView *view, char type, const char value[]);
 static int copy_file(const char src[], const char dst[]);
 static int copy_file_internal(FILE *const src, FILE *const dst);
 static void update_info_file(const char filename[]);
+static void process_hist_entry(FileView *view, const char dir[],
+		const char file[], int pos, char ***lh, int *nlh, int **lhp, size_t *nlhp);
 static char * convert_old_trash_path(const char trash_path[]);
+static int assoc_exists(assoc_list_t *assocs, const char pattern[],
+		const char cmd[]);
 static void write_options(FILE *const fp);
 static void write_assocs(FILE *fp, const char str[], char mark,
 		assoc_list_t *assocs, int prev_count, char *prev[]);
@@ -98,7 +104,7 @@ read_info_file(int reread)
 
 	snprintf(info_file, sizeof(info_file), "%s/vifminfo", cfg.config_dir);
 
-	if((fp = fopen(info_file, "r")) == NULL)
+	if((fp = os_fopen(info_file, "r")) == NULL)
 		return;
 
 	while((line = read_vifminfo_line(fp, line)) != NULL)
@@ -123,31 +129,48 @@ read_info_file(int reread)
 				process_set_args(line_val);
 			}
 		}
-		else if(type == LINE_TYPE_FILETYPE)
+		else if(type == LINE_TYPE_FILETYPE || type == LINE_TYPE_XFILETYPE)
 		{
 			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
-				/* This is to prevent old builtin fake associations to be loaded. */
-				if(!ends_with(line2, "}" VIFM_PSEUDO_CMD))
+				char *error;
+				matcher_t *m;
+				const int x = (type == LINE_TYPE_XFILETYPE);
+
+				/* Prevent loading of old builtin fake associations. */
+				if(ends_with(line2, "}" VIFM_PSEUDO_CMD))
 				{
-					set_programs(line_val, line2, 0,
+					continue;
+				}
+
+				m = matcher_alloc(line_val, 0, 1, &error);
+				if(m == NULL)
+				{
+					/* Ignore error description. */
+					free(error);
+				}
+				else
+				{
+					ft_set_programs(m, line2, x,
 							curr_stats.exec_env_type == EET_EMULATOR_WITH_X);
 				}
 			}
 		}
-		else if(type == LINE_TYPE_XFILETYPE)
-		{
-			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
-			{
-				set_programs(line_val, line2, 1,
-						curr_stats.exec_env_type == EET_EMULATOR_WITH_X);
-			}
-		}
 		else if(type == LINE_TYPE_FILEVIEWER)
 		{
 			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
-				set_fileviewer(line_val, line2);
+				char *error;
+				matcher_t *const m = matcher_alloc(line_val, 0, 1, &error);
+				if(m == NULL)
+				{
+					/* Ignore error description. */
+					free(error);
+				}
+				else
+				{
+					ft_set_viewers(m, line2);
+				}
 			}
 		}
 		else if(type == LINE_TYPE_COMMAND)
@@ -157,7 +180,7 @@ read_info_file(int reread)
 				char *cmdadd_cmd;
 				if((cmdadd_cmd = format_str("command %s %s", line_val, line2)) != NULL)
 				{
-					exec_commands(cmdadd_cmd, curr_view, GET_COMMAND);
+					exec_commands(cmdadd_cmd, curr_view, CIT_COMMAND);
 					free(cmdadd_cmd);
 				}
 			}
@@ -194,7 +217,6 @@ read_info_file(int reread)
 			if(!reread)
 			{
 				const int i = atoi(line_val);
-				cfg.show_one_window = (i == 1);
 				curr_stats.number_of_windows = (i == 1) ? 1 : 2;
 			}
 		}
@@ -233,19 +255,19 @@ read_info_file(int reread)
 		}
 		else if(type == LINE_TYPE_CMDLINE_HIST)
 		{
-			append_to_history(&cfg.cmd_hist, save_command_history, line_val);
+			append_to_history(&cfg.cmd_hist, cfg_save_command_history, line_val);
 		}
 		else if(type == LINE_TYPE_SEARCH_HIST)
 		{
-			append_to_history(&cfg.search_hist, save_search_history, line_val);
+			append_to_history(&cfg.search_hist, cfg_save_search_history, line_val);
 		}
 		else if(type == LINE_TYPE_PROMPT_HIST)
 		{
-			append_to_history(&cfg.prompt_hist, save_prompt_history, line_val);
+			append_to_history(&cfg.prompt_hist, cfg_save_prompt_history, line_val);
 		}
 		else if(type == LINE_TYPE_FILTER_HIST)
 		{
-			append_to_history(&cfg.filter_hist, save_filter_history, line_val);
+			append_to_history(&cfg.filter_hist, cfg_save_filter_history, line_val);
 		}
 		else if(type == LINE_TYPE_DIR_STACK)
 		{
@@ -296,7 +318,7 @@ read_info_file(int reread)
 		else if(type == LINE_TYPE_USE_SCREEN)
 		{
 			const int i = atoi(line_val);
-			set_use_term_multiplexer(i != 0);
+			cfg_set_use_term_multiplexer(i != 0);
 		}
 		else if(type == LINE_TYPE_COLORSCHEME)
 		{
@@ -341,7 +363,7 @@ get_sort_info(FileView *view, const char line[])
 	}
 	memset(&view->sort[j], SK_NONE, sizeof(view->sort) - j);
 
-	reset_view_sort(view);
+	fview_sorting_updated(view);
 }
 
 /* Appends item to the hist extending the history to fit it if needed. */
@@ -360,7 +382,7 @@ ensure_history_not_full(hist_t *hist)
 {
 	if(hist->pos + 1 == cfg.history_len)
 	{
-		resize_history(MAX(0, cfg.history_len + 1));
+		cfg_resize_histories(cfg.history_len + 1);
 		assert(hist->pos + 1 != cfg.history_len && "Failed to resize history.");
 	}
 }
@@ -371,7 +393,7 @@ get_history(FileView *view, int reread, const char *dir, const char *file,
 {
 	if(view->history_num == cfg.history_len)
 	{
-		resize_history(MAX(0, cfg.history_len + 1));
+		cfg_resize_histories(cfg.history_len + 1);
 	}
 
 	if(!reread)
@@ -413,7 +435,7 @@ write_info_file(void)
 	(void)snprintf(info_file, sizeof(info_file), "%s/vifminfo", cfg.config_dir);
 	(void)snprintf(tmp_file, sizeof(tmp_file), "%s_%u", info_file, get_pid());
 
-	if(access(info_file, R_OK) != 0 || copy_file(info_file, tmp_file) == 0)
+	if(os_access(info_file, R_OK) != 0 || copy_file(info_file, tmp_file) == 0)
 	{
 		update_info_file(tmp_file);
 
@@ -429,8 +451,8 @@ write_info_file(void)
 static int
 copy_file(const char src[], const char dst[])
 {
-	FILE *const src_fp = fopen(src, "rb");
-	FILE *const dst_fp = fopen(dst, "wb");
+	FILE *const src_fp = os_fopen(src, "rb");
+	FILE *const dst_fp = os_fopen(dst, "wb");
 	int result;
 
 	result = copy_file_internal(src_fp, dst_fp);
@@ -504,7 +526,7 @@ update_info_file(const char filename[])
 
 	non_conflicting_bmarks = strdup(valid_bookmarks);
 
-	if((fp = fopen(filename, "r")) != NULL)
+	if((fp = os_fopen(filename, "r")) != NULL)
 	{
 		size_t nlhp = 0UL, nrhp = 0UL, nbt = 0UL;
 		char *line = NULL, *line2 = NULL, *line3 = NULL, *line4 = NULL;
@@ -520,44 +542,30 @@ update_info_file(const char filename[])
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					assoc_record_t prog;
-					if(get_default_program_for_file(line_val, &prog))
+					if(!assoc_exists(&filetypes, line_val, line2))
 					{
-						free_assoc_record(&prog);
-						continue;
+						nft = add_to_string_array(&ft, nft, 2, line_val, line2);
 					}
-					nft = add_to_string_array(&ft, nft, 2, line_val, line2);
 				}
 			}
 			else if(type == LINE_TYPE_XFILETYPE)
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					assoc_record_t x_prog;
-					if(get_default_program_for_file(line_val, &x_prog))
+					if(!assoc_exists(&xfiletypes, line_val, line2))
 					{
-						assoc_record_t console_prog;
-						if(get_default_program_for_file(line_val, &console_prog))
-						{
-							if(strcmp(x_prog.command, console_prog.command) == 0)
-							{
-								free_assoc_record(&console_prog);
-								free_assoc_record(&x_prog);
-								continue;
-							}
-						}
-						free_assoc_record(&x_prog);
+						nfx = add_to_string_array(&fx, nfx, 2, line_val, line2);
 					}
-					nfx = add_to_string_array(&fx, nfx, 2, line_val, line2);
 				}
 			}
 			else if(type == LINE_TYPE_FILEVIEWER)
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					if(get_viewer_for_file(line_val) != NULL)
-						continue;
-					nfv = add_to_string_array(&fv, nfv, 2, line_val, line2);
+					if(!assoc_exists(&fileviewers, line_val, line2))
+					{
+						nfv = add_to_string_array(&fv, nfv, 2, line_val, line2);
+					}
 				}
 			}
 			else if(type == LINE_TYPE_COMMAND)
@@ -582,47 +590,23 @@ update_info_file(const char filename[])
 					ncmds = add_to_string_array(&cmds, ncmds, 2, line_val, line2);
 				}
 			}
-			else if(type == LINE_TYPE_LWIN_HIST)
+			else if(type == LINE_TYPE_LWIN_HIST || type == LINE_TYPE_RWIN_HIST)
 			{
 				if(line_val[0] == '\0')
 					continue;
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					int pos;
+					const int pos = read_optional_number(fp);
 
-					if(lwin.history_pos + nlh/2 == cfg.history_len - 1)
-						continue;
-					if(is_in_view_history(&lwin, line_val))
-						continue;
-
-					pos = read_optional_number(fp);
-					nlh = add_to_string_array(&lh, nlh, 2, line_val, line2);
-					if(nlh/2 > nlhp)
+					if(type == LINE_TYPE_LWIN_HIST)
 					{
-						nlhp = add_to_int_array(&lhp, nlhp, pos);
-						nlhp = MIN(nlh/2, nlhp);
+						process_hist_entry(&lwin, line_val, line2, pos, &lh, &nlh, &lhp,
+								&nlhp);
 					}
-				}
-			}
-			else if(type == LINE_TYPE_RWIN_HIST)
-			{
-				if(line_val[0] == '\0')
-					continue;
-				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
-				{
-					int pos;
-
-					if(rwin.history_pos + nrh/2 == cfg.history_len - 1)
-						continue;
-					if(is_in_view_history(&rwin, line_val))
-						continue;
-
-					pos = read_optional_number(fp);
-					nrh = add_to_string_array(&rh, nrh, 2, line_val, line2);
-					if(nrh/2 > nrhp)
+					else
 					{
-						nrhp = add_to_int_array(&rhp, nrhp, pos);
-						nrhp = MIN(nrh/2, nrhp);
+						process_hist_entry(&rwin, line_val, line2, pos, &rh, &nrh, &rhp,
+								&nrhp);
 					}
 				}
 			}
@@ -728,7 +712,7 @@ update_info_file(const char filename[])
 		fclose(fp);
 	}
 
-	if((fp = fopen(filename, "w")) != NULL)
+	if((fp = os_fopen(filename, "w")) != NULL)
 	{
 		fprintf(fp, "# You can edit this file by hand, but it's recommended not to "
 				"do that.\n");
@@ -838,6 +822,25 @@ update_info_file(const char filename[])
 	free(non_conflicting_bmarks);
 }
 
+/* Handles single directory history entry, possibly skipping merging it in. */
+static void
+process_hist_entry(FileView *view, const char dir[], const char file[], int pos,
+		char ***lh, int *nlh, int **lhp, size_t *nlhp)
+{
+	if(view->history_pos + *nlh/2 == cfg.history_len - 1 ||
+			is_in_view_history(view, dir) || !is_dir(dir))
+	{
+		return;
+	}
+
+	*nlh = add_to_string_array(lh, *nlh, 2, dir, file);
+	if(*nlh/2U > *nlhp)
+	{
+		*nlhp = add_to_int_array(lhp, *nlhp, pos);
+		*nlhp = MIN(*nlh/2U, *nlhp);
+	}
+}
+
 /* Performs conversions on files in trash required for partial backward
  * compatibility.  Returns newly allocated string that should be freed by the
  * caller. */
@@ -847,7 +850,7 @@ convert_old_trash_path(const char trash_path[])
 	if(!is_path_absolute(trash_path) && is_dir_writable(cfg.trash_dir))
 	{
 		char *const full_path = format_str("%s/%s", cfg.trash_dir, trash_path);
-		if(path_exists(full_path))
+		if(path_exists(full_path, DEREF))
 		{
 			return full_path;
 		}
@@ -856,6 +859,34 @@ convert_old_trash_path(const char trash_path[])
 	return strdup(trash_path);
 }
 
+/* Checks that given pair of pattern and command exists in specified list of
+ * associations.  Returns non-zero if so, otherwise zero is returned. */
+static int
+assoc_exists(assoc_list_t *assocs, const char pattern[], const char cmd[])
+{
+	int i;
+	for(i = 0; i < assocs->count; ++i)
+	{
+		int j;
+
+		const assoc_t assoc = assocs->list[i];
+		if(strcmp(matcher_get_expr(assoc.matcher), pattern) == 0)
+		{
+			continue;
+		}
+
+		for(j = 0; j < assoc.records.count; ++j)
+		{
+			const assoc_record_t ft_record = assoc.records.list[j];
+			if(strcmp(ft_record.command, cmd) == 0)
+			{
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
 /* Writes current values of all options into vifminfo file. */
 static void
 write_options(FILE *const fp)
@@ -864,6 +895,7 @@ write_options(FILE *const fp)
 	fprintf(fp, "=aproposprg=%s\n", escape_spaces(cfg.apropos_prg));
 	fprintf(fp, "=%sautochpos\n", cfg.auto_ch_pos ? "" : "no");
 	fprintf(fp, "=cdpath=%s\n", cfg.cd_path);
+	fprintf(fp, "=%schaselinks\n", cfg.chase_links ? "" : "no");
 	fprintf(fp, "=columns=%d\n", cfg.columns);
 	fprintf(fp, "=%sconfirm\n", cfg.confirm ? "" : "no");
 	fprintf(fp, "=cpoptions=%s%s%s\n",
@@ -885,15 +917,16 @@ write_options(FILE *const fp)
 	fprintf(fp, "=%siec\n", cfg.use_iec_prefixes ? "" : "no");
 	fprintf(fp, "=%signorecase\n", cfg.ignore_case ? "" : "no");
 	fprintf(fp, "=%sincsearch\n", cfg.inc_search ? "" : "no");
-	fprintf(fp, "=%slaststatus\n", cfg.last_status ? "" : "no");
+	fprintf(fp, "=%slaststatus\n", cfg.display_statusline ? "" : "no");
 	fprintf(fp, "=lines=%d\n", cfg.lines);
 	fprintf(fp, "=locateprg=%s\n", escape_spaces(cfg.locate_prg));
+	fprintf(fp, "=mintimeoutlen=%d\n", cfg.min_timeout_len);
 	fprintf(fp, "=rulerformat=%s\n", escape_spaces(cfg.ruler_format));
 	fprintf(fp, "=%srunexec\n", cfg.auto_execute ? "" : "no");
 	fprintf(fp, "=%sscrollbind\n", cfg.scroll_bind ? "" : "no");
 	fprintf(fp, "=scrolloff=%d\n", cfg.scroll_off);
 	fprintf(fp, "=shell=%s\n", escape_spaces(cfg.shell));
-	fprintf(fp, "=shortmess=%s\n", cfg.trunc_normal_sb_msgs ? "T" : "");
+	fprintf(fp, "=shortmess=%s\n", escape_spaces(get_option_value("shortmess")));
 #ifndef _WIN32
 	fprintf(fp, "=slowfs=%s\n", escape_spaces(cfg.slow_fs_list));
 #endif
@@ -966,6 +999,7 @@ write_options(FILE *const fp)
 
 	fprintf(fp, "=%svimhelp\n", cfg.use_vim_help ? "" : "no");
 	fprintf(fp, "=%swildmenu\n", cfg.wild_menu ? "" : "no");
+	fprintf(fp, "=wordchars=%s\n", escape_spaces(get_option_value("wordchars")));
 	fprintf(fp, "=%swrap\n", cfg.wrap_quick_view ? "" : "no");
 }
 
@@ -975,30 +1009,39 @@ write_assocs(FILE *fp, const char str[], char mark, assoc_list_t *assocs,
 		int prev_count, char *prev[])
 {
 	int i;
+
 	fprintf(fp, "\n# %s:\n", str);
-	for(i = 0; i < assocs->count; i++)
+
+	for(i = 0; i < assocs->count; ++i)
 	{
 		int j;
+
 		assoc_t assoc = assocs->list[i];
-		for(j = 0; j < assoc.records.count; j++)
+
+		for(j = 0; j < assoc.records.count; ++j)
 		{
 			assoc_record_t ft_record = assoc.records.list[j];
+
 			/* The type check is to prevent builtin fake associations to be written
 			 * into vifminfo file. */
-			if(ft_record.command[0] != '\0' && ft_record.type != ART_BUILTIN)
+			if(ft_record.command[0] == '\0' || ft_record.type == ART_BUILTIN)
 			{
-				if(ft_record.description[0] == '\0')
-				{
-					fprintf(fp, "%c%s\n\t%s\n", mark, assoc.pattern, ft_record.command);
-				}
-				else
-				{
-					fprintf(fp, "%c%s\n\t{%s}%s\n", mark, assoc.pattern,
-							ft_record.description, ft_record.command);
-				}
+				continue;
+			}
+
+			if(ft_record.description[0] == '\0')
+			{
+				fprintf(fp, "%c%s\n\t%s\n", mark, matcher_get_expr(assoc.matcher),
+						ft_record.command);
+			}
+			else
+			{
+				fprintf(fp, "%c%s\n\t{%s}%s\n", mark, matcher_get_expr(assoc.matcher),
+						ft_record.description, ft_record.command);
 			}
 		}
 	}
+
 	for(i = 0; i < prev_count; i += 2)
 	{
 		fprintf(fp, "%c%s\n\t%s\n", mark, prev[i], prev[i + 1]);
@@ -1150,8 +1193,8 @@ write_dir_stack(FILE *const fp, char *dir_stack[], int ndir_stack)
 	fputs("\n# Directory stack (oldest to newest):\n", fp);
 	if(dir_stack_changed())
 	{
-		int i;
-		for(i = 0; i < stack_top; i++)
+		unsigned int i;
+		for(i = 0U; i < stack_top; ++i)
 		{
 			fprintf(fp, "S%s\n\t%s\n", stack[i].lpane_dir, stack[i].lpane_file);
 			fprintf(fp, "S%s\n\t%s\n", stack[i].rpane_dir, stack[i].rpane_file);
@@ -1252,11 +1295,15 @@ static void
 put_sort_info(FILE *fp, char leading_char, const FileView *view)
 {
 	int i = -1;
+	const char *const sort = (flist_custom_active(view) && view->custom.unsorted)
+	                       ? view->custom.sort
+	                       : view->sort;
+
 	fputc(leading_char, fp);
-	while(++i < SK_COUNT && abs(view->sort[i]) <= SK_LAST)
+	while(++i < SK_COUNT && abs(sort[i]) <= SK_LAST)
 	{
-		int is_last_option = i >= SK_COUNT - 1 || abs(view->sort[i + 1]) > SK_LAST;
-		fprintf(fp, "%d%s", view->sort[i], is_last_option ? "" : ",");
+		int is_last_option = i >= SK_COUNT - 1 || abs(sort[i + 1]) > SK_LAST;
+		fprintf(fp, "%d%s", sort[i], is_last_option ? "" : ",");
 	}
 	fputc('\n', fp);
 }
