@@ -21,22 +21,30 @@
 
 #include <curses.h>
 
-#include <stddef.h> /* size_t */
+#include <regex.h> /* regcomp() regexec() */
+
+#include <assert.h> /* assert() */
+#include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* free() */
+#include <stdlib.h> /* free() realloc() */
 #include <string.h> /* strcpy() strlen() */
 
 #include "cfg/config.h"
+#include "compat/os.h"
 #include "engine/completion.h"
-#include "menus/menus.h"
+#include "modes/dialogs/msg_dialog.h"
+#include "ui/statusbar.h"
+#include "ui/ui.h"
 #include "utils/fs.h"
 #include "utils/fs_limits.h"
 #include "utils/macros.h"
+#include "utils/matcher.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
 #include "utils/tree.h"
+#include "utils/utils.h"
+#include "color_manager.h"
 #include "status.h"
-#include "ui.h"
 
 char *HI_GROUPS[] = {
 	[WIN_COLOR]          = "Win",
@@ -52,13 +60,14 @@ char *HI_GROUPS[] = {
 	[TOP_LINE_COLOR]     = "TopLine",
 	[TOP_LINE_SEL_COLOR] = "TopLineSel",
 	[STATUS_LINE_COLOR]  = "StatusLine",
-	[MENU_COLOR]         = "WildMenu",
+	[WILD_MENU_COLOR]    = "WildMenu",
 	[CMD_LINE_COLOR]     = "CmdLine",
 	[ERROR_MSG_COLOR]    = "ErrorMsg",
 	[BORDER_COLOR]       = "Border",
 	[OTHER_LINE_COLOR]   = "OtherLine",
+	[JOB_LINE_COLOR]     = "JobLine",
 };
-ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR - 2);
+ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR);
 
 char *LIGHT_COLOR_NAMES[8] = {
 	[COLOR_BLACK] = "lightblack",
@@ -330,7 +339,8 @@ char *XTERM256_COLOR_NAMES[256] = {
 	[255] = "Grey93",
 };
 
-static const int default_colors[][3] = {
+/* Default color scheme definition. */
+static const col_attr_t default_cs[] = {
 	                      /* fg             bg           attr */
 	[WIN_COLOR]          = { COLOR_WHITE,   COLOR_BLACK, 0                       },
 	[DIRECTORY_COLOR]    = { COLOR_CYAN,    -1,          A_BOLD                  },
@@ -345,18 +355,24 @@ static const int default_colors[][3] = {
 	[TOP_LINE_COLOR]     = { COLOR_BLACK,   COLOR_WHITE, 0                       },
 	[TOP_LINE_SEL_COLOR] = { COLOR_BLACK,   -1,          A_BOLD                  },
 	[STATUS_LINE_COLOR]  = { COLOR_BLACK,   COLOR_WHITE, A_BOLD                  },
-	[MENU_COLOR]         = { COLOR_WHITE,   COLOR_BLACK, A_UNDERLINE | A_REVERSE },
+	[WILD_MENU_COLOR]    = { COLOR_WHITE,   COLOR_BLACK, A_UNDERLINE | A_REVERSE },
 	[CMD_LINE_COLOR]     = { COLOR_WHITE,   COLOR_BLACK, 0                       },
 	[ERROR_MSG_COLOR]    = { COLOR_RED,     COLOR_BLACK, 0                       },
 	[BORDER_COLOR]       = { COLOR_BLACK,   COLOR_WHITE, 0                       },
 	[OTHER_LINE_COLOR]   = { -1,            -1,          -1                      },
+	[JOB_LINE_COLOR]     = { COLOR_BLACK,   COLOR_WHITE, A_BOLD | A_REVERSE      },
 };
-ARRAY_GUARD(default_colors, MAXNUM_COLOR - 2);
+ARRAY_GUARD(default_cs, MAXNUM_COLOR);
 
+static char ** list_color_scheme_files(int *len);
 static void restore_primary_color_scheme(const col_scheme_t *cs);
 static void reset_to_default_color_scheme(col_scheme_t *cs);
+static void free_color_scheme_highlights(col_scheme_t *cs);
+static file_hi_t * clone_color_scheme_highlights(const col_scheme_t *from);
 static void reset_color_scheme_colors(col_scheme_t *cs);
-static void load_color_pairs(int base, const col_scheme_t *cs);
+static int source_cs(const char name[]);
+static void get_cs_path(const char name[], char buf[], size_t buf_size);
+static void load_color_pairs(col_scheme_t *cs);
 static void ensure_dirs_tree_exists(void);
 
 static tree_t dirs = NULL_TREE;
@@ -366,48 +382,110 @@ check_color_scheme(col_scheme_t *cs)
 {
 	if(cs->state == CSS_BROKEN)
 	{
-		int i;
-		for(i = 0; i < ARRAY_LEN(default_colors); i++)
-		{
-			cs->color[i].fg = default_colors[i][0];
-			cs->color[i].bg = default_colors[i][1];
-			cs->color[i].attr = default_colors[i][2];
-		}
-
+		reset_color_scheme_colors(cs);
 		cs->state = CSS_DEFAULTED;
 	}
 }
 
+int
+cs_have_no_extensions(void)
+{
+	int len;
+	char **list = list_color_scheme_files(&len);
+	int i;
+
+	/* Check if extensions are in use. */
+	for(i = 0; i < len; ++i)
+	{
+		if(ends_with(list[i], ".vifm"))
+		{
+			break;
+		}
+	}
+
+	free_string_array(list, len);
+	return (len > 0 && i >= len);
+}
+
 char **
 list_color_schemes(int *len)
 {
-	char colors_dir[PATH_MAX];
-	snprintf(colors_dir, sizeof(colors_dir), "%s/colors", cfg.config_dir);
-	return list_regular_files(colors_dir, len);
+	char **const list = list_color_scheme_files(len);
+	int i, j;
+	int new_names;
+
+	/* Check if extensions are in use. */
+	new_names = 0;
+	for(i = 0; i < *len; ++i)
+	{
+		if(ends_with(list[i], ".vifm"))
+		{
+			new_names = 1;
+		}
+	}
+
+	/* Remove hidden files, maybe along with old style file names, maybe cut off
+	 * extensions from new style file names. */
+	j = 0;
+	for(i = 0; i < *len; ++i)
+	{
+		if(list[i][0] == '.' || (new_names && !cut_suffix(list[i], ".vifm")))
+		{
+			free(list[i]);
+		}
+		else
+		{
+			list[j++] = list[i];
+		}
+	}
+	*len = j;
+
+	return list;
+}
+
+/* Lists names of all files in color schemes directories.  Allocates an array of
+ * strings, which should be freed by the caller.  Always sets *len.  Returns
+ * NULL on error. */
+static char **
+list_color_scheme_files(int *len)
+{
+	char **list = NULL;
+	*len = 0;
+
+	list = list_regular_files(cfg.colors_dir, list, len);
+	list = list_regular_files(GLOBAL_COLORS_DIR, list, len);
+
+	return list;
 }
 
 int
 color_scheme_exists(const char name[])
 {
-	char full_path[PATH_MAX];
-	snprintf(full_path, sizeof(full_path), "%s/colors/%s", cfg.config_dir, name);
-	return is_regular_file(full_path);
+	char cs_path[PATH_MAX];
+	get_cs_path(name, cs_path, sizeof(cs_path));
+	return is_regular_file(cs_path);
 }
 
 void
 write_color_scheme_file(void)
 {
 	FILE *fp;
-	char colors_dir[PATH_MAX];
-	int i;
+	char def_cs_path[PATH_MAX];
+	size_t i;
 
-	snprintf(colors_dir, sizeof(colors_dir), "%s/colors", cfg.config_dir);
-	if(make_dir(colors_dir, 0777) != 0)
+	if(is_dir(cfg.colors_dir) || make_path(cfg.colors_dir, S_IRWXU) != 0)
+	{
+		/* Do nothing if local colors directory exists or we've failed to create
+		 * it. */
 		return;
+	}
 
-	strncat(colors_dir, "/Default", sizeof(colors_dir) - strlen(colors_dir) - 1);
-	if((fp = fopen(colors_dir, "w")) == NULL)
+	snprintf(def_cs_path, sizeof(def_cs_path), "%s/Default.vifm", cfg.colors_dir);
+	fp = os_fopen(def_cs_path, "w");
+	if(fp == NULL)
+	{
 		return;
+	}
 
 	fprintf(fp, "\" You can edit this file by hand.\n");
 	fprintf(fp, "\" The \" character at the beginning of a line comments out the line.\n");
@@ -454,7 +532,9 @@ write_color_scheme_file(void)
 
 	fprintf(fp, "\" highlight group cterm=attrs ctermfg=foreground_color ctermbg=background_color\n\n");
 
-	for(i = 0; i < MAXNUM_COLOR - 2; ++i)
+	fprintf(fp, "highlight clear\n\n");
+
+	for(i = 0U; i < ARRAY_LEN(default_cs); ++i)
 	{
 		char fg_buf[16], bg_buf[16];
 
@@ -464,11 +544,11 @@ write_color_scheme_file(void)
 			continue;
 		}
 
-		color_to_str(cfg.cs.color[i].fg, sizeof(fg_buf), fg_buf);
-		color_to_str(cfg.cs.color[i].bg, sizeof(bg_buf), bg_buf);
+		color_to_str(default_cs[i].fg, sizeof(fg_buf), fg_buf);
+		color_to_str(default_cs[i].bg, sizeof(bg_buf), bg_buf);
 
 		fprintf(fp, "highlight %s cterm=%s ctermfg=%s ctermbg=%s\n", HI_GROUPS[i],
-				attrs_to_str(cfg.cs.color[i].attr), fg_buf, bg_buf);
+				attrs_to_str(default_cs[i].attr), fg_buf, bg_buf);
 	}
 
 	fclose(fp);
@@ -481,7 +561,7 @@ color_to_str(int color, size_t buf_len, char str_buf[])
 	{
 		copy_str(str_buf, buf_len, "default");
 	}
-	else if(color >= 0 && color < ARRAY_LEN(XTERM256_COLOR_NAMES))
+	else if(color >= 0 && color < (int)ARRAY_LEN(XTERM256_COLOR_NAMES))
 	{
 		copy_str(str_buf, buf_len, XTERM256_COLOR_NAMES[color]);
 	}
@@ -495,7 +575,6 @@ int
 load_primary_color_scheme(const char name[])
 {
 	col_scheme_t prev_cs;
-	char full[PATH_MAX];
 
 	if(!color_scheme_exists(name))
 	{
@@ -504,16 +583,14 @@ load_primary_color_scheme(const char name[])
 	}
 
 	prev_cs = cfg.cs;
-	curr_stats.cs_base = DCOLOR_BASE;
 	curr_stats.cs = &cfg.cs;
 	cfg.cs.state = CSS_LOADING;
 
-	snprintf(full, sizeof(full), "%s/colors/%s", cfg.config_dir, name);
-	if(source_file(full) != 0)
+	if(source_cs(name) != 0)
 	{
 		restore_primary_color_scheme(&prev_cs);
 		show_error_msgf("Color Scheme Sourcing",
-				"Errors loading colors cheme: \"%s\"", name);
+				"An error occurred on loading color scheme: \"%s\"", name);
 		cfg.cs.state = CSS_NORMAL;
 		return 0;
 	}
@@ -547,9 +624,9 @@ load_color_scheme_colors(void)
 {
 	ensure_dirs_tree_exists();
 
-	load_color_pairs(DCOLOR_BASE, &cfg.cs);
-	load_color_pairs(LCOLOR_BASE, &lwin.cs);
-	load_color_pairs(RCOLOR_BASE, &rwin.cs);
+	load_color_pairs(&cfg.cs);
+	load_color_pairs(&lwin.cs);
+	load_color_pairs(&rwin.cs);
 }
 
 void
@@ -558,15 +635,16 @@ load_def_scheme(void)
 	tree_free(dirs);
 	dirs = NULL_TREE;
 
+	lwin.local_cs = 0;
+	rwin.local_cs = 0;
+
 	reset_to_default_color_scheme(&cfg.cs);
 	reset_to_default_color_scheme(&lwin.cs);
-	lwin.color_scheme = LCOLOR_BASE;
 	reset_to_default_color_scheme(&rwin.cs);
-	rwin.color_scheme = RCOLOR_BASE;
 
-	load_color_pairs(DCOLOR_BASE, &cfg.cs);
-	load_color_pairs(LCOLOR_BASE, &lwin.cs);
-	load_color_pairs(RCOLOR_BASE, &rwin.cs);
+	load_color_pairs(&cfg.cs);
+	load_color_pairs(&lwin.cs);
+	load_color_pairs(&rwin.cs);
 }
 
 /* Completely resets the cs to builtin default color scheme.  Changes: colors,
@@ -576,114 +654,174 @@ reset_to_default_color_scheme(col_scheme_t *cs)
 {
 	reset_color_scheme_colors(cs);
 
-	snprintf(cs->name, sizeof(cs->name), "%s", DEF_CS_NAME);
-	snprintf(cs->dir, sizeof(cs->dir), "%s", "/");
+	copy_str(cs->name, sizeof(cs->name), DEF_CS_NAME);
+	copy_str(cs->dir, sizeof(cs->dir), "/");
 
 	cs->state = CSS_NORMAL;
 }
 
 void
-reset_color_scheme(int color_base, col_scheme_t *cs)
+reset_color_scheme(col_scheme_t *cs)
 {
 	reset_color_scheme_colors(cs);
-	load_color_pairs(color_base, cs);
+	free_color_scheme_highlights(cs);
+	load_color_pairs(cs);
+}
+
+void
+assign_color_scheme(col_scheme_t *to, const col_scheme_t *from)
+{
+	free_color_scheme_highlights(to);
+	*to = *from;
+	to->file_hi = clone_color_scheme_highlights(from);
 }
 
 /* Resets color scheme to default builtin values. */
 static void
 reset_color_scheme_colors(col_scheme_t *cs)
 {
-	int i;
-
-	for(i = 0; i < ARRAY_LEN(default_colors); i++)
+	size_t i;
+	for(i = 0U; i < ARRAY_LEN(default_cs); ++i)
 	{
-		cs->color[i].fg = default_colors[i][0];
-		cs->color[i].bg = default_colors[i][1];
-		cs->color[i].attr = default_colors[i][2];
-	}
-
-	for(i = ARRAY_LEN(default_colors); i < MAXNUM_COLOR; i++)
-	{
-		cs->color[i].fg = -1;
-		cs->color[i].bg = -1;
-		cs->color[i].attr = 0;
+		cs->color[i] = default_cs[i];
 	}
 }
 
-/* The return value is the color scheme base number for the colorpairs.
- *
- * The color scheme with the longest matching directory path is the one that
- * should be returned.
- */
+/* Frees data structures of the color scheme that are related to filename
+ * specific highlight. */
+static void
+free_color_scheme_highlights(col_scheme_t *cs)
+{
+	int i;
+
+	for(i = 0; i < cs->file_hi_count; ++i)
+	{
+		file_hi_t *const hi = &cs->file_hi[i];
+		matcher_free(hi->matcher);
+	}
+
+	free(cs->file_hi);
+
+	cs->file_hi = NULL;
+	cs->file_hi_count = 0;
+}
+
+/* Clones filename specific highlight array of the *from color scheme and
+ * returns it. */
+static file_hi_t *
+clone_color_scheme_highlights(const col_scheme_t *from)
+{
+	int i;
+	file_hi_t *file_hi = malloc(sizeof(*from->file_hi)*(from->file_hi_count + 1));
+
+	for(i = 0; i < from->file_hi_count; ++i)
+	{
+		const file_hi_t *const hi = &from->file_hi[i];
+		file_hi[i].matcher = matcher_clone(hi->matcher);
+		file_hi[i].hi = hi->hi;
+	}
+
+	return file_hi;
+}
+
 int
-check_directory_for_color_scheme(int left, const char *dir)
+check_directory_for_color_scheme(int left, const char dir[])
 {
 	char *p;
 	char t;
+	int altered;
 
 	union
 	{
 		char *name;
 		tree_val_t buf;
-	}u;
+	}
+	u;
 
 	if(dirs == NULL_TREE)
 	{
-		return DCOLOR_BASE;
+		return 0;
 	}
 
-	curr_stats.cs_base = left ? LCOLOR_BASE : RCOLOR_BASE;
 	curr_stats.cs = left ? &lwin.cs : &rwin.cs;
-	*curr_stats.cs = cfg.cs;
+	assign_color_scheme(curr_stats.cs, &cfg.cs);
 
+	/* TODO: maybe use split_and_get() here as in io/iop:iop_mkdir(). */
 	p = (char *)dir;
+	altered = 0;
 	do
 	{
-		char full[PATH_MAX];
 		t = *p;
 		*p = '\0';
 
-		if(tree_get_data(dirs, dir, &u.buf) != 0 || !color_scheme_exists(u.name))
+		if(tree_get_data(dirs, dir, &u.buf) == 0 && color_scheme_exists(u.name))
 		{
-			*p = t;
-			if((p = strchr(p + 1, '/')) == NULL)
-				p = (char *)dir + strlen(dir);
-			continue;
+			(void)source_cs(u.name);
+			altered = 1;
 		}
 
-		snprintf(full, sizeof(full), "%s/colors/%s", cfg.config_dir, u.name);
-		(void)source_file(full);
-
 		*p = t;
-		if((p = strchr(p + 1, '/')) == NULL)
-			p = (char *)dir + strlen(dir);
+		p = until_first(p + 1, '/');
 	}
 	while(t != '\0');
 
-	check_color_scheme(curr_stats.cs);
-	load_color_pairs(curr_stats.cs_base, curr_stats.cs);
-	curr_stats.cs_base = DCOLOR_BASE;
 	curr_stats.cs = &cfg.cs;
 
-	return left ? LCOLOR_BASE : RCOLOR_BASE;
+	if(!altered)
+	{
+		return 0;
+	}
+
+	check_color_scheme(curr_stats.cs);
+	load_color_pairs(curr_stats.cs);
+
+	return 1;
 }
 
+/* Sources color scheme file.  Returns non-zero on error (e.g. file doesn't
+ * exist), and zero otherwise. */
+static int
+source_cs(const char name[])
+{
+	char cs_path[PATH_MAX];
+	get_cs_path(name, cs_path, sizeof(cs_path));
+	return cfg_source_file(cs_path);
+}
+
+/* Fill the buffer of specified size with path to color scheme, which might not
+ * exist. */
 static void
-load_color_pairs(int base, const col_scheme_t *cs)
+get_cs_path(const char name[], char buf[], size_t buf_size)
+{
+	snprintf(buf, buf_size, "%s/%s.vifm", cfg.colors_dir, name);
+	if(is_regular_file(buf))
+	{
+		return;
+	}
+
+	(void)cut_suffix(buf, ".vifm");
+	if(is_regular_file(buf))
+	{
+		return;
+	}
+
+	snprintf(buf, buf_size, GLOBAL_COLORS_DIR "/%s.vifm", name);
+	if(is_regular_file(buf))
+	{
+		return;
+	}
+
+	(void)cut_suffix(buf, ".vifm");
+}
+
+/* Loads color scheme settings into color pairs. */
+static void
+load_color_pairs(col_scheme_t *cs)
 {
 	int i;
 	for(i = 0; i < MAXNUM_COLOR; i++)
 	{
-		/* XXX: This is an ugly hack to avoid flickering of top line of the current
-		 * view.  We need colors attributes recalculated correctly before applying
-		 * them, otherwise color changes twice on the screen.  Need to generalize
-		 * this by updating all color pairs that we can, or that depend on group,
-		 * whose properties are changed.  Note that TOP_LINE_SEL_COLOR is mixed in
-		 * ui.c and will be initialized on redraw. */
-		if(i != TOP_LINE_SEL_COLOR)
-		{
-			init_pair(base + i, cs->color[i].fg, cs->color[i].bg);
-		}
+		cs->pair[i] = colmgr_get_pair(cs->color[i].fg, cs->color[i].bg);
 	}
 }
 
@@ -780,6 +918,55 @@ mix_colors(col_attr_t *base, const col_attr_t *mixup)
 	}
 }
 
+int
+add_file_hi(struct matcher_t *matcher, const col_attr_t *hi)
+{
+	file_hi_t *file_hi;
+	col_scheme_t *const cs = curr_stats.cs;
+
+	void *const p = realloc(cs->file_hi,
+			sizeof(*cs->file_hi)*(cs->file_hi_count + 1));
+	if(p == NULL)
+	{
+		show_error_msg("Color Scheme File Highlight", "Not enough memory");
+		return 1;
+	}
+	cs->file_hi = p;
+
+	file_hi = &cs->file_hi[cs->file_hi_count];
+
+	file_hi->matcher = matcher;
+	file_hi->hi = *hi;
+
+	++cs->file_hi_count;
+
+	return 0;
+}
+
+const col_attr_t *
+get_file_hi(const col_scheme_t *cs, const char fname[], int *hi_hint)
+{
+	int i;
+
+	if(*hi_hint != -1)
+	{
+		assert(*hi_hint >= 0 && "Wrong index.");
+		assert(*hi_hint < cs->file_hi_count && "Wrong index.");
+		return &cs->file_hi[*hi_hint].hi;
+	}
+
+	for(i = 0; i < cs->file_hi_count; ++i)
+	{
+		const file_hi_t *const file_hi = &cs->file_hi[i];
+		if(matcher_matches(file_hi->matcher, fname))
+		{
+			*hi_hint = i;
+			return &file_hi->hi;
+		}
+	}
+	return NULL;
+}
+
 int
 is_color_set(const col_attr_t *color)
 {
