@@ -20,12 +20,11 @@
 #ifndef VIFM__COLOR_SCHEME_H__
 #define VIFM__COLOR_SCHEME_H__
 
+#include <regex.h> /* regex_t */
+
 #include <stddef.h> /* size_t */
 
 #include "utils/fs_limits.h"
-#ifdef _WIN32
-#include "utils/utils.h"
-#endif
 #include "colors.h"
 
 /* Pseudo name of the default built-in color scheme. */
@@ -41,12 +40,27 @@ typedef enum
 }
 ColorSchemeState;
 
+struct matcher_t;
+
+/* Single file highlight description. */
+typedef struct
+{
+	struct matcher_t *matcher; /* Name matcher object. */
+	col_attr_t hi;             /* File appearance parameters. */
+}
+file_hi_t;
+
+/* Color scheme description. */
 typedef struct
 {
 	char name[NAME_MAX];            /* Name of the color scheme. */
 	char dir[PATH_MAX];             /* Associated root dir of the color scheme. */
 	ColorSchemeState state;         /* Current state. */
 	col_attr_t color[MAXNUM_COLOR]; /* Colors with their attributes. */
+	int pair[MAXNUM_COLOR];         /* Pairs for corresponding color. */
+
+	file_hi_t *file_hi; /* List of file highlight preferences. */
+	int file_hi_count;  /* Number of file highlight definitions. */
 }
 col_scheme_t;
 
@@ -65,9 +79,18 @@ void load_color_scheme_colors(void);
 void load_def_scheme(void);
 
 /* Resets color scheme to default builtin values and reloads them. */
-void reset_color_scheme(int color_base, col_scheme_t *cs);
+void reset_color_scheme(col_scheme_t *cs);
 
-int check_directory_for_color_scheme(int left, const char *dir);
+/* Copies data from *from to *to. */
+void assign_color_scheme(col_scheme_t *to, const col_scheme_t *from);
+
+/* The color scheme with the longest matching directory path is the one that
+ * is chosen.  Return non-zero if non-default colorscheme should be used for the
+ * specified directory, otherwise zero is returned. */
+int check_directory_for_color_scheme(int left, const char dir[]);
+
+/* Checks whether local colorschemes do not have file extensions. */
+int cs_have_no_extensions(void);
 
 /* Lists names of all color schemes.  Allocates an array of strings, which
  * should be freed by the caller.  Always sets *len.  Returns NULL on error. */
@@ -94,6 +117,16 @@ void color_to_str(int color, size_t buf_len, char str_buf[]);
 
 void mix_colors(col_attr_t *base, const col_attr_t *mixup);
 
+/* Registers pattern-highlight pair for active color scheme.  Returns new value
+ * for curr_stats.save_msg. */
+int add_file_hi(struct matcher_t *matcher, const col_attr_t *hi);
+
+/* Gets filename specific highlight.  hi_hint can't be NULL and should be equal
+ * to -1 initially.  Returns NULL if nothing was found, otherwise returns
+ * pointer to one of color scheme's highlights. */
+const col_attr_t * get_file_hi(const col_scheme_t *cs, const char fname[],
+		int *hi_hint);
+
 /* Checks that color is non-empty (e.g. set from outside).  Returns non-zero if
  * so, otherwise zero is returned. */
 int is_color_set(const col_attr_t *color);
