@@ -33,7 +33,7 @@ col_attr_t;
  * color scheme. */
 enum
 {
-	WIN_COLOR,          /* Pane background. */
+	WIN_COLOR,          /* Pane background and default file highlight. */
 	DIRECTORY_COLOR,    /* Directory. */
 	LINK_COLOR,         /* Symbolic link. */
 	BROKEN_LINK_COLOR,  /* Dangling symbolic link. */
@@ -46,25 +46,15 @@ enum
 	TOP_LINE_COLOR,     /* Top line of the other pane. */
 	TOP_LINE_SEL_COLOR, /* Top line of the selected pane. */
 	STATUS_LINE_COLOR,  /* Status line. */
-	MENU_COLOR,         /* Menu background. */
+	WILD_MENU_COLOR,    /* Wild menu. */
 	CMD_LINE_COLOR,     /* Command line. */
 	ERROR_MSG_COLOR,    /* Error of the command line. */
 	BORDER_COLOR,       /* Vertical border lines. */
 	OTHER_LINE_COLOR,   /* Line under the cursor in the other pane. */
-	CURRENT_COLOR,      /* Pane current line (mixed, internal use only). */
-	MENU_CURRENT_COLOR, /* Menu current line (mixed, internal use only). */
+	JOB_LINE_COLOR,     /* Line that displays status of background jobs. */
 	MAXNUM_COLOR        /* Number of elements of a color scheme. */
 };
 
-/* Start indices of different parts of color pairs space. */
-enum
-{
-	DCOLOR_BASE = 1,                          /* Default color scheme. */
-	LCOLOR_BASE = DCOLOR_BASE + MAXNUM_COLOR, /* Left pane's color scheme. */
-	RCOLOR_BASE = LCOLOR_BASE + MAXNUM_COLOR, /* Right pane's color scheme. */
-	FCOLOR_BASE = RCOLOR_BASE + MAXNUM_COLOR  /* Free. */
-};
-
 #endif /* VIFM__COLORS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
