@@ -60,9 +60,10 @@ typedef struct columns_list_t *columns_t;
 /* A column callback function, which sould fill the buf with column text. */
 typedef void (*column_func)(int id, const void *data, size_t buf_len,
 		char buf[]);
+
 /* A callback function, for displaying column contents. */
 typedef void (*column_line_print_func)(const void *data, int column_id,
-		const char buf[], size_t offset);
+		const char buf[], size_t offset, AlignType align, const char full_column[]);
 
 /* Structure containing various column display properties. */
 typedef struct
