@@ -28,19 +28,22 @@
 
 #include <sys/stat.h> /* stat */
 #include <dirent.h> /* DIR dirent */
-#include <unistd.h> /* X_OK access() */
 
 #ifndef _WIN32
 #include <grp.h> /* getgrent setgrent */
 #include <pwd.h> /* getpwent setpwent */
 #endif
 
+#include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t */
 #include <stdlib.h> /* free() */
 #include <stdio.h> /* snprintf() */
 #include <string.h> /* strdup() strlen() strncasecmp() strncmp() strrchr() */
 
 #include "cfg/config.h"
+#include "compat/os.h"
+#include "engine/abbrevs.h"
+#include "engine/cmds.h"
 #include "engine/completion.h"
 #include "engine/functions.h"
 #include "engine/options.h"
@@ -48,6 +51,8 @@
 #ifdef _WIN32
 #include "menus/menus.h"
 #endif
+#include "modes/dialogs/msg_dialog.h"
+#include "ui/statusbar.h"
 #include "utils/fs.h"
 #include "utils/fs_limits.h"
 #include "utils/macros.h"
@@ -65,48 +70,55 @@
 
 static int cmd_ends_with_space(const char *cmd);
 static void complete_colorscheme(const char *str, size_t arg_num);
+static void complete_selective_sync(const char str[]);
 static void complete_help(const char *str);
 static void complete_history(const char str[]);
 static void complete_invert(const char str[]);
-static void complete_from_string_list(const char str[], const char *list[],
-		size_t list_len);
 static int complete_chown(const char *str);
 static void complete_filetype(const char *str);
 static void complete_progs(const char *str, assoc_records_t records);
 static void complete_highlight_groups(const char *str);
 static int complete_highlight_arg(const char *str);
 static void complete_envvar(const char str[]);
-static void complete_winrun(const char *str);
+static void complete_winrun(const char str[]);
+static void complete_from_string_list(const char str[], const char *list[],
+		size_t list_len);
 static void complete_command_name(const char beginning[]);
 static void filename_completion_in_dir(const char *path, const char *str,
 		CompletionType type);
-static void filename_completion_internal(DIR * dir, const char * dirname,
-		const char * filename, CompletionType type);
+static void filename_completion_internal(DIR *dir, const char dirname[],
+		const char filename[], CompletionType type);
 static int is_dirent_targets_exec(const struct dirent *d);
 #ifdef _WIN32
 static void complete_with_shared(const char *server, const char *file);
 #endif
 
 int
-complete_args(int id, const char args[], int argc, char *argv[], int arg_pos,
-		void *extra_arg)
+complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
 {
 	/* TODO: Refactor this function complete_args() */
 
-	const char *arg;
-	const char *start;
-	const char *slash;
-	const char *dollar;
-	const char *ampersand;
+	const CompletionPreProcessing cpp = (CompletionPreProcessing)extra_arg;
 
-	arg = after_last(args, ' ');
-	start = arg;
-	slash = strrchr(args + arg_pos, '/');
-	dollar = strrchr(arg, '$');
-	ampersand = strrchr(arg, '&');
+	const char *const args = cmd_info->args;
+	int argc = cmd_info->argc;
+	char **const argv = cmd_info->argv;
+
+	const char *arg = after_last(args, ' ');
+	const char *start = arg;
+	const char *slash = strrchr(args + arg_pos, '/');
+	const char *dollar = strrchr(arg, '$');
+	const char *ampersand = strrchr(arg, '&');
 
 	if(id == COM_SET)
+	{
 		complete_options(args, &start);
+	}
+	else if(id == COM_CABBR)
+	{
+		vle_abbr_complete(args);
+		start = args;
+	}
 	else if(command_accepts_expr(id))
 	{
 		if(ampersand > dollar)
@@ -185,35 +197,46 @@ complete_args(int id, const char args[], int argc, char *argv[], int arg_pos,
 			arg = argv[arg_num];
 		}
 
-		switch((CompletionPreProcessing)extra_arg)
+		/* Pre-process input with requested method. */
+		if(cpp != CPP_NONE)
 		{
-			case CPP_NONE:
-				/* Do nothing. */
-				break;
-			case CPP_SQUOTES_UNESCAPE:
+			if(cpp != CPP_PERCENT_UNESCAPE)
+			{
 				arg = args + arg_pos + 1;
 				start = (slash == NULL) ? arg : (slash + 1);
+			}
 
-				free_me = strdup(arg);
-				expand_squotes_escaping(free_me);
-				arg = free_me;
-				break;
-			case CPP_DQUOTES_UNESCAPE:
-				arg = args + arg_pos + 1;
-				start = (slash == NULL) ? arg : (slash + 1);
+			free_me = strdup(arg);
+			arg = free_me;
 
-				free_me = strdup(arg);
-				expand_dquotes_escaping(free_me);
-				arg = free_me;
-				break;
-		};
+			switch(cpp)
+			{
+				case CPP_PERCENT_UNESCAPE: expand_percent_escaping(free_me); break;
+				case CPP_SQUOTES_UNESCAPE: expand_squotes_escaping(free_me); break;
+				case CPP_DQUOTES_UNESCAPE: expand_dquotes_escaping(free_me); break;
+
+				default:
+					assert(0 && "Unhandled preprocessing type.");
+					break;
+			};
+		}
 
 		if(id == COM_COLORSCHEME)
 		{
 			complete_colorscheme(arg, arg_num);
 		}
-		else if(id == COM_CD || id == COM_PUSHD || id == COM_SYNC ||
-				id == COM_MKDIR)
+		else if(id == COM_SYNC)
+		{
+			if(cmd_info->emark)
+			{
+				complete_selective_sync(arg);
+			}
+			else
+			{
+				filename_completion(arg, CT_DIRONLY);
+			}
+		}
+		else if(id == COM_CD || id == COM_PUSHD || id == COM_MKDIR)
 		{
 			filename_completion(arg, CT_DIRONLY);
 		}
@@ -224,7 +247,7 @@ complete_args(int id, const char args[], int argc, char *argv[], int arg_pos,
 		}
 		else if(id == COM_SPLIT || id == COM_VSPLIT)
 		{
-			filename_completion_in_dir(curr_view->curr_dir, arg, CT_DIRONLY);
+			filename_completion_in_dir(flist_get_dir(curr_view), arg, CT_DIRONLY);
 		}
 		else if(id == COM_FIND)
 		{
@@ -235,7 +258,7 @@ complete_args(int id, const char args[], int argc, char *argv[], int arg_pos,
 		{
 			if(argc == 0 || (argc == 1 && !cmd_ends_with_space(args)))
 			{
-				if(*arg == '.')
+				if(*arg == '.' || is_path_absolute(arg))
 					filename_completion(arg, CT_DIREXEC);
 				else
 					complete_command_name(arg);
@@ -283,6 +306,21 @@ complete_colorscheme(const char *str, size_t arg_num)
 	}
 }
 
+/* Completes properties for selective synchronization. */
+static void
+complete_selective_sync(const char str[])
+{
+	static const char *lines[] = {
+		"location",
+		"cursorpos",
+		"localopts",
+		"filters",
+		"all",
+	};
+
+	complete_from_string_list(str, lines, ARRAY_LEN(lines));
+}
+
 static void
 complete_help(const char *str)
 {
@@ -304,57 +342,29 @@ complete_help(const char *str)
 	vle_compl_add_last_match(str);
 }
 
+/* Completes available kinds of histories. */
 static void
 complete_history(const char str[])
 {
-	static const char *lines[] =
-	{
-		".",
-		"dir",
-		"@",
-		"input",
-		"/",
-		"search",
-		"fsearch",
-		"?",
-		"bsearch",
-		":",
-		"cmd",
-		"=",
-		"filter",
+	static const char *lines[] = {
+		".", "dir",
+		"@", "input",
+		"/", "search", "fsearch",
+		"?", "bsearch",
+		":", "cmd",
+		"=", "filter",
 	};
 	complete_from_string_list(str, lines, ARRAY_LEN(lines));
 }
 
+/* Completes available inversion kinds. */
 static void
 complete_invert(const char str[])
 {
-	static const char *lines[] =
-	{
-		"f",
-		"s",
-		"o",
-	};
+	static const char *lines[] = {"f", "s", "o"};
 	complete_from_string_list(str, lines, ARRAY_LEN(lines));
 }
 
-/* Performs str completion using items in the list of length list_len. */
-static void
-complete_from_string_list(const char str[], const char *list[], size_t list_len)
-{
-	int i;
-	const size_t len = strlen(str);
-	for(i = 0; i < list_len; i++)
-	{
-		if(strncmp(str, list[i], len) == 0)
-		{
-			vle_compl_add_match(list[i]);
-		}
-	}
-	vle_compl_finish_group();
-	vle_compl_add_last_match(str);
-}
-
 static int
 complete_chown(const char *str)
 {
@@ -379,16 +389,18 @@ complete_chown(const char *str)
 static void
 complete_filetype(const char *str)
 {
-	const char *filename = get_current_file_name(curr_view);
-	assoc_records_t ft = get_all_programs_for_file(filename);
+	char *const typed_fname = get_typed_current_fpath(curr_view);
+	assoc_records_t ft = ft_get_all_programs(typed_fname);
 
 	complete_progs(str, ft);
-	free(ft.list);
+	ft_assoc_records_free(&ft);
 
-	complete_progs(str, get_magic_handlers(filename));
+	complete_progs(str, get_magic_handlers(typed_fname));
 
 	vle_compl_finish_group();
 	vle_compl_add_last_match(str);
+
+	free(typed_fname);
 }
 
 static void
@@ -406,25 +418,28 @@ complete_progs(const char *str, assoc_records_t records)
 
 		if(strnoscmp(command, str, len) == 0)
 		{
-			char *const escaped = escape_chars(command, "|");
-			vle_compl_add_match(escaped);
-			free(escaped);
+			vle_compl_put_match(escape_chars(command, "|"));
 		}
 	}
 }
 
+/* Completes highlight groups and subcomand "clear" for :highlight command. */
 static void
 complete_highlight_groups(const char *str)
 {
 	int i;
 	const size_t len = strlen(str);
-	for(i = 0; i < MAXNUM_COLOR - 2; i++)
+	for(i = 0; i < MAXNUM_COLOR; ++i)
 	{
 		if(strncasecmp(str, HI_GROUPS[i], len) == 0)
 		{
 			vle_compl_add_match(HI_GROUPS[i]);
 		}
 	}
+	if(strncmp(str, "clear", len) == 0)
+	{
+		vle_compl_add_match("clear");
+	}
 	vle_compl_finish_group();
 	vle_compl_add_last_match(str);
 }
@@ -433,7 +448,7 @@ static int
 complete_highlight_arg(const char *str)
 {
 	/* TODO: Refactor this function complete_highlight_arg() */
-	char *equal = strchr(str, '=');
+	const char *equal = strchr(str, '=');
 	int result = (equal == NULL) ? 0 : (equal - str + 1);
 	size_t len = strlen((equal == NULL) ? str : ++equal);
 	if(equal == NULL)
@@ -444,9 +459,9 @@ complete_highlight_arg(const char *str)
 			"ctermbg",
 		};
 
-		int i;
+		size_t i;
 
-		for(i = 0; i < ARRAY_LEN(args); i++)
+		for(i = 0U; i < ARRAY_LEN(args); ++i)
 		{
 			if(strncmp(str, args[i], len) == 0)
 			{
@@ -467,9 +482,9 @@ complete_highlight_arg(const char *str)
 				"none",
 			};
 
-			int i;
+			size_t i;
 
-			char *comma = strrchr(equal, ',');
+			const char *const comma = strrchr(equal, ',');
 			if(comma != NULL)
 			{
 				result += comma - equal + 1;
@@ -477,7 +492,7 @@ complete_highlight_arg(const char *str)
 				len = strlen(equal);
 			}
 
-			for(i = 0; i < ARRAY_LEN(STYLES); i++)
+			for(i = 0U; i < ARRAY_LEN(STYLES); ++i)
 			{
 				if(strncasecmp(equal, STYLES[i], len) == 0)
 				{
@@ -487,7 +502,7 @@ complete_highlight_arg(const char *str)
 		}
 		else
 		{
-			int i;
+			size_t i;
 
 			if(strncasecmp(equal, "default", len) == 0)
 			{
@@ -498,14 +513,14 @@ complete_highlight_arg(const char *str)
 				vle_compl_add_match("none");
 			}
 
-			for(i = 0; i < ARRAY_LEN(XTERM256_COLOR_NAMES); i++)
+			for(i = 0U; i < ARRAY_LEN(XTERM256_COLOR_NAMES); ++i)
 			{
 				if(strncasecmp(equal, XTERM256_COLOR_NAMES[i], len) == 0)
 				{
 					vle_compl_add_match(XTERM256_COLOR_NAMES[i]);
 				}
 			}
-			for(i = 0; i < ARRAY_LEN(LIGHT_COLOR_NAMES); i++)
+			for(i = 0U; i < ARRAY_LEN(LIGHT_COLOR_NAMES); ++i)
 			{
 				if(strncasecmp(equal, LIGHT_COLOR_NAMES[i], len) == 0)
 				{
@@ -548,18 +563,25 @@ complete_envvar(const char str[])
 	vle_compl_add_last_match(str);
 }
 
+/* Completes first :winrun argument. */
 static void
-complete_winrun(const char *str)
+complete_winrun(const char str[])
 {
-	static const char *VARIANTS[] = { "^", "$", "%", ".", "," };
-	const size_t len = strlen(str);
-	int i;
+	static const char *win_marks[] = { "^", "$", "%", ".", "," };
+	complete_from_string_list(str, win_marks, ARRAY_LEN(win_marks));
+}
 
-	for(i = 0; i < ARRAY_LEN(VARIANTS); i++)
+/* Performs str completion using items in the list of length list_len. */
+static void
+complete_from_string_list(const char str[], const char *list[], size_t list_len)
+{
+	size_t i;
+	const size_t len = strlen(str);
+	for(i = 0; i < list_len; ++i)
 	{
-		if(strncmp(str, VARIANTS[i], len) == 0)
+		if(strncmp(str, list[i], len) == 0)
 		{
-			vle_compl_add_match(VARIANTS[i]);
+			vle_compl_add_match(list[i]);
 		}
 	}
 	vle_compl_finish_group();
@@ -623,12 +645,13 @@ fast_run_complete(const char cmd[])
 static void
 complete_command_name(const char beginning[])
 {
-	int i;
+	size_t i;
 	char ** paths;
 	size_t paths_count;
+	char *const cwd = get_cwd();
 
 	paths = get_paths(&paths_count);
-	for(i = 0; i < paths_count; i++)
+	for(i = 0U; i < paths_count; ++i)
 	{
 		if(vifm_chdir(paths[i]) == 0)
 		{
@@ -636,6 +659,8 @@ complete_command_name(const char beginning[])
 		}
 	}
 	vle_compl_add_last_path_match(beginning);
+
+	restore_cwd(cwd);
 }
 
 static void
@@ -661,16 +686,15 @@ void
 filename_completion(const char *str, CompletionType type)
 {
 	/* TODO refactor filename_completion(...) function */
-	DIR * dir;
-	char * dirname;
-	char * filename;
-	char * temp;
+	DIR *dir;
+	char *dirname;
+	char *filename;
+	char *temp;
+	char *cwd;
 
 	if(str[0] == '~' && strchr(str, '/') == NULL)
 	{
-		char *const tilde_expanded = expand_tilde(str);
-		vle_compl_add_path_match(tilde_expanded);
-		free(tilde_expanded);
+		vle_compl_put_path_match(expand_tilde(str));
 		return;
 	}
 
@@ -726,7 +750,9 @@ filename_completion(const char *str, CompletionType type)
 	}
 #endif
 
-	dir = opendir(dirname);
+	dir = os_opendir(dirname);
+
+	cwd = get_cwd();
 
 	if(dir == NULL || vifm_chdir(dirname) != 0)
 	{
@@ -735,7 +761,7 @@ filename_completion(const char *str, CompletionType type)
 	else
 	{
 		filename_completion_internal(dir, dirname, filename, type);
-		(void)vifm_chdir(curr_view->curr_dir);
+		(void)vifm_chdir(flist_get_dir(curr_view));
 	}
 
 	free(filename);
@@ -743,18 +769,20 @@ filename_completion(const char *str, CompletionType type)
 
 	if(dir != NULL)
 	{
-		closedir(dir);
+		os_closedir(dir);
 	}
+
+	restore_cwd(cwd);
 }
 
 static void
-filename_completion_internal(DIR * dir, const char * dirname,
-		const char * filename, CompletionType type)
+filename_completion_internal(DIR *dir, const char dirname[],
+		const char filename[], CompletionType type)
 {
 	struct dirent *d;
 
 	size_t filename_len = strlen(filename);
-	while((d = readdir(dir)) != NULL)
+	while((d = os_readdir(dir)) != NULL)
 	{
 		if(filename[0] == '\0' && d->d_name[0] == '.')
 			continue;
@@ -771,9 +799,7 @@ filename_completion_internal(DIR * dir, const char * dirname,
 
 		if(is_dirent_targets_dir(d) && type != CT_ALL_WOS)
 		{
-			char with_slash[strlen(d->d_name) + 1 + 1];
-			snprintf(with_slash, sizeof(with_slash), "%s/", d->d_name);
-			vle_compl_add_path_match(with_slash);
+			vle_compl_put_path_match(format_str("%s/", d->d_name));
 		}
 		else
 		{
@@ -798,7 +824,7 @@ is_dirent_targets_exec(const struct dirent *d)
 		return 0;
 	if(d->d_type == DT_LNK && get_symlink_type(d->d_name) != SLT_UNKNOWN)
 		return 0;
-	return access(d->d_name, X_OK) == 0;
+	return os_access(d->d_name, X_OK) == 0;
 #else
 	return is_win_executable(d->d_name);
 #endif
@@ -879,9 +905,7 @@ complete_with_shared(const char *server, const char *file)
 				strcat(buf, "/");
 				if(strnoscmp(buf, file, len) == 0)
 				{
-					char *const escaped = escape_filename(buf, 1);
-					vle_compl_add_match(escaped);
-					free(escaped);
+					vle_compl_put_match(escape_filename(buf, 1));
 				}
 				p++;
 			}
