@@ -20,14 +20,12 @@
 
 #if defined(ENABLE_DESKTOP_FILES)
 
-#ifndef _WIN32
-#include <sys/dir.h>
-#endif
-#include <dirent.h> /* DIR */
+#include <dirent.h>
 
 #include <stdio.h> /* snprintf() */
 #include <string.h> /* strstr() strchr() strlen() strcpy() */
 
+#include "compat/os.h"
 #include "utils/fs_limits.h"
 #include "utils/path.h"
 #include "utils/str.h"
@@ -63,14 +61,14 @@ parse_desktop_files_internal(const char *path, const char *mime_type,
 	struct dirent *dentry;
 	const char *slash;
 
-	if((dir = opendir(path)) == NULL)
+	if((dir = os_opendir(path)) == NULL)
 	{
 		return;
 	}
 
 	slash = ends_with_slash(path) ? "" : "/";
 
-	while((dentry = readdir(dir)) != NULL)
+	while((dentry = os_readdir(dir)) != NULL)
 	{
 		char buf[PATH_MAX];
 
@@ -90,7 +88,7 @@ parse_desktop_files_internal(const char *path, const char *mime_type,
 		}
 	}
 
-	closedir(dir);
+	os_closedir(dir);
 }
 
 static void
@@ -101,7 +99,7 @@ process_file(const char *path, const char *file_mime_type,
 	char exec[1024] = "", mime_type[2048] = "", name[2048] = "";
 	char buf[2048];
 
-	if(!ends_with(path, ".desktop") || (f = fopen(path, "r")) == NULL)
+	if(!ends_with(path, ".desktop") || (f = os_fopen(path, "r")) == NULL)
 	{
 		return;
 	}
@@ -133,7 +131,7 @@ process_file(const char *path, const char *file_mime_type,
 	}
 
 	expand_desktop(exec, buf);
-	add_assoc_record(result, buf, name);
+	ft_assoc_record_add(result, buf, name);
 }
 
 static void
