@@ -106,8 +106,8 @@ static cmd_t * insert_cmd(cmd_t *after);
 static int delcommand_cmd(const cmd_info_t *cmd_info);
 TSTATIC char ** dispatch_line(const char args[], int *count, char sep,
 		int regexp, int quotes, int *last_arg, int *last_begin, int *last_end);
-static int is_separator(char c, char sep);
 TSTATIC void unescape(char s[], int regexp);
+static int is_separator(char c, char sep);
 
 void
 init_cmds(int udf, cmds_conf_t *conf)
@@ -224,7 +224,7 @@ execute_cmd(const char cmd[])
 	/* Set background flag and remove background mark from raw arguments, when
 	 * command supports backgrounding. */
 	last_arg = get_last_argument(cmd_info.raw_args, &last_arg_len);
-	if(cur->bg && *last_arg == '&' && *skip_whitespace(last_arg + 1) == '\0')
+	if(cur->bg && *last_arg == '&' && *vle_cmds_at_arg(last_arg + 1) == '\0')
 	{
 		cmd_info.bg = 1;
 		*last_arg = '\0';
@@ -290,7 +290,7 @@ execute_cmd(const char cmd[])
 	{
 		cur->passed++;
 
-		if(cur->type != BUILTIN_CMD && cur->type != BUILTIN_ABBR)
+		if(cur->type == USER_CMD)
 		{
 			cmd_info.cmd = cur->cmd;
 			execution_code = inner->user_cmd_handler.handler(&cmd_info);
@@ -569,7 +569,7 @@ parse_range(const char cmd[], cmd_info_t *cmd_info)
 {
 	char last_sep;
 
-	cmd = skip_whitespace(cmd);
+	cmd = vle_cmds_at_arg(cmd);
 
 	if(isalpha(*cmd) || *cmd == '!' || *cmd == '\0')
 	{
@@ -590,9 +590,11 @@ parse_range(const char cmd[], cmd_info_t *cmd_info)
 		cmd = correct_limit(cmd, cmd_info);
 
 		if(cmd_info->begin == NOT_DEF)
+		{
 			cmd_info->begin = cmd_info->end;
+		}
 
-		cmd = skip_whitespace(cmd);
+		cmd = vle_cmds_at_arg(cmd);
 
 		if(!char_is_one_of(RANGE_SEPARATORS, *cmd))
 		{
@@ -602,7 +604,7 @@ parse_range(const char cmd[], cmd_info_t *cmd_info)
 		last_sep = *cmd;
 		cmd++;
 
-		cmd = skip_whitespace(cmd);
+		cmd = vle_cmds_at_arg(cmd);
 	}
 
 	return cmd;
@@ -686,7 +688,7 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 	if(cmd[0] == '!')
 	{
 		strcpy(buf, "!");
-		cmd = skip_whitespace(cmd + 1);
+		cmd = vle_cmds_at_arg(cmd + 1);
 		return cmd;
 	}
 
@@ -694,7 +696,7 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 	while(isalpha(*t))
 		t++;
 
-	len = MIN(t - cmd, buf_len - 1);
+	len = MIN((size_t)(t - cmd), buf_len - 1);
 	strncpy(buf, cmd, len);
 	buf[len] = '\0';
 	if(*t == '?' || *t == '!')
@@ -705,16 +707,30 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 		cur = inner->head.next;
 		while(cur != NULL && (cmp = strncmp(cur->name, buf, len)) <= 0)
 		{
-			if(cmp == 0 && cur->type == USER_CMD &&
-					cur->name[strlen(cur->name) - 1] == *t)
+			if(cmp == 0)
 			{
-				strncpy(buf, cur->name, buf_len);
-				break;
+				/* Check for user-defined command that ends with the char. */
+				if(cur->type == USER_CMD && cur->name[strlen(cur->name) - 1] == *t)
+				{
+					strncpy(buf, cur->name, buf_len);
+					break;
+				}
+				/* Or builtin abbreviation that supports the mark. */
+				if(cur->type == BUILTIN_ABBR &&
+						((*t == '!' && cur->emark) || (*t == '?' && cur->qmark)))
+				{
+					strncpy(buf, cur->name, buf_len);
+					break;
+				}
 			}
 			cur = cur->next;
 		}
-		if(cur != NULL && strncmp(cur->name, buf, len) == 0)
-			t++;
+		/* For builtin commands, the char is not part of the name. */
+		if(cur != NULL && cur->type == USER_CMD &&
+				strncmp(cur->name, buf, len) == 0)
+		{
+			++t;
+		}
 	}
 
 	return t;
@@ -732,7 +748,7 @@ complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info,
 		return 0;
 
 	args = parse_tail(cur, tmp_args, cmd_info);
-	args = skip_whitespace(args);
+	args = vle_cmds_at_arg(args);
 	result += args - tmp_args;
 
 	if(cur->id == COMMAND_CMD_ID || cur->id == DELCOMMAND_CMD_ID)
@@ -752,8 +768,12 @@ complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info,
 		int last_arg = 0;
 
 		argv = dispatch_line(args, &argc, ' ', 0, 1, &last_arg, NULL, NULL);
-		result += cmds_conf->complete_args(cur->id, args, argc, argv, last_arg,
-				arg);
+
+		cmd_info->args = (char *)args;
+		cmd_info->argc = argc;
+		cmd_info->argv = argv;
+		result += cmds_conf->complete_args(cur->id, cmd_info, last_arg, arg);
+
 		free_string_array(argv, argc);
 	}
 	return result;
@@ -821,7 +841,6 @@ add_builtin_commands(const cmd_add_t *cmds, int count)
 TSTATIC int
 add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 {
-	int i;
 	int cmp;
 	cmd_t *new;
 	cmd_t *cur = &inner->head;
@@ -835,15 +854,24 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 	}
 
 	if(strcmp(name, "!") != 0)
-		for(i = 0; name[i] != '\0'; i++)
+	{
+		unsigned int i;
+		for(i = 0U; name[i] != '\0'; ++i)
+		{
 			if(!isalpha(name[i]))
+			{
 				return -1;
+			}
+		}
+	}
 
 	cmp = -1;
 	while(cur->next != NULL && (cmp = strcmp(cur->next->name, name)) < 0)
+	{
 		cur = cur->next;
+	}
 
-	/* command with the same name already exists */
+	/* Command with the same name already exists. */
 	if(cmp == 0)
 	{
 		if(strncmp(name, "command", strlen(name)) == 0)
@@ -909,6 +937,8 @@ command_cmd(const cmd_info_t *cmd_info)
 	char cmd_name[MAX_CMD_NAME_LEN];
 	const char *args;
 	cmd_t *new, *cur;
+	size_t len;
+	int has_emark, has_qmark;
 
 	if(cmd_info->argc < 2)
 	{
@@ -919,16 +949,34 @@ command_cmd(const cmd_info_t *cmd_info)
 	}
 
 	args = get_user_cmd_name(cmd_info->args, cmd_name, sizeof(cmd_name));
-	args = skip_whitespace(args);
+	args = vle_cmds_at_arg(args);
 	if(args[0] == '\0')
 		return CMDS_ERR_TOO_FEW_ARGS;
 	else if(!is_correct_name(cmd_name))
 		return CMDS_ERR_INCORRECT_NAME;
 
+	len = strlen(cmd_name);
+	has_emark = (len > 0 && cmd_name[len - 1] == '!');
+	has_qmark = (len > 0 && cmd_name[len - 1] == '?');
+
 	cmp = -1;
 	cur = &inner->head;
 	while(cur->next != NULL && (cmp = strcmp(cur->next->name, cmd_name)) < 0)
+	{
+		if(has_emark && cur->next->type == BUILTIN_CMD && cur->next->emark &&
+				strncmp(cmd_name, cur->next->name, len - 1) == 0)
+		{
+			cmp = 0;
+			break;
+		}
+		if(has_qmark && cur->next->type == BUILTIN_CMD && cur->next->qmark &&
+				strncmp(cmd_name, cur->next->name, len - 1) == 0)
+		{
+			cmp = 0;
+			break;
+		}
 		cur = cur->next;
+	}
 
 	if(cmp == 0)
 	{
@@ -976,9 +1024,9 @@ get_user_cmd_name(const char cmd[], char buf[], size_t buf_len)
 	const char *t;
 	size_t len;
 
-	t = skip_non_whitespace(cmd);
+	t = vle_cmds_past_arg(cmd);
 
-	len = MIN(t - cmd, buf_len);
+	len = MIN((size_t)(t - cmd), buf_len);
 	strncpy(buf, cmd, len);
 	buf[len] = '\0';
 	return t;
@@ -1088,7 +1136,7 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 	args_beg = args;
 	if(sep == ' ')
 	{
-		args = skip_whitespace(args);
+		args = vle_cmds_at_arg(args);
 	}
 	cmdstr = strdup(args);
 	len = strlen(cmdstr);
@@ -1117,6 +1165,13 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 				{
 					st = i;
 					state = NO_QUOTING;
+
+					/* Skip escaped character.  If not and it's a separator, beginning of
+					 * the argument will be skipped. */
+					if(cmdstr[i] == '\\' && cmdstr[i + 1] != '\0')
+					{
+						++i;
+					}
 				}
 				else if(sep != ' ' && i > 0 && is_separator(cmdstr[i - 1], sep))
 				{
@@ -1220,16 +1275,6 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 	return params;
 }
 
-/* Checks whether character is command separator.  Special case is space as a
- * separator, which means that any whitespace can be used as separators.
- * Returns non-zero if c is counted to be a separator, otherwise zero is
- * returned. */
-static int
-is_separator(char c, char sep)
-{
-	return (sep == ' ') ? isspace(c) : (c == sep);
-}
-
 TSTATIC void
 unescape(char s[], int regexp)
 {
@@ -1317,5 +1362,43 @@ list_udf_content(const char beginning[])
 	return content;
 }
 
+char *
+vle_cmds_past_arg(const char args[])
+{
+	while(!is_separator(*args, ' ') && *args != '\0')
+	{
+		++args;
+	}
+	return (char *)args;
+}
+
+char *
+vle_cmds_at_arg(const char args[])
+{
+	while(is_separator(*args, ' '))
+	{
+		++args;
+	}
+	return (char *)args;
+}
+
+char *
+vle_cmds_next_arg(const char args[])
+{
+	args = vle_cmds_past_arg(args);
+	return vle_cmds_at_arg(args);
+}
+
+/* Checks whether character is command separator.  Special case is space as a
+ * separator, which means that any reasonable whitespace can be used as
+ * separators (we don't won't to treat line feeds or similar characters as
+ * separators to allow their appearance as part of arguments).  Returns non-zero
+ * if c is counted to be a separator, otherwise zero is returned. */
+static int
+is_separator(char c, char sep)
+{
+	return (sep == ' ') ? (c == ' ' || c == '\t') : (c == sep);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
