@@ -58,7 +58,7 @@ enum
 
 /* Detailed information resulted from command parsing, which is passed to
  * command handler (cmd_handler). */
-typedef struct
+typedef struct cmd_info_t
 {
 	int begin, end; /* Parsed range of the command. */
 	int emark, qmark, bg;
@@ -103,9 +103,9 @@ typedef struct
 	int end;     /* The highest valid number of the range. */
 
 	/* Argument completion function.  arg is user supplied value, which is passed
-	 * through. */
-	int (*complete_args)(int id, const char args[], int argc, char *argv[],
-			int arg_pos, void *arg);
+	 * through.  The functions must not modify any strings passed to it. */
+	int (*complete_args)(int id, const cmd_info_t *cmd_info, int arg_pos,
+			void *arg);
 
 	int (*swap_range)(void);
 	int (*resolve_mark)(char mark); /* should return value < 0 on error */
@@ -147,6 +147,20 @@ char ** list_udf(void);
 
 char * list_udf_content(const char beginning[]);
 
+/* Skips at most one argument of the string.  Returns pointer to the next
+ * character after that argument, if any, otherwise pointer to
+ * null-character. */
+char * vle_cmds_past_arg(const char args[]);
+
+/* Skips argument separators in the string.  Returns pointer to the first
+ * character of the first argument, if any, otherwise pointer to
+ * null-character. */
+char * vle_cmds_at_arg(const char args[]);
+
+/* Advances to the next argument in the string.  Returns pointer to the next
+ * argument, if any, otherwise pointer to null-character. */
+char * vle_cmds_next_arg(const char args[]);
+
 TSTATIC_DEFS(
 	int add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf);
 	char ** dispatch_line(const char args[], int *count, char sep, int regexp,
