@@ -38,8 +38,8 @@ typedef struct key_chunk_t
 	wchar_t key;
 	int no_remap;
 	size_t children_count;
-	int enters; /* to prevent stack overflow */
-	int deleted; /* postpone free() call for proper lazy deletion */
+	int enters;            /* To prevent stack overflow and manager lifetime. */
+	int deleted;           /* Postpone free() call for proper lazy deletion. */
 	key_conf_t conf;
 	struct key_chunk_t *child;
 	struct key_chunk_t *parent;
@@ -76,6 +76,8 @@ static int execute_next_keys(key_chunk_t *curr, const wchar_t keys[],
 		int no_remap);
 static int dispatch_key(key_info_t key_info, keys_info_t *keys_info,
 		key_chunk_t *curr, const wchar_t keys[]);
+static int has_def_handler(void);
+static default_handler def_handler(void);
 static int execute_after_remapping(const wchar_t rhs[],
 		const wchar_t left_keys[], keys_info_t keys_info, key_info_t key_info,
 		key_chunk_t *curr);
@@ -513,18 +515,19 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 	}
 	else
 	{
-		const default_handler def_handler = def_handlers[vle_mode_get()];
+		int result = has_def_handler() ? 0 : KEYS_UNKNOWN;
 
-		int result = (def_handler == NULL) ? KEYS_UNKNOWN : 0;
+		/* Protect chunk from deletion while it's in use. */
+		enter_chunk(curr);
 
-		if(curr->enters == 0)
+		if(curr->enters == 1)
 		{
 			result = execute_after_remapping(conf->data.cmd, keys, *keys_info,
 					key_info, curr);
 		}
-		else if(def_handler != NULL)
+		else if(has_def_handler())
 		{
-			result = def_handler(curr->key);
+			result = def_handler()(curr->key);
 
 			if(result == 0)
 			{
@@ -533,12 +536,11 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 			}
 		}
 
-		if(result == KEYS_UNKNOWN && def_handler != NULL)
+		if(result == KEYS_UNKNOWN && has_def_handler())
 		{
-			/* curr shouldn't be freed here as if it was result would be 0. */
-			if(curr->enters == 0)
+			if(curr->enters == 1)
 			{
-				result = def_handler(conf->data.cmd[0]);
+				result = def_handler()(conf->data.cmd[0]);
 				enter_chunk(curr);
 				execute_keys_general(conf->data.cmd + 1, 0, 1, curr->no_remap);
 				leave_chunk(curr);
@@ -548,15 +550,34 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 				int i;
 				for(i = 0; conf->data.cmd[i] != '\0'; i++)
 				{
-					result = def_handler(conf->data.cmd[i]);
+					result = def_handler()(conf->data.cmd[i]);
 				}
 			}
 		}
 
+		/* Release the chunk, this will free it if deletion was attempted. */
+		leave_chunk(curr);
+
 		return result;
 	}
 }
 
+/* Checks that default handler exists for active mode.  Returns non-zero if so,
+ * otherwise zero is returned. */
+static int
+has_def_handler(void)
+{
+	return (def_handler() != NULL);
+}
+
+/* Gets default handler of active mode.  Returns the handler, which is NULL when
+ * not set for active mode. */
+static default_handler
+def_handler(void)
+{
+	return def_handlers[vle_mode_get()];
+}
+
 /* Processes remapping of a key.  Returns error code. */
 static int
 execute_after_remapping(const wchar_t rhs[], const wchar_t left_keys[],
@@ -610,7 +631,7 @@ execute_after_remapping(const wchar_t rhs[], const wchar_t left_keys[],
 static void
 enter_chunk(key_chunk_t *chunk)
 {
-	chunk->enters = 1;
+	++chunk->enters;
 }
 
 /* Handles leaving a chunk performing postponed chunk removal if needed.
@@ -618,15 +639,14 @@ enter_chunk(key_chunk_t *chunk)
 static void
 leave_chunk(key_chunk_t *chunk)
 {
-	if(chunk->deleted)
+	--chunk->enters;
+
+	if(chunk->enters == 0 && chunk->deleted)
 	{
 		/* Removal of the chunk was postponed because it was in use, proceed with
 		 * this now. */
 		free(chunk);
-		return;
 	}
-
-	chunk->enters = 0;
 }
 
 static void
