@@ -21,7 +21,7 @@
 static vle_mode_t current_mode;
 static vle_mode_t primary_mode;
 
-const vle_mode_t
+vle_mode_t
 vle_mode_get(void)
 {
 	return current_mode;
@@ -33,12 +33,18 @@ vle_mode_is(vle_mode_t mode)
 	return current_mode == mode;
 }
 
-const vle_mode_t
+vle_mode_t
 vle_mode_get_primary(void)
 {
 	return primary_mode;
 }
 
+int
+vle_primary_mode_is(vle_mode_t mode)
+{
+	return primary_mode == mode;
+}
+
 void
 vle_mode_set(vle_mode_t mode, VleModeType type)
 {
