@@ -24,7 +24,7 @@
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* memset() strcmp() strcpy() strlen() */
+#include <string.h> /* memset() strchr() strcmp() strcpy() strlen() */
 
 #include "../utils/str.h"
 #include "../utils/string_array.h"
@@ -56,8 +56,10 @@ typedef void (*mod_t)(char buffer[], char value);
 
 static opt_t * add_option_inner(const char name[], OPT_TYPE type, int val_count,
 		const char *vals[], opt_handler handler);
+static int allocates_str_value(OPT_TYPE type);
 static void print_changed_options(void);
 static int process_option(const char arg[]);
+static int handle_all_pseudo(const char arg[], const char suffix[]);
 static void print_options(void);
 static opt_t * get_option(const char option[]);
 static int set_on(opt_t *opt);
@@ -67,6 +69,7 @@ static int set_set(opt_t *opt, const char value[]);
 static int set_reset(opt_t *opt);
 static int set_add(opt_t *opt, const char value[]);
 static int set_remove(opt_t *opt, const char value[]);
+static int set_hat(opt_t *opt, const char value[]);
 static void notify_option_update(opt_t *opt, OPT_OP op, optval_t val);
 static int set_op(opt_t *opt, const char value[], SetOp op);
 static int charset_set(opt_t *opt, const char value[]);
@@ -74,6 +77,9 @@ static int charset_add_all(opt_t *opt, const char value[]);
 static void charset_add(char buffer[], char value);
 static int charset_remove_all(opt_t *opt, const char value[]);
 static void charset_remove(char buffer[], char value);
+static int charset_toggle_all(opt_t *opt, const char value[]);
+static int charset_do_all(opt_t *opt, mod_t func, const char value[]);
+static void charset_toggle(char buffer[], char value);
 static void for_each_char_of(char buffer[], mod_t func, const char input[]);
 static int replace_if_changed(char **current, const char new[]);
 static char * str_add(char old[], const char value[]);
@@ -88,8 +94,10 @@ static int complete_option_value(const opt_t *opt, const char beginning[]);
 static int complete_list_value(const opt_t *opt, const char beginning[]);
 static int complete_char_value(const opt_t *opt, const char beginning[]);
 
+/* Characters that can occur after option values. */
 static const char ENDING_CHARS[] = "!?&";
-static const char MIDDLE_CHARS[] = "+-=:";
+/* Characters that can between option names and values. */
+static const char MIDDLE_CHARS[] = "^+-=:";
 
 static int *opts_changed;
 
@@ -117,24 +125,24 @@ reset_option_to_default(const char name[])
 void
 reset_options_to_default(void)
 {
-	int i;
-	for(i = 0; i < options_count; i++)
+	size_t i;
+	for(i = 0U; i < options_count; ++i)
 	{
-		if(options[i].full != NULL)
-			continue;
-		set_reset(&options[i]);
+		if(options[i].full == NULL)
+		{
+			set_reset(&options[i]);
+		}
 	}
 }
 
 void
 clear_options(void)
 {
-	int i;
-
-	for(i = 0; i < options_count; i++)
+	size_t i;
+	for(i = 0U; i < options_count; ++i)
 	{
 		free(options[i].name);
-		if(options[i].type == OPT_STR || options[i].type == OPT_STRLIST)
+		if(allocates_str_value(options[i].type))
 		{
 			if(options[i].full == NULL)
 				free(options[i].def.str_val);
@@ -168,7 +176,7 @@ add_option(const char name[], const char abbr[], OPT_TYPE type, int val_count,
 			abbreviated->full = full_name;
 		full = find_option(full_name);
 	}
-	if(type == OPT_STR || type == OPT_STRLIST || type == OPT_CHARSET)
+	if(allocates_str_value(type))
 	{
 		full->def.str_val = strdup(def.str_val);
 		full->val.str_val = strdup(def.str_val);
@@ -178,6 +186,20 @@ add_option(const char name[], const char abbr[], OPT_TYPE type, int val_count,
 		full->def.int_val = def.int_val;
 		full->val.int_val = def.int_val;
 	}
+
+#ifndef NDEBUG
+	switch(type)
+	{
+		case OPT_CHARSET:
+			assert((size_t)full->val_count == strlen(*full->vals) &&
+					"Number of character set items is incorrect.");
+			break;
+
+		default:
+			/* No checks for other types, at least for now. */
+			break;
+	}
+#endif
 }
 
 /* Adds option to internal array of option descriptors.  Returns a pointer to
@@ -212,6 +234,14 @@ add_option_inner(const char name[], OPT_TYPE type, int val_count,
 	return p;
 }
 
+/* Checks whether given option type allocates memory for strings on heap.
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+allocates_str_value(OPT_TYPE type)
+{
+	return (type == OPT_STR || type == OPT_STRLIST || type == OPT_CHARSET);
+}
+
 void
 set_option(const char name[], optval_t val)
 {
@@ -255,8 +285,8 @@ set_options(const char args[])
 static void
 print_changed_options(void)
 {
-	int i;
-	for(i = 0; i < options_count; i++)
+	size_t i;
+	for(i = 0; i < options_count; ++i)
 	{
 		opt_t *opt = &options[i];
 
@@ -277,36 +307,36 @@ print_changed_options(void)
 	}
 }
 
-/* Processes one :set statement.  Returns zero on success. */
+/* Processes one :set statement.  Returns zero on success, otherwize non-zero is
+ * returned. */
 static int
 process_option(const char arg[])
 {
-	char option[OPTION_NAME_MAX + 1];
+	char optname[OPTION_NAME_MAX + 1];
 	int err;
-	const char *p;
+	const char *suffix;
 	opt_t *opt;
 
-	p = skip_alphas(arg);
+	suffix = skip_alphas(arg);
 
-	snprintf(option, p - arg + 1, "%s", arg);
+	copy_str(optname, suffix - arg + 1, arg);
 
-	if(strcmp(option, "all") == 0)
+	if(strcmp(optname, "all") == 0)
 	{
-		print_options();
-		return 0;
+		return handle_all_pseudo(arg, suffix);
 	}
 
-	opt = get_option(option);
+	opt = get_option(optname);
 	if(opt == NULL)
 	{
-		text_buffer_addf("%s: %s", "Unknown option", arg);
+		vle_tb_append_linef(vle_err, "%s: %s", "Unknown option", arg);
 		return 1;
 	}
 
 	err = 0;
-	if(*p == '\0')
+	if(*suffix == '\0')
 	{
-		opt_t *o = find_option(option);
+		opt_t *o = find_option(optname);
 		if(o != NULL)
 		{
 			if(o->type == OPT_BOOL)
@@ -314,63 +344,88 @@ process_option(const char arg[])
 			else
 				err = set_print(o);
 		}
-		else if(strncmp(option, "no", 2) == 0)
+		else if(strncmp(optname, "no", 2) == 0)
 		{
 			err = set_off(opt);
 		}
-		else if(strncmp(option, "inv", 3) == 0)
+		else if(strncmp(optname, "inv", 3) == 0)
 		{
 			err = set_inv(opt);
 		}
 	}
-	else if(char_is_one_of(ENDING_CHARS, *p))
+	else if(char_is_one_of(ENDING_CHARS, *suffix))
 	{
-		if(*(p + 1) != '\0')
+		if(*(suffix + 1) != '\0')
 		{
-			text_buffer_addf("%s: %s", "Trailing characters", arg);
+			vle_tb_append_linef(vle_err, "%s: %s", "Trailing characters", arg);
 			return 1;
 		}
-		if(*p == '!')
+		if(*suffix == '!')
 			err = set_inv(opt);
-		else if(*p == '?')
+		else if(*suffix == '?')
 			err = set_print(opt);
 		else
 			err = set_reset(opt);
 	}
-	else if(strncmp(p, "+=", 2) == 0)
+	else if(strncmp(suffix, "+=", 2) == 0)
 	{
-		err = set_add(opt, p + 2);
+		err = set_add(opt, suffix + 2);
 	}
-	else if(strncmp(p, "-=", 2) == 0)
+	else if(strncmp(suffix, "-=", 2) == 0)
 	{
-		err = set_remove(opt, p + 2);
+		err = set_remove(opt, suffix + 2);
 	}
-	else if(*p == '=' || *p == ':')
+	else if(strncmp(suffix, "^=", 2) == 0)
 	{
-		err = set_set(opt, p + 1);
+		err = set_hat(opt, suffix + 2);
+	}
+	else if(*suffix == '=' || *suffix == ':')
+	{
+		err = set_set(opt, suffix + 1);
 	}
 	else
 	{
-		text_buffer_addf("%s: %s", "Trailing characters", arg);
+		vle_tb_append_linef(vle_err, "%s: %s", "Trailing characters", arg);
 	}
 
 	if(err)
 	{
-		text_buffer_addf("%s: %s", "Invalid argument", arg);
+		vle_tb_append_linef(vle_err, "%s: %s", "Invalid argument", arg);
 	}
 	return err;
 }
 
+/* Handles "all" pseudo-option.  Actual action is determined by the suffix.
+ * Returns zero on success, otherwize non-zero is returned. */
+static int
+handle_all_pseudo(const char arg[], const char suffix[])
+{
+	switch(*suffix)
+	{
+		case '\0':
+			print_options();
+			return 0;
+		case '&':
+			reset_options_to_default();
+			return 0;
+
+		default:
+			vle_tb_append_linef(vle_err, "%s: %s", "Trailing characters", arg);
+			return 1;
+	}
+}
+
 /* Prints values of all options. */
 static void
 print_options(void)
 {
-	int i;
-	for(i = 0; i < options_count; i++)
+	size_t i;
+	for(i = 0U; i < options_count; ++i)
 	{
-		if(options[i].full != NULL)
-			continue;
-		set_print(&options[i]);
+		if(options[i].full == NULL)
+		{
+			set_print(&options[i]);
+		}
 	}
 }
 
@@ -515,7 +570,7 @@ set_set(opt_t *opt, const char value[])
 		const size_t valid_len = strspn(value, *opt->vals);
 		if(valid_len != strlen(value))
 		{
-			text_buffer_addf("Illegal character: <%c>", value[valid_len]);
+			vle_tb_append_linef(vle_err, "Illegal character: <%c>", value[valid_len]);
 			return -1;
 		}
 		if(charset_set(opt, value))
@@ -585,7 +640,7 @@ set_add(opt_t *opt, const char value[])
 		const size_t valid_len = strspn(value, *opt->vals);
 		if(valid_len != strlen(value))
 		{
-			text_buffer_addf("Illegal character: <%c>", value[valid_len]);
+			vle_tb_append_linef(vle_err, "Illegal character: <%c>", value[valid_len]);
 			return -1;
 		}
 		if(charset_add_all(opt, value))
@@ -650,6 +705,24 @@ set_remove(opt_t *opt, const char value[])
 	return 0;
 }
 
+/* Toggles value(s) from the option (^= operator).  Returns non-zero on
+ * success. */
+static int
+set_hat(opt_t *opt, const char value[])
+{
+	if(opt->type != OPT_CHARSET)
+	{
+		return -1;
+	}
+
+	if(charset_toggle_all(opt, value))
+	{
+		notify_option_update(opt, OP_MODIFIED, opt->val);
+	}
+
+	return 0;
+}
+
 /* Calls option handler to notify about option change.  Also updates
  * opts_changed flag. */
 static void
@@ -676,7 +749,7 @@ set_op(opt_t *opt, const char value[], SetOp op)
 		if(p == NULL)
 			p = value + strlen(value);
 
-		snprintf(val, p - value + 1, "%s", value);
+		copy_str(val, p - value + 1, value);
 
 		i = find_val(opt, val);
 		if(i != -1)
@@ -716,16 +789,10 @@ charset_set(opt_t *opt, const char value[])
 static int
 charset_add_all(opt_t *opt, const char value[])
 {
-	char new_val[opt->val_count + 1];
-	copy_str(new_val, sizeof(new_val), opt->val.str_val);
-	assert(strlen(opt->val.str_val) <= opt->val_count);
-
-	for_each_char_of(new_val, charset_add, value);
-	return replace_if_changed(&opt->val.str_val, new_val);
+	return charset_do_all(opt, &charset_add, value);
 }
 
-/* Adds an item to the value of an option of type OPT_CHARSET.  Returns non-zero
- * when value of the option was changed, otherwise zero is returned. */
+/* Adds an item to the value of an option of type OPT_CHARSET. */
 static void
 charset_add(char buffer[], char value)
 {
@@ -740,16 +807,10 @@ charset_add(char buffer[], char value)
 static int
 charset_remove_all(opt_t *opt, const char value[])
 {
-	char new_val[opt->val_count + 1];
-	copy_str(new_val, sizeof(new_val), opt->val.str_val);
-	assert(strlen(opt->val.str_val) <= opt->val_count);
-
-	for_each_char_of(new_val, charset_remove, value);
-	return replace_if_changed(&opt->val.str_val, new_val);
+	return charset_do_all(opt, &charset_remove, value);
 }
 
-/* Removes an item from the value of an option of type OPT_CHARSET.  Returns
- * non-zero when value of the option was changed, otherwise zero is returned. */
+/* Removes an item from the value of an option of type OPT_CHARSET. */
 static void
 charset_remove(char buffer[], char value)
 {
@@ -766,15 +827,64 @@ charset_remove(char buffer[], char value)
 	*l++ = '\0';
 }
 
-/* Calls the func once for each character in the input argument, passing the
- * buffer and the character as its arguments. */
+/* Toggles items in the option of type OPT_CHARSET.  Returns non-zero when value
+ * of the option was changed, otherwise zero is returned. */
+static int
+charset_toggle_all(opt_t *opt, const char value[])
+{
+	return charset_do_all(opt, &charset_toggle, value);
+}
+
+/* Invokes modification function for the value of the option of type
+ * OPT_CHARSET.  Returns non-zero when value of the option was changed,
+ * otherwise zero is returned. */
+static int
+charset_do_all(opt_t *opt, mod_t func, const char value[])
+{
+	char new_val[opt->val_count + 1];
+	copy_str(new_val, sizeof(new_val), opt->val.str_val);
+	assert(strlen(opt->val.str_val) <= (size_t)opt->val_count &&
+			"Character set includes duplicates?");
+
+	for_each_char_of(new_val, func, value);
+	return replace_if_changed(&opt->val.str_val, new_val);
+}
+
+/* Toggles items in the value of an option of type OPT_CHARSET. */
+static void
+charset_toggle(char buffer[], char value)
+{
+	char *l = buffer;
+	const char *r = buffer;
+	while(*r != '\0')
+	{
+		if(*r != value)
+		{
+			*l++ = *r;
+		}
+		r++;
+	}
+	if(l == r)
+	{
+		*l++ = value;
+	}
+	*l++ = '\0';
+}
+
+/* Calls the func once for each unique character in the input argument, passing
+ * the buffer and the character as its arguments. */
 static void
 for_each_char_of(char buffer[], mod_t func, const char input[])
 {
 	const char *val = input;
 	while(*val != '\0')
 	{
-		func(buffer, *val++);
+		/* Process only unique characters. */
+		if(strchr(input, *val) == val)
+		{
+			func(buffer, *val);
+		}
+		++val;
 	}
 }
 
@@ -871,10 +981,23 @@ set_print(const opt_t *opt)
 	{
 		snprintf(buf, sizeof(buf), "  %s=%s", opt->name, get_value(opt));
 	}
-	text_buffer_add(buf);
+	vle_tb_append_line(vle_err, buf);
 	return 0;
 }
 
+const char *
+get_option_value(const char name[])
+{
+	opt_t *opt = find_option(name);
+	assert(opt != NULL && "Wrong option name.");
+	if(opt == NULL)
+	{
+		return NULL;
+	}
+
+	return get_value(opt);
+}
+
 const char *
 get_value(const opt_t *opt)
 {
@@ -890,11 +1013,11 @@ get_value(const opt_t *opt)
 	else if(opt->type == OPT_STR || opt->type == OPT_STRLIST ||
 			opt->type == OPT_CHARSET)
 	{
-		snprintf(buf, sizeof(buf), "%s", opt->val.str_val ? opt->val.str_val : "");
+		copy_str(buf, sizeof(buf), opt->val.str_val ? opt->val.str_val : "");
 	}
 	else if(opt->type == OPT_ENUM)
 	{
-		snprintf(buf, sizeof(buf), "%s", opt->vals[opt->val.enum_item]);
+		copy_str(buf, sizeof(buf), opt->vals[opt->val.enum_item]);
 	}
 	else if(opt->type == OPT_SET)
 	{
@@ -938,7 +1061,7 @@ complete_options(const char args[], const char **start)
 			return;
 		}
 	}
-	if(strlen(buf) != args - *start)
+	if(strlen(buf) != (size_t)(args - *start))
 	{
 		*start = args;
 		buf[0] = '\0';
@@ -947,9 +1070,10 @@ complete_options(const char args[], const char **start)
 	bool_only = 0;
 
 	p = skip_alphas(buf);
-	is_value_completion = (p[0] == '=' || p[0] == ':');
-	is_value_completion = is_value_completion || (p[0] == '-' && p[1] == '=');
-	is_value_completion = is_value_completion || (p[0] == '+' && p[1] == '=');
+	is_value_completion =  (p[0] == '=' || p[0] == ':');
+	is_value_completion |= (p[0] == '-' && p[1] == '=');
+	is_value_completion |= (p[0] == '+' && p[1] == '=');
+	is_value_completion |= (p[0] == '^' && p[1] == '=');
 	if(is_value_completion)
 	{
 		char t = *p;
@@ -981,7 +1105,7 @@ complete_options(const char args[], const char **start)
 
 	if(!is_value_completion)
 	{
-		complete_option_name(buf, bool_only, 1);
+		complete_option_name(buf, bool_only, !bool_only);
 	}
 	else if(opt != NULL)
 	{
@@ -991,9 +1115,7 @@ complete_options(const char args[], const char **start)
 		}
 		else if(*p == '\0' && opt->type != OPT_BOOL)
 		{
-			char *const escaped = escape_chars(get_value(opt), " |");
-			vle_compl_add_match(escaped);
-			free(escaped);
+			vle_compl_put_match(escape_chars(get_value(opt), " |"));
 		}
 	}
 
@@ -1104,14 +1226,14 @@ static void
 complete_option_name(const char buf[], int bool_only, int pseudo)
 {
 	const size_t len = strlen(buf);
-	int i;
+	size_t i;
 
 	if(pseudo && strncmp(buf, "all", len) == 0)
 	{
 		vle_compl_add_match("all");
 	}
 
-	for(i = 0; i < options_count; i++)
+	for(i = 0U; i < options_count; ++i)
 	{
 		opt_t *const opt = &options[i];
 
