@@ -26,15 +26,16 @@
 #include <ctype.h> /* isdigit() */
 #include <stddef.h> /* NULL size_t */
 #include <stdlib.h> /* free() malloc() realloc() strtol() */
-#include <string.h> /* memcpy() memset() strchr() strcpy() strdup() strlen() */
+#include <string.h> /* memcpy() memset() strchr() strcpy() strdup() strlen()
+                       strncpy() */
 
 #include "cfg/config.h"
+#include "ui/ui.h"
 #include "utils/test_helpers.h"
 #include "utils/str.h"
 #include "utils/utf8.h"
 #include "utils/utils.h"
 #include "color_manager.h"
-#include "ui.h"
 
 static char * add_pattern_highlights(const char line[], size_t len,
 		const char no_esc[], const int offsets[], const regex_t *re);
@@ -117,10 +118,10 @@ esc_highlight_pattern(const char line[], const regex_t *re)
 		const size_t char_width_esc = get_char_width_esc(src_sym);
 		if(*src_sym != '\033')
 		{
-			int i;
+			size_t i;
 			const int offset = src_sym - line;
 			/* Each offset value is filled over whole character width. */
-			for(i = 0; i < char_width_esc; i++)
+			for(i = 0U; i < char_width_esc; ++i)
 			{
 				offsets[no_esc_sym_pos + i] = offset;
 			}
@@ -151,8 +152,8 @@ add_pattern_highlights(const char line[], size_t len, const char no_esc[],
 		const int offsets[], const regex_t *re)
 {
 	regmatch_t match;
-	char *next;
-	char *processed = NULL;
+	char *next = NULL;
+	char *processed;
 	int shift = 0;
 	int overhead = 0;
 
@@ -167,6 +168,7 @@ add_pattern_highlights(const char line[], size_t len, const char no_esc[],
 
 	/* Before the first match. */
 	strncpy(processed, line, offsets[match.rm_so]);
+	next = processed + offsets[match.rm_so];
 
 	/* All matches. */
 	do
@@ -175,12 +177,22 @@ add_pattern_highlights(const char line[], size_t len, const char no_esc[],
 		size_t new_overhead;
 		int so_offset;
 		void *ptr;
+		const int empty_match = (match.rm_so == match.rm_eo);
 
 		match.rm_so += shift;
 		match.rm_eo += shift;
 
 		so_offset = offsets[match.rm_so];
 
+		if(empty_match)
+		{
+			if(no_esc[match.rm_eo] == '\0')
+			{
+				shift = match.rm_eo;
+				break;
+			}
+		}
+
 		/* Between matches. */
 		if(shift != 0)
 		{
@@ -188,26 +200,46 @@ add_pattern_highlights(const char line[], size_t len, const char no_esc[],
 			strncpy(next, line + corrected, so_offset - corrected);
 		}
 
-		new_overhead = INV_OVERHEAD*count_substr_chars(no_esc, &match);
-		len += new_overhead;
-		if((ptr = realloc(processed, len + 1)) == NULL)
+		if(empty_match)
 		{
-			free(processed);
-			return NULL;
+			const int corrected = (shift == 0)
+			                    ? (size_t)(next - processed)
+			                    : correct_offset(line, offsets, shift);
+			const int len = offsets[match.rm_so + 1] - corrected;
+			strncpy(next, line + corrected, len);
+			next += len;
+			++shift;
 		}
-		processed = ptr;
+		else
+		{
+			new_overhead = INV_OVERHEAD*count_substr_chars(no_esc, &match);
+			len += new_overhead;
+			if((ptr = realloc(processed, len + 1)) == NULL)
+			{
+				free(processed);
+				return NULL;
+			}
+			processed = ptr;
 
-		match_len = correct_offset(line, offsets, match.rm_eo) - so_offset;
-		next = processed + so_offset + overhead;
-		next = add_highlighted_substr(line + so_offset, match_len, next);
+			match_len = correct_offset(line, offsets, match.rm_eo) - so_offset;
+			next = processed + so_offset + overhead;
+			next = add_highlighted_substr(line + so_offset, match_len, next);
 
-		shift = match.rm_eo;
-		overhead += new_overhead;
+			shift = match.rm_eo;
+			overhead += new_overhead;
+		}
 	}
 	while(regexec(re, no_esc + shift, 1, &match, 0) == 0);
 
+	/* Abort if there were no non-empty matches. */
+	if(next == NULL)
+	{
+		free(processed);
+		return 0;
+	}
+
 	/* After the last match. */
-	strcpy(next, line + correct_offset(line, offsets, shift));
+	strcpy(next, line + (shift == 0 ? 0 : correct_offset(line, offsets, shift)));
 
 	return processed;
 }
@@ -287,13 +319,14 @@ esc_print_line(const char line[], WINDOW *win, int col, int row, int max_width,
 {
 	int offset;
 	const char *curr = line;
-	size_t pos = 0;
+	size_t pos = 0U;
 	checked_wmove(win, row, col);
-	while(pos <= max_width && *curr != '\0')
+	while(pos <= (size_t)max_width && *curr != '\0')
 	{
 		size_t screen_width;
 		const char *const char_str = strchar2str(curr, pos, &screen_width);
-		if((pos += screen_width) <= max_width)
+		pos += screen_width;
+		if(pos <= (size_t)max_width)
 		{
 			if(!dry_run || screen_width == 0)
 			{
@@ -362,14 +395,8 @@ print_char_esc(WINDOW *win, const char str[], esc_state *state)
 {
 	if(str[0] == '\033')
 	{
-		int next_pair;
-
 		esc_state_update(state, str);
-
-		next_pair = colmgr_alloc_pair(state->fg, state->bg);
-		/* Fallback to default color pair on pair allocation failure. */
-		next_pair = (next_pair == -1) ? 0 : next_pair;
-		wattrset(win, COLOR_PAIR(next_pair) | state->attrs);
+		wattrset(win, COLOR_PAIR(colmgr_get_pair(state->fg, state->bg)) | state->attrs);
 	}
 	else
 	{
@@ -520,7 +547,7 @@ strchar2str(const char str[], int pos, size_t *screen_width)
 	static char buf[32];
 
 	const size_t char_width = get_char_width(str);
-	if(char_width != 1 || str[0] >= ' ')
+	if(char_width != 1 || (unsigned char)str[0] >= (unsigned char)' ')
 	{
 		memcpy(buf, str, char_width);
 		buf[char_width] = '\0';
@@ -551,7 +578,7 @@ strchar2str(const char str[], int pos, size_t *screen_width)
 	else if(str[0] == '\033')
 	{
 		char *dst = buf;
-		while(*str != 'm' && *str != '\0' && dst - buf < sizeof(buf) - 2)
+		while(*str != 'm' && *str != '\0' && (size_t)(dst - buf) < sizeof(buf) - 2)
 		{
 			*dst++ = *str++;
 		}
