@@ -23,12 +23,12 @@
 #include <string.h> /* memmove() strchr() strlen() strrchr() */
 
 #include "cfg/config.h"
+#include "ui/ui.h"
 #include "utils/fs_limits.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "fileops.h"
 #include "status.h"
-#include "ui.h"
 
 static const char * apply_mod(const char *path, const char *parent,
 		const char *mod, int *mod_len, int for_shell);
@@ -171,9 +171,9 @@ apply_dot_mod(const char *path, char *buf, size_t buf_len)
 {
 	size_t len = strlen(curr_view->curr_dir);
 	if(strnoscmp(path, curr_view->curr_dir, len) != 0 || path[len] == '\0')
-		snprintf(buf, buf_len, "%s", path);
+		copy_str(buf, buf_len, path);
 	else
-		snprintf(buf, buf_len, "%s", path + len + 1);
+		copy_str(buf, buf_len, path + len + 1);
 	return 0;
 }
 
