@@ -20,11 +20,6 @@
 
 #ifndef ENABLE_REMOTE_CMDS
 
-void
-ipc_pre_init(void)
-{
-}
-
 void
 ipc_init(recieve_callback callback_func)
 {
@@ -46,6 +41,12 @@ ipc_server(void)
 	return 1;
 }
 
+int
+ipc_enabled(void)
+{
+	return 0;
+}
+
 #else
 
 #ifdef _WIN32
@@ -89,8 +90,11 @@ static int server;
 static int sock = -1;
 
 void
-ipc_pre_init(void)
+ipc_init(recieve_callback callback_func)
 {
+	assert(!initialized);
+	callback = callback_func;
+
 #ifdef _WIN32
 	int result;
 	WSADATA wsaData;
@@ -103,20 +107,6 @@ ipc_pre_init(void)
 	}
 #endif
 
-	(void)create_socket();
-}
-
-void
-ipc_init(recieve_callback callback_func)
-{
-	assert(!initialized);
-	callback = callback_func;
-
-	if(sock == -1)
-	{
-		ipc_pre_init();
-	}
-
 	if(create_socket() != 0)
 	{
 #ifdef _WIN32
@@ -126,7 +116,8 @@ ipc_init(recieve_callback callback_func)
 		return;
 	}
 
-	try_become_a_server();
+	/* FIXME: used to call try_become_a_server() here, but it always succeeds (see
+	 *        FIXME comment there, which breaks our logic. */
 
 	atexit(&clean_at_exit);
 	initialized = 1;
@@ -178,6 +169,9 @@ try_become_a_server(void)
 	if(server)
 		return;
 
+	/* FIXME: with SO_REUSEADDR this operation always succeeds...  Which breaks
+	 *        client/server relationships. */
+
 #ifdef _WIN32
 	if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (const char *)&yes,
 			sizeof(yes)) != 0)
@@ -315,6 +309,12 @@ ipc_server(void)
 	return (server);
 }
 
+int
+ipc_enabled(void)
+{
+	return 1;
+}
+
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
