@@ -21,13 +21,11 @@
 
 typedef void (*recieve_callback)(char *args[]);
 
-/* Initializes IPC unit basic state. */
-void ipc_pre_init(void);
-
-/* Initializes IPC unit state. The callback_func will be called by ipc_check. */
+/* Initializes IPC unit state.  The callback_func will be called by
+ * ipc_check(). */
 void ipc_init(recieve_callback callback_func);
 
-/* Checks for incoming messages. Calls callback passed to ipc_init. */
+/* Checks for incoming messages.  Calls callback passed to ipc_init(). */
 void ipc_check(void);
 
 /* Sends data to server.  The data array should end with NULL. */
@@ -36,6 +34,10 @@ void ipc_send(char *data[]);
 /* Returns non-zero value if current instance is a server. */
 int ipc_server(void);
 
+/* Checks whether IPC is in use.  Returns non-zero if so, otherwise zero is
+ * returned. */
+int ipc_enabled(void);
+
 #endif /* VIFM__IPC_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
