@@ -28,10 +28,10 @@
 
 #include "../engine/cmds.h"
 #include "../modes/menu.h"
+#include "../ui/ui.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
 #include "../commands.h"
-#include "../ui.h"
 #include "menus.h"
 
 /* Minimal length of command name column. */
@@ -45,7 +45,7 @@ show_commands_menu(FileView *view)
 {
 	char **list;
 	int i;
-	int cmdname_width = CMDNAME_COLUMN_MIN_WIDTH;
+	size_t cmdname_width = CMDNAME_COLUMN_MIN_WIDTH;
 
 	static menu_info m;
 	init_menu_info(&m, COMMANDS_MENU, strdup("No commands set"));
@@ -65,7 +65,7 @@ show_commands_menu(FileView *view)
 			cmdname_width = cmdname_len;
 		}
 
-		m.len++;
+		++m.len;
 		assert(list[m.len] != NULL && "Broken list of user-defined commands.");
 	}
 	m.len /= 2;
@@ -73,7 +73,8 @@ show_commands_menu(FileView *view)
 	m.items = (m.len != 0) ? malloc(sizeof(char *)*m.len) : NULL;
 	for(i = 0; i < m.len; i++)
 	{
-		m.items[i] = format_str("%-*s %s", cmdname_width, list[i*2], list[i*2 + 1]);
+		m.items[i] = format_str("%-*s %s", (int)cmdname_width, list[i*2],
+				list[i*2 + 1]);
 	}
 
 	free_string_array(list, m.len*2);
@@ -87,7 +88,7 @@ static int
 execute_commands_cb(FileView *view, menu_info *m)
 {
 	break_at(m->items[m->pos], ' ');
-	exec_command(m->items[m->pos], view, GET_COMMAND);
+	exec_command(m->items[m->pos], view, CIT_COMMAND);
 	return 0;
 }
 
@@ -96,7 +97,7 @@ execute_commands_cb(FileView *view, menu_info *m)
 static KHandlerResponse
 commands_khandler(menu_info *m, const wchar_t keys[])
 {
-	if(wcscmp(keys, L"dd") == 0) /* remove element */
+	if(wcscmp(keys, L"dd") == 0) /* Remove element. */
 	{
 		char cmd_buf[512];
 
