@@ -23,10 +23,11 @@
 #include <string.h> /* strdup() */
 
 #include "../cfg/config.h"
+#include "../ui/ui.h"
 #include "../utils/fs.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../ui.h"
+#include "../filelist.h"
 #include "menus.h"
 
 static int execute_dirhistory_cb(FileView *view, menu_info *m);
@@ -66,8 +67,7 @@ show_history_menu(FileView *view)
 		/* Change the current dir to reflect the current file. */
 		if(stroscmp(view->history[i].dir, view->curr_dir) == 0)
 		{
-			(void)replace_string(&view->history[i].file,
-					view->dir_entry[view->list_pos].name);
+			(void)replace_string(&view->history[i].file, get_current_file_name(view));
 			m.pos = m.len;
 		}
 
@@ -124,7 +124,7 @@ show_history_menu(FileView *view)
 static int
 execute_dirhistory_cb(FileView *view, menu_info *m)
 {
-	goto_selected_directory(view, m);
+	goto_selected_directory(view, m->items[m->pos]);
 	return 0;
 }
 
