@@ -23,11 +23,13 @@
 #include <string.h> /* strdup() */
 
 #include "../cfg/config.h"
+#include "../modes/dialogs/msg_dialog.h"
+#include "../ui/statusbar.h"
+#include "../ui/ui.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../macros.h"
-#include "../ui.h"
 #include "menus.h"
 
 #ifdef _WIN32
@@ -46,19 +48,13 @@ show_find_menu(FileView *view, int with_path, const char args[])
 	char *targets = NULL;
 	char *cmd;
 
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
 		{ .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
 		{ .letter = 'A', .value = NULL, .uses_left = 0, .group =  1 },
 	};
 
 	static menu_info m;
-	init_menu_info(&m, FIND_MENU, strdup("No files found"));
-
-	m.title = format_str(" Find %s ", args);
-	m.execute_handler = &execute_find_cb;
-	m.key_handler = &filelist_khandler;
 
 	if(with_path)
 	{
@@ -68,7 +64,13 @@ show_find_menu(FileView *view, int with_path, const char args[])
 	}
 	else
 	{
-		targets = get_cmd_target();
+		targets = prepare_targets(view);
+		if(targets == NULL)
+		{
+			show_error_msg("Find", "Failed to setup target directory.");
+			return 0;
+		}
+
 		macros[0].value = targets;
 		macros[2].value = args;
 
@@ -85,6 +87,12 @@ show_find_menu(FileView *view, int with_path, const char args[])
 		}
 	}
 
+	init_menu_info(&m, FIND_MENU, strdup("No files found"));
+
+	m.title = format_str(" Find %s ", args);
+	m.execute_handler = &execute_find_cb;
+	m.key_handler = &filelist_khandler;
+
 	status_bar_message("find...");
 
 	cmd = expand_custom_macros(cfg.find_prg, ARRAY_LEN(macros), macros);
@@ -92,7 +100,7 @@ show_find_menu(FileView *view, int with_path, const char args[])
 	free(targets);
 	free(custom_args);
 
-	save_msg = capture_output_to_menu(view, cmd, &m);
+	save_msg = capture_output_to_menu(view, cmd, 0, &m);
 	free(cmd);
 
 	return save_msg;
