@@ -23,11 +23,13 @@
 #include <string.h> /* strdup() */
 
 #include "../cfg/config.h"
+#include "../modes/dialogs/msg_dialog.h"
+#include "../ui/statusbar.h"
+#include "../ui/ui.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../macros.h"
-#include "../ui.h"
 #include "menus.h"
 
 static int execute_grep_cb(FileView *view, menu_info *m);
@@ -40,8 +42,7 @@ show_grep_menu(FileView *view, const char args[], int invert)
 	char *cmd;
 	char *escaped_args = NULL;
 
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = NULL, .uses_left = 1, .group = -1 },
 		{ .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
 		{ .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
@@ -49,13 +50,20 @@ show_grep_menu(FileView *view, const char args[], int invert)
 	};
 
 	static menu_info m;
-	init_menu_info(&m, GREP_MENU, strdup("No matches found"));
+
+	targets = prepare_targets(view);
+	if(targets == NULL)
+	{
+		show_error_msg("Grep", "Failed to setup target directory.");
+		return 0;
+	}
+
+	init_menu_info(&m, GREP_MENU, format_str("No matches found: %s", args));
 
 	m.title = format_str(" Grep %s ", args);
 	m.execute_handler = &execute_grep_cb;
 	m.key_handler = &filelist_khandler;
 
-	targets = get_cmd_target();
 	macros[0].value = invert ? "-v" : "";
 	macros[1].value = args;
 	macros[2].value = targets;
@@ -72,7 +80,7 @@ show_grep_menu(FileView *view, const char args[], int invert)
 	free(targets);
 
 	status_bar_message("grep...");
-	save_msg = capture_output_to_menu(view, cmd, &m);
+	save_msg = capture_output_to_menu(view, cmd, 0, &m);
 	free(cmd);
 
 	return save_msg;
