@@ -19,27 +19,25 @@
 
 #include "menus.h"
 
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
 #include <curses.h>
 
-#include <unistd.h> /* access() F_OK R_OK */
-
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t */
+#include <stdio.h> /* FILE */
 #include <stdlib.h> /* free() malloc() */
 #include <string.h> /* memmove() memset() strdup() strcat() strncat() strchr()
                        strlen() strrchr() */
-#include <stdarg.h>
 #include <wchar.h> /* wchar_t wcscmp() */
 
 #include "../cfg/config.h"
+#include "../compat/os.h"
+#include "../modes/dialogs/msg_dialog.h"
 #include "../modes/cmdline.h"
 #include "../modes/menu.h"
 #include "../modes/modes.h"
-#include "../utils/file_streams.h"
+#include "../ui/cancellation.h"
+#include "../ui/statusbar.h"
+#include "../ui/ui.h"
 #include "../utils/fs.h"
 #include "../utils/fs_limits.h"
 #include "../utils/log.h"
@@ -47,11 +45,11 @@
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../utils/test_helpers.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../background.h"
 #include "../bookmarks.h"
+#include "../color_manager.h"
 #include "../color_scheme.h"
 #include "../colors.h"
 #include "../filelist.h"
@@ -59,29 +57,25 @@
 #include "../running.h"
 #include "../search.h"
 #include "../status.h"
-#include "../ui.h"
+#include "../vim.h"
 
-static int prompt_error_msg_internalv(const char title[], const char format[],
-		int prompt_skip, va_list pa);
-static int prompt_error_msg_internal(const char title[], const char message[],
-		int prompt_skip);
-TSTATIC char * parse_spec(const char spec[], int *line_num);
+static void draw_menu_item(menu_info *m, char buf[], int off,
+		const col_attr_t *col);
 static void open_selected_file(const char path[], int line_num);
 static void navigate_to_selected_file(FileView *view, const char path[]);
 static void normalize_top(menu_info *m);
+static void output_handler(const char line[], void *arg);
 static void append_to_string(char **str, const char suffix[]);
 static char * expand_tabulation_a(const char line[], size_t tab_stops);
 static size_t chars_in_str(const char s[], char c);
-static void redraw_error_msg(const char title_arg[], const char message_arg[],
-		int prompt_skip);
 
 static void
 show_position_in_menu(menu_info *m)
 {
-	char pos_buf[POS_WIN_WIDTH + 1];
+	char pos_buf[POS_WIN_MIN_WIDTH + 1];
 	snprintf(pos_buf, sizeof(pos_buf), " %d-%d ", m->pos + 1, m->len);
 
-	ui_pos_window_set(pos_buf);
+	ui_ruler_set(pos_buf);
 }
 
 void
@@ -108,9 +102,8 @@ clean_menu_position(menu_info *m)
 {
 	int x, z;
 	int off = 0;
-	char * buf = (char *)NULL;
+	char *buf;
 	col_attr_t col;
-	int type = MENU_COLOR;
 
 	x = getmaxx(menu_win) + get_utf8_overhead(m->items[m->pos]);
 
@@ -148,126 +141,12 @@ clean_menu_position(menu_info *m)
 	if(cfg.hl_search && m->matches != NULL && m->matches[m->pos])
 	{
 		mix_colors(&col, &cfg.cs.color[SELECTED_COLOR]);
-		type = SELECTED_COLOR;
 	}
 
-	init_pair(DCOLOR_BASE + type, col.fg, col.bg);
-	wattrset(menu_win, COLOR_PAIR(type + DCOLOR_BASE) | col.attr);
-
-	checked_wmove(menu_win, m->current, 1);
-	if(get_screen_string_length(m->items[m->pos] + off) > getmaxx(menu_win) - 4)
-	{
-		size_t len = get_normal_utf8_string_widthn(buf,
-				getmaxx(menu_win) - 3 - 4 + 1);
-		memset(buf + len, ' ', strlen(buf) - len);
-		buf[len + 3] = '\0';
-		wprint(menu_win, buf);
-		mvwaddstr(menu_win, m->current, getmaxx(menu_win) - 5, "...");
-	}
-	else
-	{
-		size_t len = get_normal_utf8_string_widthn(buf, getmaxx(menu_win) - 4 + 1);
-		buf[len] = '\0';
-		wprint(menu_win, buf);
-	}
-	waddstr(menu_win, " ");
-
-	wattroff(menu_win, COLOR_PAIR(type + DCOLOR_BASE) | col.attr);
-
+	draw_menu_item(m, buf, off, &col);
 	free(buf);
 }
 
-void
-show_error_msg(const char title[], const char message[])
-{
-	(void)prompt_error_msg_internal(title, message, 0);
-}
-
-void
-show_error_msgf(const char title[], const char format[], ...)
-{
-	va_list pa;
-
-	va_start(pa, format);
-	(void)prompt_error_msg_internalv(title, format, 0, pa);
-	va_end(pa);
-}
-
-int
-prompt_error_msg(const char title[], const char message[])
-{
-	return prompt_error_msg_internal(title, message, 1);
-}
-
-int
-prompt_error_msgf(const char title[], const char format[], ...)
-{
-	int result;
-	va_list pa;
-
-	va_start(pa, format);
-	result = prompt_error_msg_internalv(title, format, 1, pa);
-	va_end(pa);
-
-	return result;
-}
-
-/* Just a varargs wrapper over prompt_error_msg_internal. */
-static int
-prompt_error_msg_internalv(const char title[], const char format[],
-		int prompt_skip, va_list pa)
-{
-	char buf[2048];
-	vsnprintf(buf, sizeof(buf), format, pa);
-	return prompt_error_msg_internal(title, buf, prompt_skip);
-}
-
-/* Internal function for displaying error messages to a user.  Automatically
- * skips whitespace in front of the message and does nothing for empty messages
- * (due to skipping whitespace-only are counted as empty). When the prompt_skip
- * isn't zero, asks user about successive messages.  Returns non-zero if all
- * successive messages should be skipped, otherwise zero is returned. */
-static int
-prompt_error_msg_internal(const char title[], const char message[],
-		int prompt_skip)
-{
-	static int skip_until_started;
-	int key;
-
-	if(curr_stats.load_stage == 0)
-		return 1;
-	if(curr_stats.load_stage < 2 && skip_until_started)
-		return 1;
-
-	message = skip_whitespace(message);
-	if(*message == '\0')
-	{
-		return 0;
-	}
-
-	curr_stats.errmsg_shown = 1;
-
-	redraw_error_msg(title, message, prompt_skip);
-
-	do
-		key = wgetch(error_win);
-	while(key != 13 && (!prompt_skip || key != 3)); /* ascii Return, Ctrl-c */
-
-	if(curr_stats.load_stage < 2)
-		skip_until_started = key == 3;
-
-	werase(error_win);
-	wrefresh(error_win);
-
-	curr_stats.errmsg_shown = 0;
-
-	modes_update();
-	if(curr_stats.need_update != UT_NONE)
-		modes_redraw();
-
-	return key == 3;
-}
-
 void
 init_menu_info(menu_info *m, int menu_type, char empty_msg[])
 {
@@ -318,9 +197,9 @@ setup_menu(void)
 	curs_set(FALSE);
 	werase(menu_win);
 	werase(status_bar);
-	werase(pos_win);
+	werase(ruler_win);
 	wrefresh(status_bar);
-	wrefresh(pos_win);
+	wrefresh(ruler_win);
 }
 
 void
@@ -408,20 +287,35 @@ move_to_menu_pos(int pos, menu_info *m)
 	col = cfg.cs.color[WIN_COLOR];
 
 	if(cfg.hl_search && m->matches != NULL && m->matches[pos])
+	{
 		mix_colors(&col, &cfg.cs.color[SELECTED_COLOR]);
+	}
 
 	mix_colors(&col, &cfg.cs.color[CURR_LINE_COLOR]);
+	m->pos = pos;
 
-	init_pair(DCOLOR_BASE + MENU_CURRENT_COLOR, col.fg, col.bg);
-	wattrset(menu_win, COLOR_PAIR(DCOLOR_BASE + MENU_CURRENT_COLOR) | col.attr);
+	draw_menu_item(m, buf, off, &col);
+	free(buf);
+	show_position_in_menu(m);
+}
+
+/* Draws single menu item at current position. */
+static void
+draw_menu_item(menu_info *m, char buf[], int off, const col_attr_t *col)
+{
+	wattrset(menu_win, COLOR_PAIR(colmgr_get_pair(col->fg, col->bg)) | col->attr);
 
 	checked_wmove(menu_win, m->current, 1);
-	if(get_screen_string_length(m->items[pos] + off) > getmaxx(menu_win) - 4)
+	if(get_screen_string_length(m->items[m->pos] + off) >
+			(size_t)(getmaxx(menu_win) - 4))
 	{
 		size_t len = get_normal_utf8_string_widthn(buf,
 				getmaxx(menu_win) - 3 - 4 + 1);
 		memset(buf + len, ' ', strlen(buf) - len);
-		buf[len + 3] = '\0';
+		if(strlen(buf) > len + 3)
+		{
+			buf[len + 3] = '\0';
+		}
 		wprint(menu_win, buf);
 		mvwaddstr(menu_win, m->current, getmaxx(menu_win) - 5, "...");
 	}
@@ -433,17 +327,17 @@ move_to_menu_pos(int pos, menu_info *m)
 	}
 	waddstr(menu_win, " ");
 
-	wattroff(menu_win, COLOR_PAIR(DCOLOR_BASE + MENU_CURRENT_COLOR) | col.attr);
-
-	m->pos = pos;
-	free(buf);
-	show_position_in_menu(m);
+	wattroff(menu_win, COLOR_PAIR(colmgr_get_pair(col->fg, col->bg)) | col->attr);
 }
 
 void
 redraw_menu(menu_info *m)
 {
-	resize_for_menu_like();
+	if(resize_for_menu_like() != 0)
+	{
+		return;
+	}
+
 	m->win_rows = getmaxy(menu_win);
 
 	draw_menu(m);
@@ -457,14 +351,14 @@ goto_selected_file(FileView *view, const char spec[], int try_open)
 	char *path_buf;
 	int line_num;
 
-	path_buf = parse_spec(spec, &line_num);
+	path_buf = parse_file_spec(spec, &line_num);
 	if(path_buf == NULL)
 	{
 		show_error_msg("Memory Error", "Unable to allocate enough memory");
 		return;
 	}
 
-	if(access(path_buf, F_OK) == 0)
+	if(path_exists(path_buf, DEREF))
 	{
 		if(try_open)
 		{
@@ -483,67 +377,13 @@ goto_selected_file(FileView *view, const char spec[], int try_open)
 	free(path_buf);
 }
 
-/* Extracts path and line number from the spec (1 when absent from the spec).
- * Returns path and sets *line_num to line number, otherwise NULL is
- * returned. */
-TSTATIC char *
-parse_spec(const char spec[], int *line_num)
-{
-	char *path_buf;
-	const char *colon;
-	int colon_lookup_offset = 0;
-	const size_t bufs_len = 2 + strlen(spec) + 1 + 1;
-
-	path_buf = malloc(bufs_len);
-	if(path_buf == NULL)
-	{
-		return NULL;
-	}
-
-	if(is_path_absolute(spec))
-	{
-		path_buf[0] = '\0';
-	}
-	else
-	{
-		copy_str(path_buf, bufs_len, "./");
-	}
-
-#ifdef _WIN32
-	if(is_path_absolute(spec))
-	{
-		colon_lookup_offset = 2;
-	}
-#endif
-
-	colon = strchr(spec + colon_lookup_offset, ':');
-	if(colon != NULL)
-	{
-		strncat(path_buf, spec, colon - spec);
-		*line_num = atoi(colon + 1);
-	}
-	else
-	{
-		strcat(path_buf, spec);
-		*line_num = 1;
-	}
-
-	chomp(path_buf);
-
-#ifdef _WIN32
-	to_forward_slash(path_buf);
-#endif
-
-	return path_buf;
-}
-
 /* Opens file specified by its path on the given line number. */
 static void
 open_selected_file(const char path[], int line_num)
 {
-	if(access(path, R_OK) == 0)
+	if(os_access(path, R_OK) == 0)
 	{
-		(void)view_file(path, line_num, -1, 1);
+		(void)vim_view_file(path, line_num, -1, 1);
 	}
 	else
 	{
@@ -555,10 +395,6 @@ open_selected_file(const char path[], int line_num)
 static void
 navigate_to_selected_file(FileView *view, const char path[])
 {
-	/* Check whether target path is directory while we don't change current
-	 * working directory by invoking change_directory() function below. */
-	const int dst_is_dir = is_dir(path);
-
 	char name[NAME_MAX];
 	char *dir = strdup(path);
 	char *const last_slash = find_slashr(dir);
@@ -579,10 +415,6 @@ navigate_to_selected_file(FileView *view, const char path[])
 
 		load_dir_list(view, 0);
 
-		if(dst_is_dir)
-		{
-			strcat(name, "/");
-		}
 		(void)ensure_file_is_selected(view, name);
 	}
 	else
@@ -594,14 +426,14 @@ navigate_to_selected_file(FileView *view, const char path[])
 }
 
 void
-goto_selected_directory(FileView *view, menu_info *m)
+goto_selected_directory(FileView *view, const char path[])
 {
 	if(!cfg.auto_ch_pos)
 	{
-		clean_positions_in_history(curr_view);
+		clean_positions_in_history(view);
 		curr_stats.ch_pos = 0;
 	}
-	navigate_to(view, m->items[m->pos]);
+	navigate_to(view, path);
 	if(!cfg.auto_ch_pos)
 	{
 		curr_stats.ch_pos = 1;
@@ -635,7 +467,6 @@ draw_menu(menu_info *m)
 		char *buf;
 		char *ptr = NULL;
 		col_attr_t col;
-		int type = WIN_COLOR;
 
 		chomp(m->items[x]);
 		if((ptr = strchr(m->items[x], '\n')) || (ptr = strchr(m->items[x], '\r')))
@@ -646,11 +477,9 @@ draw_menu(menu_info *m)
 		if(cfg.hl_search && m->matches != NULL && m->matches[x])
 		{
 			mix_colors(&col, &cfg.cs.color[SELECTED_COLOR]);
-			type = SELECTED_COLOR;
 		}
 
-		init_pair(DCOLOR_BASE + type, col.fg, col.bg);
-		wattron(menu_win, COLOR_PAIR(DCOLOR_BASE + type) | col.attr);
+		wattron(menu_win, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr);
 
 		z = m->hor_pos;
 		off = 0;
@@ -666,7 +495,7 @@ draw_menu(menu_info *m)
 				buf[z] = ' ';
 
 		checked_wmove(menu_win, i, 2);
-		if(get_screen_string_length(buf) > win_len - 4)
+		if(get_screen_string_length(buf) > (size_t)(win_len - 4))
 		{
 			size_t len = get_normal_utf8_string_widthn(buf, win_len - 3 - 4);
 			memset(buf + len, ' ', strlen(buf) - len);
@@ -684,7 +513,7 @@ draw_menu(menu_info *m)
 
 		free(buf);
 
-		wattroff(menu_win, COLOR_PAIR(DCOLOR_BASE + type) | col.attr);
+		wattroff(menu_win, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr);
 
 		if(i + 3 > y)
 			break;
@@ -699,50 +528,15 @@ normalize_top(menu_info *m)
 }
 
 int
-capture_output_to_menu(FileView *view, const char cmd[], menu_info *m)
+capture_output_to_menu(FileView *view, const char cmd[], int user_sh,
+		menu_info *m)
 {
-	FILE *file, *err;
-	char *line = NULL;
-	int x;
-	pid_t pid;
-
-	LOG_INFO_MSG("Capturing output of the command to a menu: %s", cmd);
-
-	pid = background_and_capture((char *)cmd, &file, &err);
-	if(pid == (pid_t)-1)
+	if(process_cmd_output("Loading menu", cmd, user_sh, &output_handler, m) != 0)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
 		return 0;
 	}
 
-	show_progress("", 0);
-
-	ui_cancellation_reset();
-	ui_cancellation_enable();
-
-	wait_for_data_from(pid, file, 0);
-
-	x = 0;
-	while((line = read_line(file, line)) != NULL)
-	{
-		char *expanded_line;
-		show_progress("Loading menu", 1000);
-		m->items = realloc(m->items, sizeof(char *)*(x + 1));
-		expanded_line = expand_tabulation_a(line, cfg.tab_stop);
-		if(expanded_line != NULL)
-		{
-			m->items[x++] = expanded_line;
-		}
-
-		wait_for_data_from(pid, file, 0);
-	}
-	m->len = x;
-
-	ui_cancellation_disable();
-
-	fclose(file);
-	print_errors(err);
-
 	if(ui_cancellation_requested())
 	{
 		append_to_string(&m->title, "(cancelled) ");
@@ -752,6 +546,21 @@ capture_output_to_menu(FileView *view, const char cmd[], menu_info *m)
 	return display_menu(m, view);
 }
 
+/* Implements process_cmd_output() callback that loads lines to a menu. */
+static void
+output_handler(const char line[], void *arg)
+{
+	menu_info *m = arg;
+	char *expanded_line;
+
+	m->items = realloc(m->items, sizeof(char *)*(m->len + 1));
+	expanded_line = expand_tabulation_a(line, cfg.tab_stop);
+	if(expanded_line != NULL)
+	{
+		m->items[m->len++] = expanded_line;
+	}
+}
+
 /* Replaces *str with a copy of the with string extended by the suffix.  *str
  * can be NULL in which case it's treated as empty string. equal to the with (then function does nothing).  Returns non-zero if memory allocation
  * failed. */
@@ -823,179 +632,20 @@ display_menu(menu_info *m, FileView *view)
 	}
 }
 
-int
-query_user_menu(const char title[], const char message[])
-{
-	int key;
-	char *dup = strdup(message);
-
-	curr_stats.errmsg_shown = 2;
-
-	redraw_error_msg(title, message, 0);
-
-	do
-	{
-		key = wgetch(error_win);
-	}
-	while(key != 'y' && key != 'n' && key != ERR);
-
-	free(dup);
-
-	curr_stats.errmsg_shown = 0;
-
-	werase(error_win);
-	wrefresh(error_win);
-
-	touchwin(stdscr);
-
-	update_all_windows();
-
-	if(curr_stats.need_update != UT_NONE)
-		update_screen(UT_FULL);
-
-	return key == 'y';
-}
-
-void
-redraw_error_msg_window(void)
-{
-	redraw_error_msg(NULL, NULL, 0);
-}
-
-/* Draws error message on the screen or redraws the last message when both
- * title_arg and message_arg are NULL. */
-static void
-redraw_error_msg(const char title_arg[], const char message_arg[],
-		int prompt_skip)
-{
-	/* TODO: refactor this function redraw_error_msg() */
-
-	static const char *title;
-	static const char *message;
-	static int ctrl_c;
-
-	int sx, sy;
-	int x, y;
-	int z;
-	const char *text;
-
-	if(title_arg != NULL && message_arg != NULL)
-	{
-		title = title_arg;
-		message = message_arg;
-		ctrl_c = prompt_skip;
-	}
-
-	assert(message != NULL);
-
-	curs_set(FALSE);
-	werase(error_win);
-
-	getmaxyx(stdscr, sy, sx);
-
-	y = sy - 3 + !cfg.last_status;
-	x = sx - 2;
-	wresize(error_win, y, x);
-
-	z = strlen(message);
-	if(z <= x - 2 && strchr(message, '\n') == NULL)
-	{
-		y = 6;
-		wresize(error_win, y, x);
-		mvwin(error_win, (sy - y)/2, (sx - x)/2);
-		checked_wmove(error_win, 2, (x - z)/2);
-		wprint(error_win, message);
-	}
-	else
-	{
-		int i;
-		int cy = 2;
-		i = 0;
-		while(i < z)
-		{
-			int j;
-			char buf[x - 2 + 1];
-
-			snprintf(buf, sizeof(buf), "%s", message + i);
-
-			for(j = 0; buf[j] != '\0'; j++)
-				if(buf[j] == '\n')
-					break;
-
-			if(buf[j] != '\0')
-				i++;
-			buf[j] = '\0';
-			i += j;
-
-			if(buf[0] == '\0')
-				continue;
-
-			y = cy + 4;
-			mvwin(error_win, (sy - y)/2, (sx - x)/2);
-			wresize(error_win, y, x);
-
-			checked_wmove(error_win, cy++, 1);
-			wprint(error_win, buf);
-		}
-	}
-
-	box(error_win, 0, 0);
-	if(title[0] != '\0')
-		mvwprintw(error_win, 0, (x - strlen(title) - 2)/2, " %s ", title);
-
-	if(curr_stats.errmsg_shown == 1)
-	{
-		if(ctrl_c)
-		{
-			text = "Press Return to continue or Ctrl-C to skip other error messages";
-		}
-		else
-		{
-			text = "Press Return to continue";
-		}
-	}
-	else
-	{
-		text = "Enter [y]es or [n]o";
-	}
-	mvwaddstr(error_win, y - 2, (x - strlen(text))/2, text);
-}
-
-void
-print_errors(FILE *ef)
-{
-	char linebuf[160];
-	char buf[sizeof(linebuf)*5];
-
-	if(ef == NULL)
-		return;
-
-	buf[0] = '\0';
-	while(fgets(linebuf, sizeof(linebuf), ef) == linebuf)
-	{
-		if(linebuf[0] == '\n')
-			continue;
-		if(strlen(buf) + strlen(linebuf) + 1 >= sizeof(buf))
-		{
-			int skip = (prompt_error_msg("Background Process Error", buf) != 0);
-			buf[0] = '\0';
-			if(skip)
-				break;
-		}
-		strcat(buf, linebuf);
-	}
-
-	if(buf[0] != '\0')
-		show_error_msg("Background Process Error", buf);
-
-	fclose(ef);
-}
-
 char *
-get_cmd_target(void)
+prepare_targets(FileView *view)
 {
-	return (curr_view->selected_files > 0) ?
-		expand_macros("%f", NULL, NULL, 1) : strdup(".");
+	if(view->selected_files > 0)
+	{
+		return expand_macros("%f", NULL, NULL, 1);
+	}
+
+	if(!flist_custom_active(view))
+	{
+		return strdup(".");
+	}
+
+	return (vifm_chdir(flist_get_dir(view)) == 0) ? strdup(".") : NULL;
 }
 
 KHandlerResponse
@@ -1014,5 +664,70 @@ filelist_khandler(menu_info *m, const wchar_t keys[])
 	return KHR_UNHANDLED;
 }
 
+int
+menu_to_custom_view(menu_info *m, FileView *view)
+{
+	int i;
+	char *current = NULL;
+
+	flist_custom_start(view, m->title);
+
+	for(i = 0; i < m->len; ++i)
+	{
+		char *path;
+		int line_num;
+
+		/* Skip empty lines. */
+		if(skip_whitespace(m->items[i])[0] == '\0')
+		{
+			continue;
+		}
+
+		path = parse_file_spec(m->items[i], &line_num);
+		if(path == NULL)
+		{
+			continue;
+		}
+
+		flist_custom_add(view, path);
+
+		/* Use either exact position or the next path. */
+		if(i == m->pos || (current == NULL && i > m->pos))
+		{
+			current = path;
+			continue;
+		}
+
+		free(path);
+	}
+
+	/* If current line and none of the lines below didn't contain valid path, try
+	 * to use file above cursor position. */
+	if(current == NULL && view->custom.entry_count != 0)
+	{
+		char full_path[PATH_MAX];
+		get_full_path_of(&view->custom.entries[view->custom.entry_count - 1],
+				sizeof(full_path), full_path);
+
+		current = strdup(full_path);
+	}
+
+	view->custom.unsorted = 0;
+
+	if(flist_custom_finish(view) != 0)
+	{
+		free(current);
+		return 1;
+	}
+
+	if(current != NULL)
+	{
+		flist_goto_by_path(view, current);
+		free(current);
+	}
+
+	return 0;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
