@@ -19,7 +19,7 @@
 #ifndef VIFM__MENUS__TRASH_MENU_H__
 #define VIFM__MENUS__TRASH_MENU_H__
 
-#include "../ui.h"
+#include "../ui/ui.h"
 
 /* Returns non-zero if status bar message should be saved. */
 int show_trash_menu(FileView *view);
