@@ -23,8 +23,8 @@
 #include <stdlib.h> /* realloc() */
 #include <string.h> /* strdup() strlen() */
 
+#include "../ui/ui.h"
 #include "../utils/string_array.h"
-#include "../ui.h"
 #include "../undo.h"
 #include "menus.h"
 
@@ -32,7 +32,6 @@ int
 show_undolist_menu(FileView *view, int with_details)
 {
 	char **p;
-	size_t len;
 
 	static menu_info m;
 	init_menu_info(&m, UNDOLIST_MENU, strdup("Undolist is empty"));
@@ -47,6 +46,8 @@ show_undolist_menu(FileView *view, int with_details)
 
 	if(m.len > 0)
 	{
+		size_t len;
+
 		m.len = add_to_string_array(&m.items, m.len, 1, "list end");
 
 		/* Add current position mark to menu item. */
