@@ -21,37 +21,40 @@
 
 #include <curses.h>
 
-#ifndef _WIN32
-#include <grp.h>
-#include <pwd.h>
-#endif
 #include <sys/stat.h>
 
 #include <assert.h> /* assert() */
 #include <inttypes.h> /* PRId64 */
+#include <stddef.h> /* size_t */
 #include <stdint.h> /* uint64_t */
 #include <stdio.h>
 #include <string.h> /* strlen() */
-#include <time.h>
+#include <time.h> /* tm localtime() strftime() */
 
+#include "../cfg/config.h"
+#include "../compat/os.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../menus/menus.h"
+#include "../ui/ui.h"
 #include "../utils/fs.h"
 #include "../utils/fs_limits.h"
 #include "../utils/macros.h"
+#include "../utils/str.h"
 #include "../utils/tree.h"
+#include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../filelist.h"
 #include "../file_magic.h"
 #include "../status.h"
 #include "../types.h"
-#include "../ui.h"
 #include "modes.h"
 
 static void leave_file_info_mode(void);
+static int print_item(const char label[], const char path[], int curr_y);
 static int show_file_type(FileView *view, int curr_y);
 static int show_mime_type(FileView *view, int curr_y);
+static void format_time(time_t t, char buf[], size_t buf_size);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
 
@@ -110,62 +113,48 @@ leave_file_info_mode(void)
 void
 redraw_file_info_dialog(void)
 {
-	char name_buf[NAME_MAX];
+	const dir_entry_t *entry;
 	char perm_buf[26];
 	char size_buf[56];
 	char buf[256];
 #ifndef _WIN32
-	char uid_buf[26];
-	struct passwd *pwd_buf;
-	struct group *grp_buf;
+	char id_buf[26];
 #endif
-	struct tm *tm_ptr;
 	int curr_y;
 	uint64_t size;
 	int size_not_precise;
 
 	assert(view != NULL);
 
-	resize_for_menu_like();
+	if(resize_for_menu_like() != 0)
+	{
+		return;
+	}
 
 	werase(menu_win);
 
-	snprintf(name_buf, sizeof(name_buf), "%s",
-			view->dir_entry[view->list_pos].name);
+	entry = &view->dir_entry[view->list_pos];
 
 	size = 0;
-	if(view->dir_entry[view->list_pos].type == DIRECTORY)
-		tree_get_data(curr_stats.dirsize_cache,
-				view->dir_entry[view->list_pos].name, &size);
+	if(entry->type == FT_DIR)
+	{
+		char full_path[PATH_MAX];
+		get_current_full_path(view, sizeof(full_path), full_path);
+		tree_get_data(curr_stats.dirsize_cache, full_path, &size);
+	}
 
 	if(size == 0)
-		size = view->dir_entry[view->list_pos].size;
+	{
+		size = entry->size;
+	}
 
 	size_not_precise = friendly_size_notation(size, sizeof(size_buf), size_buf);
 
-#ifndef _WIN32
-	if((pwd_buf = getpwuid(view->dir_entry[view->list_pos].uid)) == NULL)
-	{
-		snprintf(uid_buf, sizeof(uid_buf), "%d",
-				(int) view->dir_entry[view->list_pos].uid);
-	}
-	else
-	{
-		snprintf(uid_buf, sizeof(uid_buf), "%s", pwd_buf->pw_name);
-	}
-	get_perm_string(perm_buf, sizeof(perm_buf),
-			view->dir_entry[view->list_pos].mode);
-#else
-	snprintf(perm_buf, sizeof(perm_buf), "%s",
-			attr_str_long(view->dir_entry[view->list_pos].attrs));
-#endif
-
 	curr_y = 2;
-	mvwaddstr(menu_win, curr_y, 2, "File: ");
-	name_buf[getmaxx(menu_win) - 8] = '\0';
-	checked_wmove(menu_win, curr_y, 8);
-	wprint(menu_win, name_buf);
-	curr_y += 2;
+
+	curr_y += print_item("Path: ", entry->origin, curr_y);
+	curr_y += print_item("Name: ", entry->name, curr_y);
+
 	mvwaddstr(menu_win, curr_y, 2, "Size: ");
 	mvwaddstr(menu_win, curr_y, 8, size_buf);
 	if(size_not_precise)
@@ -179,48 +168,37 @@ redraw_file_info_dialog(void)
 	curr_y += show_mime_type(view, curr_y);
 
 #ifndef _WIN32
-	mvwaddstr(menu_win, curr_y, 2, "Permissions: ");
+	get_perm_string(perm_buf, sizeof(perm_buf), entry->mode);
+	curr_y += print_item("Permissions: ", perm_buf, curr_y);
 #else
-	mvwaddstr(menu_win, curr_y, 2, "Attributes: ");
+	copy_str(perm_buf, sizeof(perm_buf), attr_str_long(entry->attrs));
+	curr_y += print_item("Attributes: ", perm_buf, curr_y);
 #endif
-	mvwaddstr(menu_win, curr_y, 15, perm_buf);
-	curr_y += 2;
 
-	mvwaddstr(menu_win, curr_y, 2, "Modified: ");
-	tm_ptr = localtime(&view->dir_entry[view->list_pos].mtime);
-	strftime(buf, sizeof (buf), "%a %b %d %Y %I:%M %p", tm_ptr);
-	checked_wmove(menu_win, curr_y, 13);
-	wprint(menu_win, buf);
-	curr_y += 2;
+	format_time(entry->mtime, buf, sizeof(buf));
+	curr_y += print_item("Modified: ", buf, curr_y);
 
-	mvwaddstr(menu_win, curr_y, 2, "Accessed: ");
-	tm_ptr = localtime(&view->dir_entry[view->list_pos].atime);
-	strftime(buf, sizeof (buf), "%a %b %d %Y %I:%M %p", tm_ptr);
-	checked_wmove(menu_win, curr_y, 13);
-	wprint(menu_win, buf);
-	curr_y += 2;
+	format_time(entry->atime, buf, sizeof(buf));
+	curr_y += print_item("Accessed: ", buf, curr_y);
+
+	format_time(entry->ctime, buf, sizeof(buf));
+#ifndef _WIN32
+	curr_y += print_item("Changed: ", buf, curr_y);
+#else
+	curr_y += print_item("Created: ", buf, curr_y);
+#endif
 
 #ifndef _WIN32
-	mvwaddstr(menu_win, curr_y, 2, "Changed: ");
-#else
-	mvwaddstr(menu_win, curr_y, 2, "Created: ");
-#endif
-	tm_ptr = localtime(&view->dir_entry[view->list_pos].ctime);
-	strftime(buf, sizeof (buf), "%a %b %d %Y %I:%M %p", tm_ptr);
-	checked_wmove(menu_win, curr_y, 13);
-	wprint(menu_win, buf);
-	curr_y += 2;
+	get_uid_string(entry, 0, sizeof(id_buf), id_buf);
+	curr_y += print_item("Owner: ", id_buf, curr_y);
 
-#ifndef _WIN32
-	mvwaddstr(menu_win, curr_y, 2, "Owner: ");
-	mvwaddstr(menu_win, curr_y, 10, uid_buf);
-	curr_y += 2;
-
-	mvwaddstr(menu_win, curr_y, 2, "Group: ");
-	if((grp_buf = getgrgid(view->dir_entry[view->list_pos].gid)) != NULL)
-		mvwaddstr(menu_win, curr_y, 10, grp_buf->gr_name);
+	get_gid_string(entry, 0, sizeof(id_buf), id_buf);
+	curr_y += print_item("Group: ", id_buf, curr_y);
 #endif
 
+	/* Fake use after last assignment. */
+	(void)curr_y;
+
 	box(menu_win, 0, 0);
 	checked_wmove(menu_win, 0, 3);
 	wprint(menu_win, " File Information ");
@@ -229,47 +207,78 @@ redraw_file_info_dialog(void)
 	was_redraw = 1;
 }
 
-/* Returns increment for curr_y */
+/* Prints item prefixed with a label truncating the item if it's too long.
+ * Returns increment for curr_y. */
+static int
+print_item(const char label[], const char path[], int curr_y)
+{
+	const int max_width = getmaxx(menu_win) - strlen(label) - 2;
+	const size_t print_len = get_normal_utf8_string_widthn(path, max_width);
+
+	mvwaddstr(menu_win, curr_y, 2, label);
+	if(path[print_len] == '\0')
+	{
+		wprint(menu_win, path);
+	}
+	else
+	{
+		char path_buf[PATH_MAX];
+		copy_str(path_buf, MIN(sizeof(path_buf), print_len + 1), path);
+		wprint(menu_win, path_buf);
+	}
+
+	return 2;
+}
+
+/* Returns increment for curr_y. */
 static int
 show_file_type(FileView *view, int curr_y)
 {
+	const dir_entry_t *entry;
 	int x;
 	int old_curr_y = curr_y;
 	x = getmaxx(menu_win);
 
+	entry = &view->dir_entry[view->list_pos];
+
 	mvwaddstr(menu_win, curr_y, 2, "Type: ");
-	if(view->dir_entry[view->list_pos].type == LINK)
+	if(entry->type == FT_LINK)
 	{
+		char full_path[PATH_MAX];
 		char linkto[PATH_MAX + NAME_MAX];
-		char *filename = view->dir_entry[view->list_pos].name;
+
+		get_current_full_path(view, sizeof(full_path), full_path);
 
 		mvwaddstr(menu_win, curr_y, 8, "Link");
 		curr_y += 2;
 		mvwaddstr(menu_win, curr_y, 2, "Link To: ");
 
-		if(get_link_target(filename, linkto, sizeof(linkto)) == 0)
+		if(get_link_target(full_path, linkto, sizeof(linkto)) == 0)
 		{
 			mvwaddnstr(menu_win, curr_y, 11, linkto, x - 11);
 
-			if(!path_exists(linkto))
+			if(!path_exists(linkto, DEREF))
+			{
 				mvwaddstr(menu_win, curr_y - 2, 12, " (BROKEN)");
+			}
 		}
 		else
 		{
 			mvwaddstr(menu_win, curr_y, 11, "Couldn't Resolve Link");
 		}
 	}
-	else if(view->dir_entry[view->list_pos].type == EXECUTABLE ||
-			view->dir_entry[view->list_pos].type == REGULAR)
+	else if(entry->type == FT_EXEC || entry->type == FT_REG)
 	{
 #ifdef HAVE_FILE_PROG
+		char full_path[PATH_MAX];
 		FILE *pipe;
 		char command[1024];
 		char buf[NAME_MAX];
 
-		/* Use the file command to get file information */
-		snprintf(command, sizeof(command), "file \"%s\" -b",
-				view->dir_entry[view->list_pos].name);
+		get_current_full_path(view, sizeof(full_path), full_path);
+
+		/* Use the file command to get file information. */
+		snprintf(command, sizeof(command), "file \"%s\" -b", full_path);
 
 		if((pipe = popen(command, "r")) == NULL)
 		{
@@ -283,55 +292,71 @@ show_file_type(FileView *view, int curr_y)
 		pclose(pipe);
 
 		mvwaddnstr(menu_win, curr_y, 8, buf, x - 9);
-		if(x > 9 && strlen(buf) > x - 9)
+		if(x > 9 && strlen(buf) > (size_t)(x - 9))
 		{
 			mvwaddnstr(menu_win, curr_y + 1, 8, buf + x - 9, x - 9);
 		}
 #else /* #ifdef HAVE_FILE_PROG */
-		if(view->dir_entry[view->list_pos].type == EXECUTABLE)
+		if(entry->type == FT_EXEC)
 			mvwaddstr(menu_win, curr_y, 8, "Executable");
 		else
 			mvwaddstr(menu_win, curr_y, 8, "Regular File");
 #endif /* #ifdef HAVE_FILE_PROG */
 	}
-	else if(view->dir_entry[view->list_pos].type == DIRECTORY)
+	else if(entry->type == FT_DIR)
 	{
-	  mvwaddstr(menu_win, curr_y, 8, "Directory");
+		mvwaddstr(menu_win, curr_y, 8, "Directory");
 	}
 #ifndef _WIN32
-	else if(S_ISCHR(view->dir_entry[view->list_pos].mode))
+	else if(entry->type == FT_CHAR_DEV || entry->type == FT_BLOCK_DEV)
 	{
-	  mvwaddstr(menu_win, curr_y, 8, "Character Device");
+		const char *const type = (entry->type == FT_CHAR_DEV)
+		                       ? "Character Device"
+		                       : "Block Device";
+		char full_path[PATH_MAX];
+		struct stat st;
+
+		mvwaddstr(menu_win, curr_y, 8, type);
+
+		get_current_full_path(view, sizeof(full_path), full_path);
+		if(os_stat(full_path, &st) == 0)
+		{
+			char info[64];
+
+			snprintf(info, sizeof(info), "Device Id: 0x%x:0x%x", major(st.st_rdev),
+					minor(st.st_rdev));
+
+			curr_y += 2;
+			mvwaddstr(menu_win, curr_y, 2, info);
+		}
 	}
-	else if(S_ISBLK(view->dir_entry[view->list_pos].mode))
+	else if(entry->type == FT_SOCK)
 	{
-	  mvwaddstr(menu_win, curr_y, 8, "Block Device");
-	}
-	else if(view->dir_entry[view->list_pos].type == FIFO)
-	{
-	  mvwaddstr(menu_win, curr_y, 8, "Fifo Pipe");
-	}
-	else if(S_ISSOCK(view->dir_entry[view->list_pos].mode))
-	{
-	  mvwaddstr(menu_win, curr_y, 8, "Socket");
+		mvwaddstr(menu_win, curr_y, 8, "Socket");
 	}
 #endif
+	else if(entry->type == FT_FIFO)
+	{
+		mvwaddstr(menu_win, curr_y, 8, "Fifo Pipe");
+	}
 	else
 	{
-	  mvwaddstr(menu_win, curr_y, 8, "Unknown");
+		mvwaddstr(menu_win, curr_y, 8, "Unknown");
 	}
 	curr_y += 2;
 
 	return curr_y - old_curr_y;
 }
 
-/* Returns increment for curr_y */
+/* Returns increment for curr_y. */
 static int
 show_mime_type(FileView *view, int curr_y)
 {
+	char full_path[PATH_MAX];
 	const char* mimetype = NULL;
 
-	mimetype = get_mimetype(get_current_file_name(view));
+	get_current_full_path(view, sizeof(full_path), full_path);
+	mimetype = get_mimetype(full_path);
 
 	mvwaddstr(menu_win, curr_y, 2, "Mime Type: ");
 
@@ -343,6 +368,20 @@ show_mime_type(FileView *view, int curr_y)
 	return 2;
 }
 
+/* Formats single time field as a string.  Writes empty string on error. */
+static void
+format_time(time_t t, char buf[], size_t buf_size)
+{
+	struct tm *const tm = localtime(&t);
+	if(tm == NULL)
+	{
+		copy_str(buf, buf_size, "");
+		return;
+	}
+
+	strftime(buf, buf_size, "%a, %d %b %Y %H:%M:%S", tm);
+}
+
 static void
 cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 {
