@@ -33,19 +33,22 @@
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../menus/menus.h"
+#include "../modes/dialogs/msg_dialog.h"
+#include "../ui/statusbar.h"
+#include "../ui/ui.h"
 #include "../utils/macros.h"
 #include "../utils/utils.h"
 #include "../commands.h"
 #include "../filelist.h"
+#include "../fileview.h"
 #include "../status.h"
-#include "../ui.h"
 #include "cmdline.h"
 #include "modes.h"
 
 static const int SCROLL_GAP = 2;
 
-static int complete_args(int id, const char *args, int argc, char **argv,
-		int arg_pos, void *arg);
+static int complete_args(int id, const cmd_info_t *cmd_info, int arg_pos,
+		void *extra_arg);
 static int swap_range(void);
 static int resolve_mark(char mark);
 static char * menu_expand_macros(const char *str, int for_shell, int *usr1,
@@ -67,6 +70,7 @@ static int can_scroll_menu_down(const menu_info *menu);
 static void change_menu_top(menu_info *const menu, int delta);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info);
+static void update_ui_on_leaving(void);
 static void cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info);
 static int get_effective_menu_scroll_offset(const menu_info *menu);
 static void cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info);
@@ -78,6 +82,7 @@ static void cmd_H(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_L(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_M(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_N(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_b(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_dd(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gf(key_info_t key_info, keys_info_t *keys_info);
 static int pass_combination_to_khandler(const wchar_t keys[]);
@@ -134,6 +139,7 @@ static keys_add_info_t builtin_cmds[] = {
 	{L"N", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_N}}},
 	{L"ZZ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
 	{L"ZQ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
+	{L"b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
 	{L"dd", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_dd}}},
 	{L"gf", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gf}}},
 	{L"gg", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gg}}},
@@ -185,8 +191,7 @@ static cmds_conf_t cmds_conf = {
 };
 
 static int
-complete_args(int id, const char *args, int argc, char **argv, int arg_pos,
-		void *arg)
+complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
 {
 	return 0;
 }
@@ -283,8 +288,8 @@ enter_menu_mode(menu_info *m, FileView *active_view)
 void
 menu_pre(void)
 {
-	touchwin(pos_win);
-	wrefresh(pos_win);
+	touchwin(ruler_win);
+	wrefresh(ruler_win);
 }
 
 void
@@ -292,8 +297,7 @@ menu_post(void)
 {
 	if(curr_stats.need_update != UT_NONE)
 	{
-		touchwin(menu_win);
-		wrefresh(menu_win);
+		menu_redraw();
 		curr_stats.need_update = UT_NONE;
 	}
 	status_bar_message(curr_stats.save_msg ? NULL : "");
@@ -304,14 +308,6 @@ menu_redraw(void)
 {
 	was_redraw = 1;
 	redraw_menu(menu);
-
-	if(curr_stats.errmsg_shown)
-	{
-		redraw_error_msg_window();
-		redrawwin(error_win);
-		wnoutrefresh(error_win);
-		doupdate();
-	}
 }
 
 static void
@@ -324,14 +320,7 @@ leave_menu_mode(void)
 
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 
-	if(was_redraw)
-	{
-		update_screen(UT_FULL);
-	}
-	else
-	{
-		update_all_windows();
-	}
+	update_ui_on_leaving();
 }
 
 static void
@@ -426,10 +415,11 @@ get_last_visible_line(const menu_info *menu)
 	return menu->top + (menu->win_rows - 2) - 1;
 }
 
+/* Redraw TUI. */
 static void
 cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 {
-	redraw_menu(menu);
+	menu_redraw();
 }
 
 static void
@@ -456,10 +446,21 @@ cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info)
 		update_menu();
 	}
 
+	update_ui_on_leaving();
+}
+
+/* Updates UI on leaving the mode trying to minimize efforts to do this. */
+static void
+update_ui_on_leaving(void)
+{
 	if(was_redraw)
+	{
 		update_screen(UT_FULL);
+	}
 	else
+	{
 		update_all_windows();
+	}
 }
 
 static void
@@ -508,7 +509,7 @@ cmd_slash(key_info_t key_info, keys_info_t *keys_info)
 	menu->match_dir = NONE;
 	free(menu->regexp);
 	menu->regexp = NULL;
-	enter_cmdline_mode(MENU_SEARCH_FORWARD_SUBMODE, L"", menu);
+	enter_cmdline_mode(CLS_MENU_FSEARCH, L"", menu);
 }
 
 static void
@@ -517,7 +518,7 @@ cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 	cmds_conf.begin = 1;
 	cmds_conf.current = menu->pos;
 	cmds_conf.end = menu->len;
-	enter_cmdline_mode(MENU_CMD_SUBMODE, L"", menu);
+	enter_cmdline_mode(CLS_MENU_COMMAND, L"", menu);
 }
 
 static void
@@ -527,7 +528,7 @@ cmd_question(key_info_t key_info, keys_info_t *keys_info)
 	last_search_backward = 1;
 	menu->match_dir = NONE;
 	free(menu->regexp);
-	enter_cmdline_mode(MENU_SEARCH_BACKWARD_SUBMODE, L"", menu);
+	enter_cmdline_mode(CLS_MENU_BSEARCH, L"", menu);
 }
 
 static void
@@ -609,12 +610,26 @@ cmd_N(key_info_t key_info, keys_info_t *keys_info)
 		search(!last_search_backward);
 }
 
+/* Populates view with list of files. */
+static void
+cmd_b(key_info_t key_info, keys_info_t *keys_info)
+{
+	if(menu_to_custom_view(menu, view) != 0)
+	{
+		show_error_msg("Menu transformation",
+				"No valid paths discovered in menu content");
+		return;
+	}
+
+	leave_menu_mode();
+}
+
 static void
 cmd_dd(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(pass_combination_to_khandler(L"dd") && menu->len == 0)
 	{
-		show_error_msg("No more items in the menu", "Menu will be closed");
+		show_error_msg("Menu is closing", "No more items in the menu");
 		leave_menu_mode();
 	}
 }
@@ -710,13 +725,14 @@ search(int backward)
 {
 	if(menu->regexp != NULL)
 	{
-		int was_msg;
 		menu->match_dir = backward ? UP : DOWN;
-		was_msg = search_menu_list(NULL, menu);
+		(void)search_menu_list(NULL, menu);
 		wrefresh(menu_win);
 
-		if(!was_msg)
+		if(menu->matching_entries > 0)
+		{
 			status_bar_messagef("%c%s", backward ? '?' : '/', menu->regexp);
+		}
 	}
 	else
 	{
@@ -868,21 +884,21 @@ search_menu_list(const char pattern[], menu_info *m)
 		draw_menu(m);
 	}
 
-	for(i = 0; i < search_repeat; i++)
+	for(i = 0; i < search_repeat; ++i)
+	{
 		switch(m->match_dir)
 		{
 			case NONE:
+			case DOWN:
 				save = search_menu_forwards(m, m->pos + 1);
 				break;
 			case UP:
 				save = search_menu_backwards(m, m->pos - 1);
 				break;
-			case DOWN:
-				save = search_menu_forwards(m, m->pos + 1);
-				break;
 			default:
 				break;
 		}
+	}
 	return save;
 }
 
@@ -929,7 +945,7 @@ search_menu(menu_info *m, int start_pos)
 static int
 search_menu_forwards(menu_info *m, int start_pos)
 {
-	/* FIXME: code duplicatio with search_menu_backwards. */
+	/* FIXME: code duplication with search_menu_backwards. */
 
 	int match_up = -1;
 	int match_down = -1;
@@ -1036,15 +1052,14 @@ search_menu_backwards(menu_info *m, int start_pos)
 		{
 			menu_print_search_msg(m);
 		}
-		return 1;
 	}
-	return 0;
+	return 1;
 }
 
 void
 execute_cmdline_command(const char cmd[])
 {
-	if(exec_command(cmd, curr_view, GET_COMMAND) < 0)
+	if(exec_command(cmd, curr_view, CIT_COMMAND) < 0)
 	{
 		status_bar_error("An error occurred while trying to execute command");
 	}
