@@ -25,13 +25,16 @@
 
 #include "../engine/keys.h"
 #include "../engine/mode.h"
+#include "../ui/statusbar.h"
+#include "../ui/statusline.h"
+#include "../ui/ui.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
-#include "../main_loop.h"
+#include "../event_loop.h"
 #include "../status.h"
-#include "../ui.h"
 #include "dialogs/attr_dialog.h"
 #include "dialogs/change_dialog.h"
+#include "dialogs/msg_dialog.h"
 #include "dialogs/sort_dialog.h"
 #include "cmdline.h"
 #include "file_info.h"
@@ -50,6 +53,7 @@ static int mode_flags[] = {
 	MF_USES_COUNT,                /* CHANGE_MODE */
 	MF_USES_COUNT,                /* VIEW_MODE */
 	0,                            /* FILE_INFO_MODE */
+	0,                            /* MSG_MODE */
 };
 ARRAY_GUARD(mode_flags, MODES_COUNT);
 
@@ -63,6 +67,7 @@ static char uses_input_bar[] = {
 	1, /* CHANGE_MODE */
 	1, /* VIEW_MODE */
 	1, /* FILE_INFO_MODE */
+	0, /* MSG_MODE */
 };
 ARRAY_GUARD(uses_input_bar, MODES_COUNT);
 
@@ -77,6 +82,7 @@ static mode_init_func mode_init_funcs[] = {
 	&init_change_dialog_mode, /* CHANGE_MODE */
 	&init_view_mode,          /* VIEW_MODE */
 	&init_file_info_mode,     /* FILE_INFO_MODE */
+	&init_msg_dialog_mode,    /* MSG_MODE */
 };
 ARRAY_GUARD(mode_init_funcs, MODES_COUNT);
 
@@ -129,6 +135,13 @@ modes_pre(void)
 	}
 }
 
+void
+modes_periodic(void)
+{
+	/* Trigger possible view updates. */
+	view_check_for_updates();
+}
+
 void
 modes_post(void)
 {
@@ -159,7 +172,7 @@ modes_post(void)
 		if(!is_status_bar_multiline())
 		{
 			update_stat_window(curr_view);
-			update_pos_window(curr_view);
+			ui_ruler_update(curr_view);
 		}
 	}
 
@@ -205,7 +218,7 @@ modes_redraw(void)
 		return;
 	}
 
-	if(curr_stats.too_small_term)
+	if(curr_stats.term_state != TS_NORMAL)
 	{
 		update_screen(UT_REDRAW);
 		goto finish;
@@ -216,9 +229,13 @@ modes_redraw(void)
 		redraw_cmdline();
 		goto finish;
 	}
-	else if(vle_mode_is(MENU_MODE))
+	else if(vle_primary_mode_is(MENU_MODE))
 	{
 		menu_redraw();
+		if(vle_mode_is(MSG_MODE))
+		{
+			redraw_msg_dialog(0);
+		}
 		goto finish;
 	}
 	else if(vle_mode_is(FILE_INFO_MODE))
@@ -250,6 +267,10 @@ modes_redraw(void)
 	{
 		view_redraw();
 	}
+	else if(vle_mode_is(MSG_MODE))
+	{
+		redraw_msg_dialog(0);
+	}
 
 finish:
 	if(--in_here > 0)
@@ -320,7 +341,7 @@ clear_input_bar(void)
 int
 is_in_menu_like_mode(void)
 {
-	return ANY(vle_mode_is, MENU_MODE, FILE_INFO_MODE);
+	return ANY(vle_primary_mode_is, MENU_MODE, FILE_INFO_MODE);
 }
 
 void
