@@ -33,6 +33,7 @@ enum
 	CHANGE_MODE,
 	VIEW_MODE,
 	FILE_INFO_MODE,
+	MSG_MODE,
 	MODES_COUNT
 };
 
@@ -40,6 +41,9 @@ void init_modes(void);
 
 void modes_pre(void);
 
+/* Executes poll-based requests for any of the active modes. */
+void modes_periodic(void);
+
 void modes_post(void);
 
 void modes_redraw(void);
