@@ -21,9 +21,13 @@
 #ifdef _WIN32
 #include <windows.h>
 #include <shellapi.h>
+
+#include "utils/utf8.h"
 #endif
 
-#include <sys/stat.h> /* gid_t uid_t lstat() stat() */
+#include <curses.h> /* noraw() raw() */
+
+#include <sys/stat.h> /* gid_t uid_t */
 
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t */
@@ -32,15 +36,19 @@
 #include <string.h> /* strdup() */
 
 #include "cfg/config.h"
+#include "compat/os.h"
+#include "engine/text_buffer.h"
 #include "io/ioeta.h"
 #include "io/iop.h"
 #include "io/ior.h"
-#include "menus/menus.h"
+#include "modes/dialogs/msg_dialog.h"
+#include "ui/cancellation.h"
 #include "utils/fs.h"
 #include "utils/fs_limits.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/path.h"
+#include "utils/str.h"
 #include "utils/utils.h"
 #include "background.h"
 #include "status.h"
@@ -68,6 +76,9 @@ typedef enum
 }
 ConflictAction;
 
+/* Type of function that implements single operation. */
+typedef int (*op_func)(ops_t *ops, void *data, const char *src, const char *dst);
+
 static int op_none(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_remove(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_removesl(ops_t *ops, void *data, const char *src,
@@ -98,48 +109,56 @@ static int op_rmdir(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_mkfile(ops_t *ops, void *data, const char *src, const char *dst);
 static int exec_io_op(ops_t *ops, int (*func)(io_args_t *const),
 		io_args_t *const args);
+static char * err_list_to_str(const ioe_errlst_t *elist);
+static int confirm_overwrite(io_args_t *args, const char src[],
+		const char dst[]);
+static char * pretty_dir_path(const char path[]);
 
-typedef int (*op_func)(ops_t *ops, void *data, const char *src, const char *dst);
-
+/* List of functions that implement operations. */
 static op_func op_funcs[] = {
-	op_none,     /* OP_NONE */
-	op_none,     /* OP_USR */
-	op_remove,   /* OP_REMOVE */
-	op_removesl, /* OP_REMOVESL */
-	op_copy,     /* OP_COPY */
-	op_copyf,    /* OP_COPYF */
-	op_copya,    /* OP_COPYA */
-	op_move,     /* OP_MOVE */
-	op_movef,    /* OP_MOVEF */
-	op_movea,    /* OP_MOVEA */
-	op_move,     /* OP_MOVETMP1 */
-	op_move,     /* OP_MOVETMP2 */
-	op_move,     /* OP_MOVETMP3 */
-	op_move,     /* OP_MOVETMP4 */
-	op_chown,    /* OP_CHOWN */
-	op_chgrp,    /* OP_CHGRP */
+	[OP_NONE]     = &op_none,
+	[OP_USR]      = &op_none,
+	[OP_REMOVE]   = &op_remove,
+	[OP_REMOVESL] = &op_removesl,
+	[OP_COPY]     = &op_copy,
+	[OP_COPYF]    = &op_copyf,
+	[OP_COPYA]    = &op_copya,
+	[OP_MOVE]     = &op_move,
+	[OP_MOVEF]    = &op_movef,
+	[OP_MOVEA]    = &op_movea,
+	[OP_MOVETMP1] = &op_move,
+	[OP_MOVETMP2] = &op_move,
+	[OP_MOVETMP3] = &op_move,
+	[OP_MOVETMP4] = &op_move,
+	[OP_CHOWN]    = &op_chown,
+	[OP_CHGRP]    = &op_chgrp,
 #ifndef _WIN32
-	op_chmod,    /* OP_CHMOD */
-	op_chmodr,   /* OP_CHMODR */
+	[OP_CHMOD]    = &op_chmod,
+	[OP_CHMODR]   = &op_chmodr,
 #else
-	op_addattr,  /* OP_ADDATTR */
-	op_subattr,  /* OP_SUBATTR */
+	[OP_ADDATTR]  = &op_addattr,
+	[OP_SUBATTR]  = &op_subattr,
 #endif
-	op_symlink,  /* OP_SYMLINK */
-	op_symlink,  /* OP_SYMLINK2 */
-	op_mkdir,    /* OP_MKDIR */
-	op_rmdir,    /* OP_RMDIR */
-	op_mkfile,   /* OP_MKFILE */
+	[OP_SYMLINK]  = &op_symlink,
+	[OP_SYMLINK2] = &op_symlink,
+	[OP_MKDIR]    = &op_mkdir,
+	[OP_RMDIR]    = &op_rmdir,
+	[OP_MKFILE]   = &op_mkfile,
 };
 ARRAY_GUARD(op_funcs, OP_COUNT);
 
+/* Operation that is processed at the moment. */
+static ops_t *curr_ops;
+
 ops_t *
-ops_alloc(OPS main_op, const char descr[], const char base_dir[])
+ops_alloc(OPS main_op, const char descr[], const char base_dir[],
+		const char target_dir[])
 {
 	ops_t *const ops = calloc(1, sizeof(*ops));
 	ops->main_op = main_op;
 	ops->descr = descr;
 	ops->base_dir = strdup(base_dir);
+	ops->target_dir = strdup(target_dir);
 	return ops;
 }
 
@@ -196,9 +215,7 @@ ops_enqueue(ops_t *ops, const char src[], const char dst[])
 		}
 	}
 
-	ui_cancellation_enable();
 	ioeta_calculate(ops->estim, src, ops->shallow_eta);
-	ui_cancellation_disable();
 }
 
 void
@@ -222,7 +239,9 @@ ops_free(ops_t *ops)
 	}
 
 	ioeta_free(ops->estim);
+	free(ops->errors);
 	free(ops->base_dir);
+	free(ops->target_dir);
 	free(ops);
 }
 
@@ -244,7 +263,7 @@ op_remove(ops_t *ops, void *data, const char *src, const char *dst)
 {
 	if(cfg.confirm && !curr_stats.confirmed)
 	{
-		curr_stats.confirmed = query_user_menu("Permanent deletion",
+		curr_stats.confirmed = prompt_msg("Permanent deletion",
 				"Are you sure? If you undoing a command and want to see file names, "
 				"use :undolist! command");
 		if(!curr_stats.confirmed)
@@ -278,40 +297,52 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 #else
 		if(is_dir(src))
 		{
-			char buf[PATH_MAX];
+			char path[PATH_MAX];
 			int err;
-			int i;
-			snprintf(buf, sizeof(buf), "%s%c", src, '\0');
-			for(i = 0; buf[i] != '\0'; i++)
-				if(buf[i] == '/')
-					buf[i] = '\\';
-			SHFILEOPSTRUCTA fo = {
+
+			copy_str(path, sizeof(path), src);
+			to_back_slash(path);
+
+			wchar_t *const utf16_path = utf8_to_utf16(path);
+
+			SHFILEOPSTRUCTW fo =
+			{
 				.hwnd = NULL,
 				.wFunc = FO_DELETE,
-				.pFrom = buf,
+				.pFrom = utf16_path,
 				.pTo = NULL,
 				.fFlags = FOF_SILENT | FOF_NOCONFIRMATION | FOF_NOERRORUI,
 			};
-			err = SHFileOperation(&fo);
+			err = SHFileOperationW(&fo);
+
 			log_msg("Error: %d", err);
+			free(utf16_path);
+
 			return err;
 		}
 		else
 		{
 			int ok;
-			DWORD attributes = GetFileAttributesA(src);
+			wchar_t *const utf16_path = utf8_to_utf16(src);
+			DWORD attributes = GetFileAttributesW(utf16_path);
 			if(attributes & FILE_ATTRIBUTE_READONLY)
-				SetFileAttributesA(src, attributes & ~FILE_ATTRIBUTE_READONLY);
-			ok = DeleteFile(src);
+			{
+				SetFileAttributesW(utf16_path, attributes & ~FILE_ATTRIBUTE_READONLY);
+			}
+
+			ok = DeleteFileW(utf16_path);
 			if(!ok)
+			{
 				LOG_WERROR(GetLastError());
+			}
+
+			free(utf16_path);
 			return !ok;
 		}
 #endif
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.path = src,
 
 		.cancellable = data == NULL,
@@ -393,19 +424,22 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 				strcat(cmd, "/Y ");
 			}
 			strcat(cmd, "/E /I /H /R > NUL");
-			ret = system(cmd);
+			ret = os_system(cmd);
 		}
 		else
 		{
-			ret = (CopyFileA(src, dst, 0) == 0);
+			wchar_t *const utf16_src = utf8_to_utf16(src);
+			wchar_t *const utf16_dst = utf8_to_utf16(dst);
+			ret = (CopyFileW(utf16_src, utf16_dst, 0) == 0);
+			free(utf16_dst);
+			free(utf16_src);
 		}
 
 		return ret;
 #endif
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.src = src,
 		.arg2.dst = dst,
 		.arg3.crs = ca_to_crs(conflict_action),
@@ -456,7 +490,7 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 		char cmd[6 + PATH_MAX*2 + 1];
 		const int cancellable = data == NULL;
 
-		if(conflict_action == CA_FAIL && lstat(dst, &st) == 0)
+		if(conflict_action == CA_FAIL && os_lstat(dst, &st) == 0)
 		{
 			return -1;
 		}
@@ -483,7 +517,14 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 			return result;
 		}
 #else
-		BOOL ret = MoveFile(src, dst);
+		wchar_t *const utf16_src = utf8_to_utf16(src);
+		wchar_t *const utf16_dst = utf8_to_utf16(dst);
+
+		BOOL ret = MoveFileW(utf16_src, utf16_dst);
+
+		free(utf16_src);
+		free(utf16_dst);
+
 		if(!ret && GetLastError() == 5)
 		{
 			const int r = op_cp(ops, data, src, dst, conflict_action);
@@ -498,8 +539,7 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 	}
 	else
 	{
-		io_args_t args =
-		{
+		io_args_t args = {
 			.arg1.src = src,
 			.arg2.dst = dst,
 			.arg3.crs = ca_to_crs(conflict_action),
@@ -609,17 +649,21 @@ static int
 op_addattr(ops_t *ops, void *data, const char *src, const char *dst)
 {
 	const DWORD add_mask = (size_t)data;
-	const DWORD attrs = GetFileAttributesA(src);
+	wchar_t *const utf16_path = utf8_to_utf16(src);
+	const DWORD attrs = GetFileAttributesW(utf16_path);
 	if(attrs == INVALID_FILE_ATTRIBUTES)
 	{
+		free(utf16_path);
 		LOG_WERROR(GetLastError());
 		return -1;
 	}
-	if(!SetFileAttributesA(src, attrs | add_mask))
+	if(!SetFileAttributesW(utf16_path, attrs | add_mask))
 	{
+		free(utf16_path);
 		LOG_WERROR(GetLastError());
 		return -1;
 	}
+	free(utf16_path);
 	return 0;
 }
 
@@ -627,17 +671,21 @@ static int
 op_subattr(ops_t *ops, void *data, const char *src, const char *dst)
 {
 	const DWORD sub_mask = (size_t)data;
-	const DWORD attrs = GetFileAttributesA(src);
+	wchar_t *const utf16_path = utf8_to_utf16(src);
+	const DWORD attrs = GetFileAttributesW(utf16_path);
 	if(attrs == INVALID_FILE_ATTRIBUTES)
 	{
+		free(utf16_path);
 		LOG_WERROR(GetLastError());
 		return -1;
 	}
-	if(!SetFileAttributesA(src, attrs & ~sub_mask))
+	if(!SetFileAttributesW(utf16_path, attrs & ~sub_mask))
 	{
+		free(utf16_path);
 		LOG_WERROR(GetLastError());
 		return -1;
 	}
+	free(utf16_path);
 	return 0;
 }
 #endif
@@ -651,7 +699,7 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		char cmd[6 + PATH_MAX*2 + 1];
 		int result;
 #ifdef _WIN32
-		char buf[PATH_MAX + 2];
+		char exe_dir[PATH_MAX + 2];
 #endif
 
 		escaped_src = escape_filename(src, 0);
@@ -668,17 +716,16 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		LOG_INFO_MSG("Running ln command: \"%s\"", cmd);
 		result = background_and_wait_for_errors(cmd, 1);
 #else
-		if(GetModuleFileNameA(NULL, buf, ARRAY_LEN(buf)) == 0)
+		if(get_exe_dir(exe_dir, ARRAY_LEN(exe_dir)) != 0)
 		{
 			free(escaped_dst);
 			free(escaped_src);
 			return -1;
 		}
 
-		*strrchr(buf, '\\') = '\0';
-		snprintf(cmd, sizeof(cmd), "%s\\win_helper -s %s %s", buf, escaped_src,
+		snprintf(cmd, sizeof(cmd), "%s\\win_helper -s %s %s", exe_dir, escaped_src,
 				escaped_dst);
-		result = system(cmd);
+		result = os_system(cmd);
 #endif
 
 		free(escaped_dst);
@@ -686,8 +733,7 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		return result;
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.path = src,
 		.arg2.target = dst,
 		.arg3.crs = IO_CRS_REPLACE_FILES,
@@ -713,7 +759,10 @@ op_mkdir(ops_t *ops, void *data, const char *src, const char *dst)
 #else
 		if(data == NULL)
 		{
-			return CreateDirectory(src, NULL) == 0;
+			wchar_t *const utf16_path = utf8_to_utf16(src);
+			int r = CreateDirectoryW(utf16_path, NULL) == 0;
+			free(utf16_path);
+			return r;
 		}
 		else
 		{
@@ -728,11 +777,14 @@ op_mkdir(ops_t *ops, void *data, const char *src, const char *dst)
 
 				if(!is_dir(src))
 				{
-					if(!CreateDirectory(src, NULL))
+					wchar_t *const utf16_path = utf8_to_utf16(src);
+					if(!CreateDirectoryW(utf16_path, NULL))
 					{
+						free(utf16_path);
 						*p = t;
 						return -1;
 					}
+					free(utf16_path);
 				}
 
 				*p = t;
@@ -745,8 +797,7 @@ op_mkdir(ops_t *ops, void *data, const char *src, const char *dst)
 #endif
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.path = src,
 		.arg2.process_parents = data != NULL,
 		.arg3.mode = 0755,
@@ -769,12 +820,14 @@ op_rmdir(ops_t *ops, void *data, const char *src, const char *dst)
 		LOG_INFO_MSG("Running rmdir command: \"%s\"", cmd);
 		return background_and_wait_for_errors(cmd, 1);
 #else
-		return RemoveDirectory(src) == 0;
+		wchar_t *const utf16_path = utf8_to_utf16(src);
+		const BOOL r = RemoveDirectoryW(utf16_path);
+		free(utf16_path);
+		return r == FALSE;
 #endif
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.path = src,
 	};
 	return exec_io_op(ops, &iop_rmdir, &args);
@@ -797,18 +850,21 @@ op_mkfile(ops_t *ops, void *data, const char *src, const char *dst)
 #else
 		HANDLE hfile;
 
-		hfile = CreateFileA(src, 0, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL,
-				NULL);
+		wchar_t *const utf16_path = utf8_to_utf16(src);
+		hfile = CreateFileW(utf16_path, 0, 0, NULL, CREATE_NEW,
+				FILE_ATTRIBUTE_NORMAL, NULL);
+		free(utf16_path);
 		if(hfile == INVALID_HANDLE_VALUE)
+		{
 			return -1;
+		}
 
 		CloseHandle(hfile);
 		return 0;
 #endif
 	}
 
-	io_args_t args =
-	{
+	io_args_t args = {
 		.arg1.path = src,
 	};
 	return exec_io_op(ops, &iop_mkfile, &args);
@@ -822,21 +878,136 @@ exec_io_op(ops_t *ops, int (*func)(io_args_t *const), io_args_t *const args)
 	int result;
 
 	args->estim = (ops == NULL) ? NULL : ops->estim;
+	args->confirm = &confirm_overwrite;
+
+	if(ops != NULL)
+	{
+		ioe_errlst_init(&args->result.errors);
+	}
 
 	if(args->cancellable)
 	{
 		ui_cancellation_enable();
 	}
 
+	curr_ops = ops;
 	result = func(args);
+	curr_ops = NULL;
 
 	if(args->cancellable)
 	{
 		ui_cancellation_disable();
 	}
 
+	if(ops != NULL)
+	{
+		ops->errors = err_list_to_str(&args->result.errors);
+		ioe_errlst_free(&args->result.errors);
+	}
+
 	return result;
 }
 
+/* Converts list of errors from I/O module into multi-line string.  Returns
+ * newly allocated string. */
+static char *
+err_list_to_str(const ioe_errlst_t *elist)
+{
+	/* TODO: maybe move this to io/ioe unit. */
+
+	size_t i;
+
+	vle_textbuf *str = vle_tb_create();
+	if(str == NULL)
+	{
+		return NULL;
+	}
+
+	for(i = 0U; i < elist->error_count; ++i)
+	{
+		const ioe_err_t *const err = &elist->errors[i];
+		vle_tb_append_linef(str, "%s: %s", replace_home_part(err->path), err->msg);
+	}
+
+	return vle_tb_release(str);
+}
+
+/* Asks user to confirm file overwrite.  Returns non-zero on positive user
+ * answer, otherwise zero is returned. */
+static int
+confirm_overwrite(io_args_t *args, const char src[], const char dst[])
+{
+	/* TODO: think about adding "append" and "rename" options here. */
+	static const response_variant responses[] = {
+		{ .key = 'y', .descr = "[y]es", },
+		{ .key = 'Y', .descr = "[Y]es for all", },
+		{ .key = 'n', .descr = "[n]o", },
+		{ .key = 'N', .descr = "[N]o for all", },
+		{ },
+	};
+
+	char *msg;
+	char response;
+	char *src_dir, *dst_dir;
+	const char *fname = get_last_path_component(dst);
+
+	if(curr_ops->crp != CRP_ASK)
+	{
+		return (curr_ops->crp == CRP_OVERWRITE_ALL) ? 1 : 0;
+	}
+
+	src_dir = pretty_dir_path(src);
+	dst_dir = pretty_dir_path(dst);
+
+	msg = format_str("Overwrite \"%s\" in\n%s\nwith \"%s\" from\n%s\n?", fname,
+			dst_dir, fname, src_dir);
+
+	free(dst_dir);
+	free(src_dir);
+
+	/* Active cancellation conflicts with input processing by putting terminal in
+	 * a cocked mode. */
+	if(args->cancellable)
+	{
+		raw();
+	}
+	response = prompt_msg_custom("File overwrite", msg, responses);
+	if(args->cancellable)
+	{
+		noraw();
+	}
+
+	free(msg);
+
+	switch(response)
+	{
+		case 'Y': curr_ops->crp = CRP_OVERWRITE_ALL; /* Fall through. */
+		case 'y': return 1;
+
+		case 'N': curr_ops->crp = CRP_SKIP_ALL; /* Fall through. */
+		case 'n': return 0;
+
+		default:
+			assert(0 && "Unexpected response.");
+			return 0;
+	}
+}
+
+/* Prepares path to presenting to the user.  Returns newly allocated string,
+ * which should be freed by the caller, or NULL if there is not enough
+ * memory. */
+static char *
+pretty_dir_path(const char path[])
+{
+	char dir_only[strlen(path) + 1];
+	char canonic[PATH_MAX];
+
+	copy_str(dir_only, sizeof(dir_only), path);
+	remove_last_path_component(dir_only);
+	canonicalize_path(dir_only, canonic, sizeof(canonic));
+
+	return strdup(canonic);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
