@@ -21,6 +21,7 @@
 
 #include "io/ioeta.h"
 
+/* Kinds of operations on files. */
 typedef enum
 {
 	OP_NONE,
@@ -55,6 +56,15 @@ typedef enum
 }
 OPS;
 
+/* Policy on treating conflicts during operating processing. */
+typedef enum
+{
+	CRP_ASK,           /* Prompt user for the decision. */
+	CRP_SKIP_ALL,      /* Automatically skip file. */
+	CRP_OVERWRITE_ALL, /* Automatically overwrite file. */
+}
+ConflictResolutionPolicy;
+
 /* Description of file operation on a set of files.  Collects information and
  * helps to keep track of progress. */
 typedef struct
@@ -67,12 +77,22 @@ typedef struct
 	                         also frees it on ops_free(). */
 	const char *descr;    /* Description of operations. */
 	int shallow_eta;      /* Count only top level items, without recursion. */
-	char *base_dir;       /* Base directory in which operation is taking place. */
+	char *errors;         /* Multi-line string of errors. */
+
+	char *base_dir;   /* Base directory in which operation is taking place. */
+	char *target_dir; /* Target directory of the operation (same as base_dir if
+	                     none). */
+
+	/* What should be done on conflicts. */
+	ConflictResolutionPolicy crp;
+
+	/* TODO: count number of skipped files. */
 }
 ops_t;
 
 /* Allocates and initializes new ops_t.  Returns just allocated structure. */
-ops_t * ops_alloc(OPS main_op, const char descr[], const char base_dir[]);
+ops_t * ops_alloc(OPS main_op, const char descr[], const char base_dir[],
+		const char target_dir[]);
 
 /* Describes main operation with one generic word.  Returns the description. */
 const char * ops_describe(const ops_t *ops);
