@@ -21,8 +21,9 @@
 #include <curses.h> /* stdscr wnoutrefresh() */
 
 #include <assert.h> /* assert() */
+#include <ctype.h> /* isdigit() */
 #include <limits.h> /* INT_MIN */
-#include <stddef.h> /* NULL */
+#include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* abs() free() */
 #include <string.h> /* memcpy() memmove() strchr() strdup() strlen() strncat()
@@ -32,6 +33,8 @@
 #include "engine/options.h"
 #include "engine/text_buffer.h"
 #include "modes/view.h"
+#include "ui/statusbar.h"
+#include "ui/ui.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/str.h"
@@ -39,13 +42,13 @@
 #include "utils/utils.h"
 #include "column_view.h"
 #include "filelist.h"
+#include "fileview.h"
 #include "quickview.h"
 #include "search.h"
 #include "sort.h"
 #include "status.h"
 #include "trash.h"
 #include "types.h"
-#include "ui.h"
 #include "viewcolumns_parser.h"
 
 /* TODO: provide default primitive type based handlers (see *prg_handler). */
@@ -74,6 +77,7 @@ optinit_t;
 
 static void init_classify(optval_t *val);
 static void init_cpoptions(optval_t *val);
+static const char * to_endpoint(int i, char buffer[]);
 static void init_timefmt(optval_t *val);
 static void init_trash_dir(optval_t *val);
 static void init_lsview(optval_t *val);
@@ -85,14 +89,15 @@ static void init_sort(optval_t *val);
 static void init_sortorder(optval_t *val);
 static void init_tuioptions(optval_t *val);
 static void init_viewcolumns(optval_t *val);
+static void init_wordchars(optval_t *val);
 static void load_options_defaults(void);
 static void add_options(void);
 static void aproposprg_handler(OPT_OP op, optval_t val);
 static void autochpos_handler(OPT_OP op, optval_t val);
 static void cdpath_handler(OPT_OP op, optval_t val);
+static void chaselinks_handler(OPT_OP op, optval_t val);
 static void classify_handler(OPT_OP op, optval_t val);
-static int str_to_classify(const char str[],
-		char decorations[FILE_TYPE_COUNT][2]);
+static int str_to_classify(const char str[], char decorations[FT_COUNT][2]);
 static const char * pick_out_decoration(const char classify_item[],
 		FileType *type);
 static void columns_handler(OPT_OP op, optval_t val);
@@ -112,9 +117,12 @@ static void hlsearch_handler(OPT_OP op, optval_t val);
 static void iec_handler(OPT_OP op, optval_t val);
 static void ignorecase_handler(OPT_OP op, optval_t val);
 static void incsearch_handler(OPT_OP op, optval_t val);
+static int parse_range(const char range[], int *from, int *to);
+static int parse_endpoint(const char **str, int *endpoint);
 static void laststatus_handler(OPT_OP op, optval_t val);
 static void lines_handler(OPT_OP op, optval_t val);
 static void locateprg_handler(OPT_OP op, optval_t val);
+static void mintimeoutlen_handler(OPT_OP op, optval_t val);
 static void scroll_line_down(FileView *view);
 static void rulerformat_handler(OPT_OP op, optval_t val);
 static void runexec_handler(OPT_OP op, optval_t val);
@@ -138,7 +146,7 @@ static void viewcolumns_handler(OPT_OP op, optval_t val);
 static void set_view_columns_option(FileView *view, const char value[],
 		int update_ui);
 static void add_column(columns_t columns, column_info_t column_info);
-static int map_name(const char *name);
+static int map_name(const char name[]);
 static void resort_view(FileView * view);
 static void statusline_handler(OPT_OP op, optval_t val);
 static void syscalls_handler(OPT_OP op, optval_t val);
@@ -154,6 +162,7 @@ static void vixcmd_handler(OPT_OP op, optval_t val);
 static void vifminfo_handler(OPT_OP op, optval_t val);
 static void vimhelp_handler(OPT_OP op, optval_t val);
 static void wildmenu_handler(OPT_OP op, optval_t val);
+static void wordchars_handler(OPT_OP op, optval_t val);
 static void wrap_handler(OPT_OP op, optval_t val);
 static void text_option_changed(void);
 static void wrapscan_handler(OPT_OP op, optval_t val);
@@ -176,13 +185,14 @@ static const char * sort_enum[] = {
 #ifndef _WIN32
 	"perms",
 #endif
+	"dir",
 	"type",
 };
 ARRAY_GUARD(sort_enum, SK_COUNT);
 
 static const char cpoptions_list[] = "fst";
 static const char * cpoptions_vals = cpoptions_list;
-#define cpoptions_count ARRAY_LEN(cpoptions_list)
+#define cpoptions_count (ARRAY_LEN(cpoptions_list) - 1)
 
 static const char * dotdirs_vals[] = {
 	"rootparent",
@@ -190,14 +200,15 @@ static const char * dotdirs_vals[] = {
 };
 ARRAY_GUARD(dotdirs_vals, NUM_DOT_DIRS);
 
-static const char shortmess_list[] = "T";
+/* Possible flags of 'shortmess' and their count. */
+static const char shortmess_list[] = "Tp";
 static const char *shortmess_vals = shortmess_list;
-#define shortmess_count ARRAY_LEN(shortmess_list)
+#define shortmess_count (ARRAY_LEN(shortmess_list) - 1)
 
 /* Possible flags of 'tuioptions' and their count. */
 static const char tuioptions_list[] = "ps";
 static const char *tuioptions_vals = tuioptions_list;
-#define tuioptions_count ARRAY_LEN(tuioptions_list)
+#define tuioptions_count (ARRAY_LEN(tuioptions_list) - 1)
 
 /* Possible keys of 'fillchars' option. */
 static const char *fillchars_enum[] = {
@@ -223,6 +234,7 @@ static const char *sort_types[] = {
 #ifndef _WIN32
 	"perms", "+perms", "-perms",
 #endif
+	"dir", "+dir", "-dir",
 	"type", "+type", "-type",
 };
 ARRAY_GUARD(sort_types, SK_COUNT*3);
@@ -267,9 +279,8 @@ static struct
 	optinit_t initializer;
 	optval_t val;
 }
-options[] =
-{
-	/* global options */
+options[] = {
+	/* Global options. */
 	{ "aproposprg", "",
 	  OPT_STR, 0, NULL, &aproposprg_handler,
 	  { .ref.str_val = &cfg.apropos_prg },
@@ -282,6 +293,10 @@ options[] =
 	  OPT_STRLIST, 0, NULL, &cdpath_handler,
 	  { .ref.str_val = &cfg.cd_path },
 	},
+	{ "chaselinks", "",
+	  OPT_BOOL, 0, NULL, &chaselinks_handler,
+	  { .ref.bool_val = &cfg.chase_links },
+	},
 	{ "classify", "",
 	  OPT_STRLIST, 0, NULL, &classify_handler,
 	  { .init = &init_classify },
@@ -352,7 +367,7 @@ options[] =
 	},
 	{ "laststatus", "ls",
 	  OPT_BOOL, 0, NULL, &laststatus_handler,
-	  { .ref.bool_val = &cfg.last_status },
+	  { .ref.bool_val = &cfg.display_statusline },
 	},
 	{ "lines", "",
 	  OPT_INT, 0, NULL, &lines_handler,
@@ -362,6 +377,10 @@ options[] =
 	  OPT_STR, 0, NULL, &locateprg_handler,
 	  { .ref.str_val = &cfg.locate_prg },
 	},
+	{ "mintimeoutlen", "",
+	  OPT_INT, 0, NULL, &mintimeoutlen_handler,
+	  { .ref.int_val = &cfg.min_timeout_len },
+	},
 	{ "rulerformat", "ruf",
 	  OPT_STR, 0, NULL, &rulerformat_handler,
 	  { .ref.str_val = &cfg.ruler_format },
@@ -456,6 +475,10 @@ options[] =
 	  OPT_BOOL, 0, NULL, &wildmenu_handler,
 	  { .ref.bool_val = &cfg.wild_menu },
 	},
+	{ "wordchars", "",
+	  OPT_STRLIST, 0, NULL, &wordchars_handler,
+	  { .init = &init_wordchars },
+	},
 	{ "wrap", "",
 	  OPT_BOOL, 0, NULL, &wrap_handler,
 	  { .ref.bool_val = &cfg.wrap_quick_view },
@@ -466,7 +489,6 @@ options[] =
 	},
 
 	/* Local options. */
-
 	{ "lsview", "",
 	  OPT_BOOL, 0, NULL, &lsview_handler,
 	  { .init = &init_lsview },
@@ -522,7 +544,7 @@ classify_to_str(void)
 	size_t len = 0;
 	int filetype;
 	buf[0] = '\0';
-	for(filetype = 0; filetype < FILE_TYPE_COUNT; filetype++)
+	for(filetype = 0; filetype < FT_COUNT; ++filetype)
 	{
 		const char prefix[2] = { cfg.decorations[filetype][DECORATION_PREFIX] };
 		const char suffix[2] = { cfg.decorations[filetype][DECORATION_SUFFIX] };
@@ -556,6 +578,15 @@ init_cpoptions(optval_t *val)
 	val->str_val = buf;
 }
 
+/* Convenience function to format an endpoint inside the buffer, which should be
+ * large enough.  Returns the buffer. */
+static const char *
+to_endpoint(int i, char buffer[])
+{
+	sprintf(buffer, "%d", i);
+	return buffer;
+}
+
 static void
 init_timefmt(optval_t *val)
 {
@@ -574,11 +605,13 @@ init_lsview(optval_t *val)
 	val->bool_val = curr_view->ls_view;
 }
 
+/* Initializes 'shortmess' from current configuration state. */
 static void
 init_shortmess(optval_t *val)
 {
 	static char buf[32];
-	snprintf(buf, sizeof(buf), "%s", cfg.trunc_normal_sb_msgs ? "T" : "");
+	snprintf(buf, sizeof(buf), "%s%s", cfg.trunc_normal_sb_msgs ? "T" : "",
+			cfg.shorten_title_paths ? "p" : "");
 	val->str_val = buf;
 }
 
@@ -633,11 +666,64 @@ init_viewcolumns(optval_t *val)
 	val->str_val = "";
 }
 
+/* Formats 'wordchars' initial value from cfg.word_chars array. */
+static void
+init_wordchars(optval_t *val)
+{
+	static char *str;
+
+	size_t i;
+	size_t len;
+
+	len = 0U;
+	i = 0U;
+	while(i < sizeof(cfg.word_chars))
+	{
+		int l, r;
+
+		while(i < sizeof(cfg.word_chars) && !cfg.word_chars[i])
+		{
+			++i;
+		}
+		l = i;
+		while(i < sizeof(cfg.word_chars) && cfg.word_chars[i])
+		{
+			++i;
+		}
+		r = i - 1;
+
+		if((size_t)l < sizeof(cfg.word_chars))
+		{
+			char left[16];
+			char range[32];
+
+			if(len != 0U)
+			{
+				(void)strappendch(&str, &len, ',');
+			}
+
+			if(r == l)
+			{
+				snprintf(range, sizeof(range), "%s", to_endpoint(l, left));
+			}
+			else
+			{
+				char right[16];
+				snprintf(range, sizeof(range), "%s-%s", to_endpoint(l, left),
+						to_endpoint(r, right));
+			}
+			(void)strappend(&str, &len, range);
+		}
+	}
+
+	val->str_val = str;
+}
+
 static void
 load_options_defaults(void)
 {
 	size_t i;
-	for(i = 0; i < ARRAY_LEN(options); i++)
+	for(i = 0U; i < ARRAY_LEN(options); ++i)
 	{
 		if(options[i].initializer.init != NULL)
 		{
@@ -653,8 +739,8 @@ load_options_defaults(void)
 static void
 add_options(void)
 {
-	int i;
-	for(i = 0; i < ARRAY_LEN(options); i++)
+	size_t i;
+	for(i = 0U; i < ARRAY_LEN(options); ++i)
 	{
 		add_option(options[i].name, options[i].abbr, options[i].type,
 				options[i].val_count, options[i].vals, options[i].handler,
@@ -685,6 +771,17 @@ load_local_options(FileView *view)
 	set_option("relativenumber", val);
 }
 
+void
+clone_local_options(const FileView *from, FileView *to)
+{
+	to->view_columns = from->view_columns;
+	to->num_width = from->num_width;
+	to->num_type = from->num_type;
+
+	memcpy(to->sort, from->sort, sizeof(to->sort));
+	fview_set_lsview(to, from->ls_view);
+}
+
 void
 load_sort_option(FileView *view)
 {
@@ -700,7 +797,7 @@ load_sort_option(FileView *view)
 
 	opt_val[0] = '\0';
 
-	ui_view_sort_list_ensure_well_formed(view->sort);
+	ui_view_sort_list_ensure_well_formed(view);
 
 	/* Produce a string, which represents a list of sorting keys. */
 	i = -1;
@@ -729,19 +826,19 @@ process_set_args(const char *args)
 	int set_options_error;
 	const char *text_buffer;
 
-	text_buffer_clear();
+	vle_tb_clear(vle_err);
 
 	/* Call of set_options() can change error. */
 	error = 0;
 	set_options_error = set_options(args) != 0;
 	error = error || set_options_error;
-	text_buffer = text_buffer_get();
+	text_buffer = vle_tb_get_data(vle_err);
 
 	if(error)
 	{
-		text_buffer_add("Invalid argument for :set command");
+		vle_tb_append_line(vle_err, "Invalid argument for :set command");
 		/* We just modified text buffer and can't use text_buffer variable. */
-		status_bar_error(text_buffer_get());
+		status_bar_error(vle_tb_get_data(vle_err));
 	}
 	else if(text_buffer[0] != '\0')
 	{
@@ -774,17 +871,33 @@ cdpath_handler(OPT_OP op, optval_t val)
 	(void)replace_string(&cfg.cd_path, val.str_val);
 }
 
+/* Whether directory path should always be resolved to real path (no symbolic
+ * links). */
+static void
+chaselinks_handler(OPT_OP op, optval_t val)
+{
+	cfg.chase_links = val.bool_val;
+
+	/* Trigger paths update. */
+	(void)change_directory(other_view, other_view->curr_dir);
+	(void)change_directory(curr_view, curr_view->curr_dir);
+	ui_views_update_titles();
+}
+
 static void
 classify_handler(OPT_OP op, optval_t val)
 {
-	char decorations[FILE_TYPE_COUNT][2] = {};
+	char decorations[FT_COUNT][2] = {};
 
 	if(str_to_classify(val.str_val, decorations) == 0)
 	{
-		assert(sizeof(cfg.decorations) == sizeof(decorations) && "Arrays diverged.");
+		assert(sizeof(cfg.decorations) == sizeof(decorations) && "Arrays diverged");
 		memcpy(&cfg.decorations, &decorations, sizeof(cfg.decorations));
 
-		update_screen(UT_REDRAW);
+		/* 'classify' option affects columns layout, hence views must be reloaded as
+		 * loading list of files performs calculation of filename properties. */
+		ui_view_schedule_reload(curr_view);
+		ui_view_schedule_reload(other_view);
 	}
 	else
 	{
@@ -799,7 +912,7 @@ classify_handler(OPT_OP op, optval_t val)
  * It's assumed that decorations array is zeroed.  Returns zero on success,
  * otherwise non-zero is returned. */
 static int
-str_to_classify(const char str[], char decorations[FILE_TYPE_COUNT][2])
+str_to_classify(const char str[], char decorations[FT_COUNT][2])
 {
 	char *saveptr;
 	char *str_copy;
@@ -820,19 +933,19 @@ str_to_classify(const char str[], char decorations[FILE_TYPE_COUNT][2])
 		const char *suffix = pick_out_decoration(token, &type);
 		if(suffix == NULL)
 		{
-			text_buffer_addf("Invalid filetype: %s", token);
+			vle_tb_append_linef(vle_err, "Invalid filetype: %s", token);
 			error_encountered = 1;
 		}
 		else
 		{
 			if(strlen(token) > 1)
 			{
-				text_buffer_addf("Invalid prefix: %s", token);
+				vle_tb_append_linef(vle_err, "Invalid prefix: %s", token);
 				error_encountered = 1;
 			}
 			if(strlen(suffix) > 1)
 			{
-				text_buffer_addf("Invalid suffix: %s", suffix);
+				vle_tb_append_linef(vle_err, "Invalid suffix: %s", suffix);
 				error_encountered = 1;
 			}
 		}
@@ -854,7 +967,7 @@ static const char *
 pick_out_decoration(const char classify_item[], FileType *type)
 {
 	int filetype;
-	for(filetype = 0; filetype < FILE_TYPE_COUNT; filetype++)
+	for(filetype = 0; filetype < FT_COUNT; ++filetype)
 	{
 		char name[16];
 		char *item_name;
@@ -887,7 +1000,7 @@ columns_handler(OPT_OP op, optval_t val)
 	if(val.int_val < MIN_TERM_WIDTH)
 	{
 		val.int_val = MIN_TERM_WIDTH;
-		text_buffer_addf("At least %d columns needed", MIN_TERM_WIDTH);
+		vle_tb_append_linef(vle_err, "At least %d columns needed", MIN_TERM_WIDTH);
 		error = 1;
 	}
 
@@ -972,7 +1085,8 @@ fillchars_handler(OPT_OP op, optval_t val)
 		else
 		{
 			break_at(part, ':');
-			text_buffer_addf("Unknown key for 'fillchars' option: %s", part);
+			vle_tb_append_linef(vle_err, "Unknown key for 'fillchars' option: %s",
+					part);
 			break;
 		}
 	}
@@ -1022,7 +1136,7 @@ fusehome_handler(OPT_OP op, optval_t val)
 {
 	char *const expanded_path = expand_path(val.str_val);
 	/* Set cfg.fuse_home in the correct way. */
-	if(set_fuse_home(expanded_path) != 0)
+	if(cfg_set_fuse_home(expanded_path) != 0)
 	{
 		/* Reset the 'fusehome' options to its previous value. */
 		val.str_val = cfg.fuse_home;
@@ -1046,7 +1160,7 @@ grepprg_handler(OPT_OP op, optval_t val)
 static void
 history_handler(OPT_OP op, optval_t val)
 {
-	resize_history(MAX(0, val.int_val));
+	cfg_resize_histories(val.int_val);
 }
 
 static void
@@ -1080,11 +1194,77 @@ incsearch_handler(OPT_OP op, optval_t val)
 	cfg.inc_search = val.bool_val;
 }
 
+/* Parses range, which can be shortened to single endpoint if first element
+ * matches last one.  Returns non-zero on error, otherwise zero is returned. */
+static int
+parse_range(const char range[], int *from, int *to)
+{
+	const char *p = range;
+
+	if(parse_endpoint(&p, from) != 0)
+	{
+		vle_tb_append_linef(vle_err, "Wrong range: %s", range);
+		return 1;
+	}
+	if(*p == '-')
+	{
+		++p;
+		if(parse_endpoint(&p, to) != 0)
+		{
+			vle_tb_append_linef(vle_err, "Wrong range: %s", range);
+			return 1;
+		}
+	}
+	else
+	{
+		*to = *from;
+	}
+
+	if(*p != '\0')
+	{
+		vle_tb_append_linef(vle_err, "Wrong range: %s", range);
+		return 1;
+	}
+
+	if(*from > *to)
+	{
+		vle_tb_append_linef(vle_err, "Inversed range: %s", range);
+		return 1;
+	}
+
+	return 0;
+}
+
+/* Parses single endpoint of a range.  It's either a number or a character.
+ * Returns non-zero on error, otherwise zero is returned. */
+static int
+parse_endpoint(const char **str, int *endpoint)
+{
+	if(isdigit(**str))
+	{
+		char *endptr;
+		const long int val = strtol(*str, &endptr, 10);
+		if(val < 0 || val >= 256)
+		{
+			vle_tb_append_linef(vle_err, "Wrong value: %ld", val);
+			return 1;
+		}
+		*str = endptr;
+		*endpoint = val;
+	}
+	else
+	{
+		*endpoint = **str;
+		++*str;
+	}
+	return 0;
+}
+
 static void
 laststatus_handler(OPT_OP op, optval_t val)
 {
-	cfg.last_status = val.bool_val;
-	if(cfg.last_status)
+	cfg.display_statusline = val.bool_val;
+	if(cfg.display_statusline)
 	{
 		if(curr_stats.split == VSPLIT)
 		{
@@ -1114,7 +1294,7 @@ lines_handler(OPT_OP op, optval_t val)
 	if(val.int_val < MIN_TERM_HEIGHT)
 	{
 		val.int_val = MIN_TERM_HEIGHT;
-		text_buffer_addf("At least %d lines needed", MIN_TERM_HEIGHT);
+		vle_tb_append_linef(vle_err, "At least %d lines needed", MIN_TERM_HEIGHT);
 		error = 1;
 	}
 
@@ -1137,6 +1317,22 @@ locateprg_handler(OPT_OP op, optval_t val)
 	(void)replace_string(&cfg.locate_prg, val.str_val);
 }
 
+/* Minimum period on waiting for the input.  Works together with timeoutlen. */
+static void
+mintimeoutlen_handler(OPT_OP op, optval_t val)
+{
+	if(val.int_val <= 0)
+	{
+		vle_tb_append_linef(vle_err, "Argument must be > 0: %d", val.int_val);
+		error = 1;
+		val.int_val = 1;
+		set_option("mintimeoutlen", val);
+		return;
+	}
+
+	cfg.min_timeout_len = val.int_val;
+}
+
 static void
 scroll_line_down(FileView *view)
 {
@@ -1174,7 +1370,7 @@ scrolloff_handler(OPT_OP op, optval_t val)
 {
 	if(val.int_val < 0)
 	{
-		text_buffer_addf("Invalid scroll size: %d", val.int_val);
+		vle_tb_append_linef(vle_err, "Invalid scroll size: %d", val.int_val);
 		error = 1;
 		reset_option_to_default("scrolloff");
 		return;
@@ -1182,7 +1378,9 @@ scrolloff_handler(OPT_OP op, optval_t val)
 
 	cfg.scroll_off = val.int_val;
 	if(cfg.scroll_off > 0)
-		move_to_list_pos(curr_view, curr_view->list_pos);
+	{
+		fview_cursor_redraw(curr_view);
+	}
 }
 
 static void
@@ -1191,12 +1389,15 @@ shell_handler(OPT_OP op, optval_t val)
 	cfg_set_shell(val.str_val);
 }
 
+/* Handles changes of 'shortmess' option by expanding flags into actual option
+ * values. */
 static void
 shortmess_handler(OPT_OP op, optval_t val)
 {
 	const char *p;
 
 	cfg.trunc_normal_sb_msgs = 0;
+	cfg.shorten_title_paths = 0;
 
 	p = val.str_val;
 	while(*p != '\0')
@@ -1205,8 +1406,14 @@ shortmess_handler(OPT_OP op, optval_t val)
 		{
 			cfg.trunc_normal_sb_msgs = 1;
 		}
-		p++;
+		else if(*p == 'p')
+		{
+			cfg.shorten_title_paths = 1;
+		}
+		++p;
 	}
+
+	ui_views_update_titles();
 }
 
 #ifndef _WIN32
@@ -1232,28 +1439,11 @@ sortnumbers_handler(OPT_OP op, optval_t val)
 	redraw_lists();
 }
 
-/* Handles switch that controls columns vs. list like view. */
+/* Handles switch that controls column vs. ls-like view. */
 static void
 lsview_handler(OPT_OP op, optval_t val)
 {
-	curr_view->ls_view = val.bool_val;
-
-	if(curr_view->ls_view)
-	{
-		column_info_t column_info =
-		{
-			.column_id = SK_BY_NAME, .full_width = 0UL, .text_width = 0UL,
-			.align = AT_LEFT,        .sizing = ST_AUTO, .cropping = CT_ELLIPSIS,
-		};
-
-		columns_clear(curr_view->columns);
-		columns_add_column(curr_view->columns, column_info);
-		redraw_current_view();
-	}
-	else
-	{
-		load_view_columns_option(curr_view, curr_view->view_columns);
-	}
+	fview_set_lsview(curr_view, val.bool_val);
 }
 
 /* Handles file numbers displaying toggle. */
@@ -1287,7 +1477,7 @@ relativenumber_handler(OPT_OP op, optval_t val)
 static void
 update_num_type(NumberingType num_type, int enable)
 {
-	const int old_num_type = curr_view->num_type;
+	const NumberingType old_num_type = curr_view->num_type;
 
 	curr_view->num_type = enable
 	                    ? (old_num_type | num_type)
@@ -1299,66 +1489,64 @@ update_num_type(NumberingType num_type, int enable)
 	}
 }
 
+/* Handler for 'sort' option, parses the value and checks it for correctness. */
 static void
 sort_handler(OPT_OP op, optval_t val)
 {
-	char *p = val.str_val - 1;
-	int i;
+	char *part = val.str_val, *state = NULL;
+	int key_count = 0;
+	char *const sort_keys = curr_view->sort;
 
-	i = 0;
-	do
+	while((part = split_and_get(part, ',', &state)) != NULL)
 	{
-		char key[32];
-		char *t;
-		int minus;
+		char *name = part;
+		int reverse = 0;
 		int pos;
-		int j;
+		int key_pos;
 
-		t = p + 1;
-		p = strchr(t, ',');
-		if(p == NULL)
-			p = t + strlen(t);
+		if(*name == '-' || *name == '+')
+		{
+			reverse = (*name == '-');
+			++name;
+		}
 
-		minus = (*t == '-');
-		if(*t == '-' || *t == '+')
-			t++;
-
-		snprintf(key, MIN(p - t + 1, sizeof(key)), "%s", t);
-
-		pos = string_array_pos((char **)sort_enum, ARRAY_LEN(sort_enum), key);
+		pos = string_array_pos((char **)sort_enum, ARRAY_LEN(sort_enum), name);
 		if(pos == -1)
 		{
-			if(key[0] != '\0')
+			if(name[0] != '\0')
 			{
-				text_buffer_addf("Skipped unknown 'sort' value: %s", key);
+				vle_tb_append_linef(vle_err, "Skipped unknown 'sort' value: %s", name);
 				error = 1;
 			}
 			continue;
 		}
-		pos++;
+		++pos;
 
-		for(j = 0; j < i; j++)
-			if(abs(curr_view->sort[j]) == pos)
+		for(key_pos = 0; key_pos < key_count; ++key_pos)
+		{
+			if(abs(sort_keys[key_pos]) == pos)
+			{
 				break;
+			}
+		}
 
-		if(minus)
-			curr_view->sort[j] = -pos;
-		else
-			curr_view->sort[j] = pos;
-		if(j == i)
-			i++;
+		sort_keys[key_pos] = reverse ? -pos : pos;
+
+		if(key_pos == key_count)
+		{
+			++key_count;
+		}
 	}
-	while(*p != '\0');
 
-	if(i < SK_COUNT)
+	if(key_count < SK_COUNT)
 	{
-		curr_view->sort[i] = SK_NONE;
+		sort_keys[key_count] = SK_NONE;
 	}
-	ui_view_sort_list_ensure_well_formed(curr_view->sort);
+	ui_view_sort_list_ensure_well_formed(curr_view);
 
-	reset_view_sort(curr_view);
+	fview_sorting_updated(curr_view);
 	resort_view(curr_view);
-	move_to_list_pos(curr_view, curr_view->list_pos);
+	fview_cursor_redraw(curr_view);
 	load_sort_option(curr_view);
 }
 
@@ -1407,7 +1595,7 @@ set_view_columns_option(FileView *view, const char value[], int update_ui)
 	{
 		optval_t val;
 
-		text_buffer_add("Invalid format of 'viewcolumns' option");
+		vle_tb_append_line(vle_err, "Invalid format of 'viewcolumns' option");
 		error = 1;
 
 		if(update_ui)
@@ -1444,7 +1632,7 @@ add_column(columns_t columns, column_info_t column_info)
 
 /* Maps column name to column id.  Returns column id. */
 static int
-map_name(const char *name)
+map_name(const char name[])
 {
 	if(*name != '\0')
 	{
@@ -1452,7 +1640,7 @@ map_name(const char *name)
 		pos = string_array_pos((char **)sort_enum, ARRAY_LEN(sort_enum), name);
 		return (pos >= 0) ? (pos + 1) : -1;
 	}
-	return get_secondary_key(abs(curr_view->sort[0]));
+	return (int)get_secondary_key((SortingKey)abs(curr_view->sort[0]));
 }
 
 void
@@ -1493,7 +1681,7 @@ tabstop_handler(OPT_OP op, optval_t val)
 {
 	if(val.int_val <= 0)
 	{
-		text_buffer_addf("Argument must be positive: %d", val.int_val);
+		vle_tb_append_linef(vle_err, "Argument must be positive: %d", val.int_val);
 		error = 1;
 		reset_option_to_default("tabstop");
 		return;
@@ -1514,12 +1702,14 @@ timefmt_handler(OPT_OP op, optval_t val)
 	redraw_lists();
 }
 
+/* Maximum period on waiting for the input.  Works together with
+ * mintimeoutlen. */
 static void
 timeoutlen_handler(OPT_OP op, optval_t val)
 {
 	if(val.int_val < 0)
 	{
-		text_buffer_addf("Argument must be >= 0: %d", val.int_val);
+		vle_tb_append_linef(vle_err, "Argument must be >= 0: %d", val.int_val);
 		error = 1;
 		val.int_val = 0;
 		set_option("timeoutlen", val);
@@ -1591,18 +1781,14 @@ static void
 vicmd_handler(OPT_OP op, optval_t val)
 {
 	(void)replace_string(&cfg.vi_command, val.str_val);
-	cfg.vi_cmd_bg = ends_with(cfg.vi_command, "&");
-	if(cfg.vi_cmd_bg)
-		cfg.vi_command[strlen(cfg.vi_command) - 1] = '\0';
+	cfg.vi_cmd_bg = cut_suffix(cfg.vi_command, "&");
 }
 
 static void
 vixcmd_handler(OPT_OP op, optval_t val)
 {
 	(void)replace_string(&cfg.vi_x_command, val.str_val);
-	cfg.vi_x_cmd_bg = ends_with(cfg.vi_x_command, "&");
-	if(cfg.vi_x_cmd_bg)
-		cfg.vi_x_command[strlen(cfg.vi_x_command) - 1] = '\0';
+	cfg.vi_x_cmd_bg = cut_suffix(cfg.vi_x_command, "&");
 }
 
 static void
@@ -1623,6 +1809,37 @@ wildmenu_handler(OPT_OP op, optval_t val)
 	cfg.wild_menu = val.bool_val;
 }
 
+/* Handles setting value of 'wordchars' by parsing list of ranges into character
+ * array. */
+static void
+wordchars_handler(OPT_OP op, optval_t val)
+{
+	char word_chars[256] = { };
+
+	char *new_val = strdup(val.str_val);
+	char *part = new_val, *state = NULL;
+	while((part = split_and_get(part, ',', &state)) != NULL)
+	{
+		int from, to;
+		if(parse_range(part, &from, &to) != 0)
+		{
+			error = 1;
+			break;
+		}
+
+		while(from < to)
+		{
+			word_chars[from++] = 1;
+		}
+	}
+	free(new_val);
+
+	if(part == NULL)
+	{
+		memcpy(&cfg.word_chars, &word_chars, sizeof(cfg.word_chars));
+	}
+}
+
 static void
 wrap_handler(OPT_OP op, optval_t val)
 {
