@@ -20,15 +20,19 @@
 #ifndef VIFM__QUICKVIEW_H__
 #define VIFM__QUICKVIEW_H__
 
-#include "ui.h"
+#include <stdio.h> /* FILE */
 
-void quick_view_file(FileView * view);
+#include "ui/ui.h"
+
+void quick_view_file(FileView *view);
 
 void toggle_quick_view(void);
 
 /* Quits preview pane or view modes. */
 void preview_close(void);
 
+FILE * use_info_prog(const char viewer[]);
+
 #endif /* VIFM__QUICKVIEW_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
