@@ -26,27 +26,28 @@
 #include <curses.h>
 
 #include <signal.h>
-#include <stdio.h> /* fprintf */
+#include <stdio.h> /* fprintf() */
+#include <string.h> /* strsignal() */
 
 #include "cfg/info.h"
+#include "ui/cancellation.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "fuse.h"
 #include "term_title.h"
 
-static void _gnuc_noreturn shutdown_nicely(void);
+static void _gnuc_noreturn shutdown_nicely(int sig, const char descr[]);
 
 #ifndef _WIN32
 
 #include <sys/types.h> /* pid_t */
-#include <sys/wait.h> /* waitpid */
+#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
 
 #include <stdlib.h> /* exit() */
 
 #include "utils/macros.h"
 #include "background.h"
 #include "status.h"
-#include "ui.h"
 
 /* Handle term resizing in X */
 static void
@@ -80,7 +81,12 @@ received_sigchld(void)
 
 	/* This needs to be a loop in case of multiple blocked signals. */
 	while((pid = waitpid(-1, &status, WNOHANG)) > 0)
-		add_finished_job(pid, status);
+	{
+		if(WIFEXITED(status))
+		{
+			add_finished_job(pid, WEXITSTATUS(status));
+		}
+	}
 }
 
 static void
@@ -104,7 +110,7 @@ handle_signal(int sig)
 		case SIGHUP:
 		case SIGQUIT:
 		case SIGTERM:
-			shutdown_nicely();
+			shutdown_nicely(sig, strsignal(sig));
 			break;
 		default:
 			break;
@@ -124,9 +130,13 @@ ctrl_handler(DWORD dwCtrlType)
 			ui_cancellation_request();
 			break;
 		case CTRL_CLOSE_EVENT:
+			shutdown_nicely(dwCtrlType, "Ctrl-C");
+			break;
 		case CTRL_LOGOFF_EVENT:
+			shutdown_nicely(dwCtrlType, "Logoff");
+			break;
 		case CTRL_SHUTDOWN_EVENT:
-			shutdown_nicely();
+			shutdown_nicely(dwCtrlType, "Shutdown");
 			break;
 	}
 
@@ -135,15 +145,15 @@ ctrl_handler(DWORD dwCtrlType)
 #endif
 
 static void _gnuc_noreturn
-shutdown_nicely(void)
+shutdown_nicely(int sig, const char descr[])
 {
   LOG_FUNC_ENTER;
 
 	endwin();
 	set_term_title(NULL);
-	unmount_fuse();
+	fuse_unmount_all();
 	write_info_file();
-	fprintf(stdout, "Vifm killed by signal.\n");
+	fprintf(stdout, "Vifm killed by signal: %d (%s).\n", sig, descr);
 	exit(0);
 }
 
