@@ -22,9 +22,10 @@
 #include <assert.h> /* assert() */
 #include <ctype.h>
 #include <stdlib.h> /* abs() qsort() */
-#include <string.h> /* strrchr() */
+#include <string.h> /* strcmp() strrchr() */
 
 #include "cfg/config.h"
+#include "ui/ui.h"
 #include "utils/fs_limits.h"
 #include "utils/path.h"
 #include "utils/str.h"
@@ -34,11 +35,14 @@
 #include "filelist.h"
 #include "status.h"
 #include "types.h"
-#include "ui.h"
 
+/* View which is being sorted. */
 static FileView* view;
+/* Whether the view displays custom file list. */
+static int custom_view;
 static int sort_descending;
-static int sort_type;
+/* Key used to sort entries in current sorting round. */
+static SortingKey sort_type;
 
 static void sort_by_key(char key);
 static int sort_dir_list(const void *one, const void *two);
@@ -48,15 +52,26 @@ static int vercmp(const char s[], const char t[]);
 #else
 static char * skip_leading_zeros(const char str[]);
 #endif
-static int compare_file_names(int dirs, const char s[], const char t[],
+static int compare_entry_names(const dir_entry_t *a, const dir_entry_t *b,
 		int ignore_case);
+static int compare_full_file_names(const char s[], const char t[],
+		int ignore_case);
+static int compare_file_names(const char s[], const char t[], int ignore_case);
 
 void
 sort_view(FileView *v)
 {
 	int i;
 
+	if(v->sort[0] > SK_LAST)
+	{
+		/* Completely skip sorting if primary key isn't set. */
+		return;
+	}
+
 	view = v;
+	custom_view = flist_custom_active(v);
+
 	i = SK_COUNT;
 	while(--i >= 0)
 	{
@@ -70,9 +85,9 @@ sort_view(FileView *v)
 		sort_by_key(sorting_key);
 	}
 
-	if(!ui_view_sort_list_contains(v->sort, SK_BY_TYPE))
+	if(!ui_view_sort_list_contains(v->sort, SK_BY_DIR))
 	{
-		sort_by_key(SK_BY_TYPE);
+		sort_by_key(SK_BY_DIR);
 	}
 }
 
@@ -83,7 +98,7 @@ sort_by_key(char key)
 	int j;
 
 	sort_descending = (key < 0);
-	sort_type = abs(key);
+	sort_type = (SortingKey)abs(key);
 
 	for(j = 0; j < view->list_rows; j++)
 	{
@@ -163,7 +178,6 @@ sort_dir_list(const void *one, const void *two)
 	dir_entry_t *const second = (dir_entry_t *)two;
 	int first_is_dir;
 	int second_is_dir;
-	int dirs;
 
 	if(is_parent_dir(first->name))
 	{
@@ -177,51 +191,64 @@ sort_dir_list(const void *one, const void *two)
 	first_is_dir = is_directory_entry(first);
 	second_is_dir = is_directory_entry(second);
 
-	dirs = first_is_dir || second_is_dir;
-
 	retval = 0;
 	switch(sort_type)
 	{
 		case SK_BY_NAME:
 		case SK_BY_INAME:
-			if(first->name[0] == '.' && second->name[0] != '.')
-				retval = -1;
-			else if(first->name[0] != '.' && second->name[0] == '.')
-				retval = 1;
+			if(custom_view)
+			{
+				retval = compare_entry_names(first, second, sort_type == SK_BY_INAME);
+			}
 			else
-				retval = compare_file_names(dirs, first->name, second->name,
+			{
+				retval = compare_full_file_names(first->name, second->name,
 						sort_type == SK_BY_INAME);
+			}
 			break;
 
-		case SK_BY_TYPE:
+		case SK_BY_DIR:
 			if(first_is_dir != second_is_dir)
 			{
 				retval = first_is_dir ? -1 : 1;
 			}
 			break;
 
+		case SK_BY_TYPE:
+			retval = strcmp(get_type_str(first->type), get_type_str(second->type));
+			break;
+
 		case SK_BY_EXTENSION:
-			pfirst  = strrchr(first->name,  '.');
+			pfirst = strrchr(first->name,  '.');
 			psecond = strrchr(second->name, '.');
 
 			if(pfirst && psecond)
-				retval = compare_file_names(dirs, ++pfirst, ++psecond, 0);
+				retval = compare_file_names(++pfirst, ++psecond, 0);
 			else if(pfirst || psecond)
 				retval = pfirst ? -1 : 1;
 			else
-				retval = compare_file_names(dirs, first->name, second->name, 0);
+				retval = compare_file_names(first->name, second->name, 0);
 			break;
 
 		case SK_BY_SIZE:
 			{
 				if(first_is_dir)
-					tree_get_data(curr_stats.dirsize_cache, first->name, &first->size);
+				{
+					char full_path[PATH_MAX];
+					get_full_path_of(first, sizeof(full_path), full_path);
+					tree_get_data(curr_stats.dirsize_cache, full_path, &first->size);
+				}
 
 				if(second_is_dir)
-					tree_get_data(curr_stats.dirsize_cache, second->name, &second->size);
+				{
+					char full_path[PATH_MAX];
+					get_full_path_of(second, sizeof(full_path), full_path);
+					tree_get_data(curr_stats.dirsize_cache, full_path, &second->size);
+				}
 
-				retval = (first->size < second->size) ?
-						-1 : (first->size > second->size);
+				retval = (first->size < second->size)
+				       ? -1
+				       : (first->size > second->size);
 			}
 			break;
 
@@ -260,10 +287,6 @@ sort_dir_list(const void *one, const void *two)
 			}
 			break;
 #endif
-
-		default:
-			assert(0 && "All possible sort options should be handled");
-			break;
 	}
 
 	if(retval == 0)
@@ -278,44 +301,74 @@ sort_dir_list(const void *one, const void *two)
 	return retval;
 }
 
-/* Compares two filenames.  Returns positive value if s greater than t, zero if
+/* Compares names of two file entries.  Returns positive value if a is greater
+ * than b, zero if they are equal, otherwise negative value is returned. */
+static int
+compare_entry_names(const dir_entry_t *a, const dir_entry_t *b, int ignore_case)
+{
+	char a_short_path[PATH_MAX];
+	char b_short_path[PATH_MAX];
+
+	get_short_path_of(view, a, 0, sizeof(a_short_path), a_short_path);
+	get_short_path_of(view, b, 0, sizeof(b_short_path), b_short_path);
+
+	return compare_full_file_names(a_short_path, b_short_path, ignore_case);
+}
+
+/* Compares two full filenames and assumes that dot character is smaller than
+ * any other character.  Returns positive value if s is greater than t, zero if
  * they are equal, otherwise negative value is returned. */
 static int
-compare_file_names(int dirs, const char s[], const char t[], int ignore_case)
+compare_full_file_names(const char s[], const char t[], int ignore_case)
 {
+	if(s[0] == '.' && t[0] != '.')
+	{
+		return -1;
+	}
+	else if(s[0] != '.' && t[0] == '.')
+	{
+		return 1;
+	}
+	else
+	{
+		return compare_file_names(s, t, ignore_case);
+	}
+}
+
+/* Compares two file names or their parts (e.g. extensions).  Returns positive
+ * value if s is greater than t, zero if they are equal, otherwise negative
+ * value is returned. */
+static int
+compare_file_names(const char s[], const char t[], int ignore_case)
+{
+	const char *s_val = s, *t_val = t;
 	char s_buf[NAME_MAX];
 	char t_buf[NAME_MAX];
-
-	/* TODO: FIXME: get rid of this when slash is removed from directory names. */
-	if(dirs)
-	{
-		copy_substr(s_buf, sizeof(s_buf), s, '/');
-		s = s_buf;
-
-		copy_substr(t_buf, sizeof(t_buf), t, '/');
-		t = t_buf;
-	}
+	int result;
 
 	if(ignore_case)
 	{
-		if(!dirs)
-		{
-			copy_str(s_buf, sizeof(s_buf), s);
-			s = s_buf;
+		/* Ignore too small buffer errors by not caring about part that didn't
+		 * fit. */
+		(void)str_to_lower(s, s_buf, sizeof(s_buf));
+		(void)str_to_lower(t, t_buf, sizeof(t_buf));
 
-			copy_str(t_buf, sizeof(t_buf), t);
-			t = t_buf;
-		}
-
-		strtolower(s_buf);
-		strtolower(t_buf);
+		s_val = s_buf;
+		t_val = t_buf;
 	}
 
-	return cfg.sort_numbers ? strnumcmp(s, t) : strcmp(s, t);
+	result = cfg.sort_numbers ? strnumcmp(s_val, t_val) : strcmp(s_val, t_val);
+	if(result == 0 && ignore_case)
+	{
+		/* Resort to comparing original names when their normalized versions match
+		 * to always solve ties in deterministic way. */
+		result = strcmp(s, t);
+	}
+	return result;
 }
 
-int
-get_secondary_key(int primary_key)
+SortingKey
+get_secondary_key(SortingKey primary_key)
 {
 	switch(primary_key)
 	{
@@ -327,6 +380,7 @@ get_secondary_key(int primary_key)
 		case SK_BY_MODE:
 		case SK_BY_PERMISSIONS:
 #endif
+		case SK_BY_TYPE:
 		case SK_BY_TIME_MODIFIED:
 		case SK_BY_TIME_ACCESSED:
 		case SK_BY_TIME_CHANGED:
@@ -335,9 +389,11 @@ get_secondary_key(int primary_key)
 		case SK_BY_INAME:
 		case SK_BY_EXTENSION:
 		case SK_BY_SIZE:
-		default:
+		case SK_BY_DIR:
 			return SK_BY_SIZE;
 	}
+	assert(0 && "Unhandled sorting key?");
+	return SK_BY_SIZE;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
