@@ -28,9 +28,11 @@
 
 #include <assert.h> /* assert() */
 #include <limits.h> /* INT_MIN */
+#include <stddef.h> /* NULL */
 #include <string.h>
 
 #include "cfg/config.h"
+#include "ui/ui.h"
 #include "utils/env.h"
 #include "utils/fs_limits.h"
 #include "utils/log.h"
@@ -38,7 +40,9 @@
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/tree.h"
+#include "utils/utils.h"
 #include "colors.h"
+#include "commands_completion.h"
 
 /* Environment variables by which application hosted by terminal multiplexer can
  * identify the host. */
@@ -46,9 +50,8 @@
 #define TMUX_ENVVAR "TMUX"
 
 static void load_def_values(status_t *stats, config_t *config);
+static void determine_fuse_umount_cmd(status_t *stats);
 static void set_gtk_available(status_t *stats);
-static void set_number_of_windows(status_t *stats, config_t *config);
-static void set_env_type(status_t *stats);
 static int reset_dircache(status_t *stats);
 static void set_last_cmdline_command(const char cmd[]);
 
@@ -65,9 +68,9 @@ init_status(config_t *config)
 	inside_tmux = !is_null_or_empty(env_get(TMUX_ENVVAR));
 
 	load_def_values(&curr_stats, config);
+	determine_fuse_umount_cmd(&curr_stats);
 	set_gtk_available(&curr_stats);
-	set_number_of_windows(&curr_stats, config);
-	set_env_type(&curr_stats);
+	curr_stats.exec_env_type = get_exec_env_type();
 	stats_update_shell_type(config->shell);
 
 	return reset_status(config);
@@ -86,19 +89,20 @@ load_def_values(status_t *stats, config_t *config)
 	stats->use_register = 0;
 	stats->curr_register = -1;
 	stats->register_saved = 0;
-	stats->view = 0;
+	stats->number_of_windows = 2;
 	stats->use_input_bar = 1;
-	stats->errmsg_shown = 0;
 	stats->load_stage = 0;
-	stats->too_small_term = 0;
+	stats->term_state = TS_NORMAL;
 	stats->dirsize_cache = NULL_TREE;
 	stats->ch_pos = 1;
 	stats->confirmed = 0;
 	stats->skip_shellout_redraw = 0;
-	stats->cs_base = DCOLOR_BASE;
 	stats->cs = &config->cs;
 	strcpy(stats->color_scheme, "");
 
+	stats->view = 0;
+	stats->graphics_preview = 0;
+
 	stats->msg_head = 0;
 	stats->msg_tail = 0;
 	stats->save_msg_in_list = 1;
@@ -121,15 +125,40 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->shell_type = ST_NORMAL;
 
-	stats->file_picker_mode = 0;
+	stats->fuse_umount_cmd = "";
+
+	stats->original_stdout = NULL;
+
+	stats->chosen_files_out = NULL;
+	stats->chosen_dir_out = NULL;
+	(void)replace_string(&stats->output_delimiter, "\n");
+
+	stats->on_choose = NULL;
+
+	stats->preview_hint = NULL;
 
 #ifdef HAVE_LIBGTK
 	stats->gtk_available = 0;
 #endif
+}
 
-#ifdef _WIN32
-	stats->as_admin = 0;
-#endif
+/* Initializes stats->fuse_umount_cmd field of the stats. */
+static void
+determine_fuse_umount_cmd(status_t *stats)
+{
+	if(external_command_exists("fusermount"))
+	{
+		stats->fuse_umount_cmd = "fusermount -u";
+	}
+	else if(external_command_exists("umount"))
+	{
+		/* Some systems use regular umount command for FUSE. */
+		stats->fuse_umount_cmd = "umount";
+	}
+	else
+	{
+		/* Leave default value. */
+	}
 }
 
 static void
@@ -139,38 +168,7 @@ set_gtk_available(status_t *stats)
 	char *argv[] = { "vifm", NULL };
 	int argc = ARRAY_LEN(argv) - 1;
 	char **ptr = argv;
-	curr_stats.gtk_available = gtk_init_check(&argc, &ptr);
-#endif
-}
-
-static void
-set_number_of_windows(status_t *stats, config_t *config)
-{
-	if(config->show_one_window)
-		curr_stats.number_of_windows = 1;
-	else
-		curr_stats.number_of_windows = 2;
-}
-
-/* Checks if running in X, terminal emulator or linux native console. */
-static void
-set_env_type(status_t *stats)
-{
-#ifndef _WIN32
-	const char *term = env_get("TERM");
-	if(term != NULL && ends_with(term, "linux"))
-	{
-		curr_stats.exec_env_type = EET_LINUX_NATIVE;
-	}
-	else
-	{
-		const char *display = env_get("DISPLAY");
-		curr_stats.exec_env_type = is_null_or_empty(display)
-		                         ? EET_EMULATOR
-		                         : EET_EMULATOR_WITH_X;
-	}
-#else
-	curr_stats.exec_env_type = EET_EMULATOR_WITH_X;
+	stats->gtk_available = gtk_init_check(&argc, &ptr);
 #endif
 }
 
@@ -201,7 +199,7 @@ schedule_redraw(void)
 }
 
 int
-is_redraw_scheduled(void)
+fetch_redraw_scheduled(void)
 {
 	if(pending_redraw)
 	{
@@ -257,22 +255,53 @@ set_last_cmdline_command(const char cmd[])
 void
 stats_update_shell_type(const char shell_cmd[])
 {
-#ifdef _WIN32
-	char shell[NAME_MAX];
-	const char *shell_name;
+	curr_stats.shell_type = get_shell_type(shell_cmd);
+}
 
-	(void)extract_cmd_name(shell_cmd, 0, sizeof(shell), shell);
-	shell_name = get_last_path_component(shell);
+TermState
+stats_update_term_state(int screen_x, int screen_y)
+{
+	if(screen_x < MIN_TERM_WIDTH || screen_y < MIN_TERM_HEIGHT)
+	{
+		curr_stats.term_state = TS_TOO_SMALL;
+	}
+	else if(curr_stats.term_state != TS_NORMAL)
+	{
+		curr_stats.term_state = TS_BACK_TO_NORMAL;
+	}
 
-	if(stroscmp(shell_name, "cmd") == 0 || stroscmp(shell_name, "cmd.exe") == 0)
-	{
-		curr_stats.shell_type = ST_CMD;
-	}
-	else
-#endif
-	{
-		curr_stats.shell_type = ST_NORMAL;
-	}
+	return curr_stats.term_state;
+}
+
+void
+stats_set_chosen_files_out(const char output[])
+{
+	(void)replace_string(&curr_stats.chosen_files_out, output);
+}
+
+void
+stats_set_chosen_dir_out(const char output[])
+{
+	(void)replace_string(&curr_stats.chosen_dir_out, output);
+}
+
+void
+stats_set_output_delimiter(const char delimiter[])
+{
+	(void)replace_string(&curr_stats.output_delimiter, delimiter);
+}
+
+void
+stats_set_on_choose(const char command[])
+{
+	(void)replace_string(&curr_stats.on_choose, command);
+}
+
+int
+stats_file_choose_action_set(void)
+{
+	return !is_null_or_empty(curr_stats.chosen_files_out)
+	    || !is_null_or_empty(curr_stats.on_choose);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
