@@ -20,6 +20,8 @@
 #ifndef VIFM__STATUS_H__
 #define VIFM__STATUS_H__
 
+#include <stdio.h> /* FILE */
+
 #include "utils/tree.h"
 #include "utils/fs_limits.h"
 
@@ -68,6 +70,15 @@ typedef enum
 }
 UpdateType;
 
+/* Possible states of terminal with regard to its size. */
+typedef enum
+{
+	TS_NORMAL,         /* OK to draw UI. */
+	TS_TOO_SMALL,      /* Too small terminal. */
+	TS_BACK_TO_NORMAL, /* Was too small some moments ago, need to restore UI. */
+}
+TermState;
+
 typedef enum
 {
 	/* Shell that is aware of command escaping and backslashes in paths. */
@@ -87,13 +98,14 @@ typedef struct
 	int curr_register;
 	int register_saved;
 	int number_of_windows;
-	int view; /* Shows whether preview mode is activated. */
 	int skip_history;
 	int load_stage; /* 0 - no TUI, 1 - part of TUI, 2 - TUI, 3 - all */
 
-	int errmsg_shown; /* 0 - none, 1 - error, 2 - query */
+	int view;             /* Whether preview mode is active. */
+	int graphics_preview; /* Whether current preview displays graphics. */
 
-	int too_small_term;
+	/* Describes terminal state with regard to its dimensions. */
+	TermState term_state;
 
 	tree_t dirsize_cache; /* ga command results */
 
@@ -106,7 +118,6 @@ typedef struct
 	/* Whether to skip complete UI redraw after returning from a shellout. */
 	int skip_shellout_redraw;
 
-	int cs_base;
 	col_scheme_t *cs;
 	char color_scheme[NAME_MAX];
 
@@ -134,20 +145,26 @@ typedef struct
 	 * (e.g. right after startup or :restart command). */
 	char *last_cmdline_command;
 
-	int initial_lines; /* Initial terminal height in lines. */
+	int initial_lines;   /* Initial terminal height in lines. */
 	int initial_columns; /* Initial terminal width in characters. */
 
 	ShellType shell_type; /* Specifies type of shell. */
 
-	int file_picker_mode; /* Whether vifm was started in file picking mode. */
+	const char *fuse_umount_cmd; /* Command to use for fuse unmounting. */
+
+	FILE *original_stdout; /* Saved original standard output. */
+
+	char *chosen_files_out; /* Destination for writing chosen files. */
+	char *chosen_dir_out;   /* Destination for writing chosen directory. */
+	char *output_delimiter; /* Delimiter for writing out list of paths. */
+
+	char *on_choose; /* Command to execute on picking files. */
+
+	void *preview_hint; /* Hint on which view is used for preview. */
 
 #ifdef HAVE_LIBGTK
 	int gtk_available; /* for mimetype detection */
 #endif
-
-#ifdef _WIN32
-	int as_admin;
-#endif
 }
 status_t;
 
@@ -160,12 +177,12 @@ int init_status(struct config_t *config);
  * Returns non-zero on error. */
 int reset_status(const struct config_t *config);
 
-/* Sets internal flag to schedule postponed redraw operation. */
+/* Sets internal flag to schedule postponed redraw operation of the UI. */
 void schedule_redraw(void);
 
-/* Checks for postponed redraw operations. Returns non-zero if redraw operation
- * was scheduled and resets internal flag. */
-int is_redraw_scheduled(void);
+/* Checks for postponed redraw operations of the UI.  Has side effects.  Returns
+ * non-zero if redraw operation was scheduled and resets internal flag. */
+int fetch_redraw_scheduled(void);
 
 /* Updates curr_stats to reflect whether terminal multiplexers support is
  * enabled. */
@@ -177,6 +194,30 @@ void update_last_cmdline_command(const char cmd[]);
 /* Updates curr_stats.shell_type field according to passed shell command. */
 void stats_update_shell_type(const char shell_cmd[]);
 
+/* Updates curr_stats.term_state field according to specified terminal
+ * dimensions.  Returns new state. */
+TermState stats_update_term_state(int screen_x, int screen_y);
+
+/* Sets output location (curr_stats.chosen_files_out) for list of chosen
+ * files. */
+void stats_set_chosen_files_out(const char output[]);
+
+/* Sets output location (curr_stats.chosen_dir_out) for last visited
+ * directory. */
+void stats_set_chosen_dir_out(const char output[]);
+
+/* Sets delimiter (curr_stats.output_delimiter) for separating multiple paths in
+ * output. */
+void stats_set_output_delimiter(const char delimiter[]);
+
+/* Sets command to run on file selection right before exiting
+ * exit (curr_stats.on_choose). */
+void stats_set_on_choose(const char command[]);
+
+/* Checks whether custom actions on file choosing is set.  Returns non-zero if
+ * so, otherwise zero is returned. */
+int stats_file_choose_action_set(void);
+
 #endif /* VIFM__STATUS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
