@@ -4,12 +4,14 @@ const char *tags[] = {
 	"vifm-$HOME",
 	"vifm-$MYVIFMRC",
 	"vifm-$VIFM",
+	"vifm-$VIFM_FUSE_FILE",
 	"vifm-%",
 	"vifm-%C",
 	"vifm-%D",
 	"vifm-%F",
 	"vifm-%M",
 	"vifm-%S",
+	"vifm-%U",
 	"vifm-%a",
 	"vifm-%b",
 	"vifm-%c",
@@ -18,14 +20,20 @@ const char *tags[] = {
 	"vifm-%i",
 	"vifm-%m",
 	"vifm-%n",
+	"vifm-%ph",
+	"vifm-%pw",
+	"vifm-%px",
+	"vifm-%py",
 	"vifm-%r",
 	"vifm-%s",
+	"vifm-%u",
 	"vifm-'",
 	"vifm-'aproposprg'",
 	"vifm-'autochpos'",
 	"vifm-'cd'",
 	"vifm-'cdpath'",
 	"vifm-'cf'",
+	"vifm-'chaselinks'",
 	"vifm-'classify'",
 	"vifm-'co'",
 	"vifm-'columns'",
@@ -39,6 +47,7 @@ const char *tags[] = {
 	"vifm-'findprg'",
 	"vifm-'followlinks'",
 	"vifm-'fusehome'",
+	"vifm-'gd'",
 	"vifm-'gdefault'",
 	"vifm-'grepprg'",
 	"vifm-'hi'",
@@ -55,6 +64,7 @@ const char *tags[] = {
 	"vifm-'locateprg'",
 	"vifm-'ls'",
 	"vifm-'lsview'",
+	"vifm-'mintimeoutlen'",
 	"vifm-'nu'",
 	"vifm-'number'",
 	"vifm-'numberwidth'",
@@ -89,7 +99,7 @@ const char *tags[] = {
 	"vifm-'trash'",
 	"vifm-'trashdir'",
 	"vifm-'ts'",
-	"vifm-'tuioptions",
+	"vifm-'tuioptions'",
 	"vifm-'ul'",
 	"vifm-'undolevels'",
 	"vifm-'vicmd'",
@@ -99,6 +109,7 @@ const char *tags[] = {
 	"vifm-'vixcmd'",
 	"vifm-'wildmenu'",
 	"vifm-'wmnu'",
+	"vifm-'wordchars'",
 	"vifm-'wrap'",
 	"vifm-'wrapscan'",
 	"vifm-'ws'",
@@ -106,9 +117,13 @@ const char *tags[] = {
 	"vifm-)",
 	"vifm-,",
 	"vifm--+c",
+	"vifm---choose-dir",
+	"vifm---choose-files",
+	"vifm---delimiter",
 	"vifm---help",
 	"vifm---logging",
 	"vifm---no-configs",
+	"vifm---on-choose",
 	"vifm---remote",
 	"vifm---select",
 	"vifm---version",
@@ -126,6 +141,8 @@ const char *tags[] = {
 	"vifm-:apropos",
 	"vifm-:bar",
 	"vifm-:c",
+	"vifm-:ca",
+	"vifm-:cabbrev",
 	"vifm-:cd",
 	"vifm-:change",
 	"vifm-:chmod",
@@ -134,6 +151,8 @@ const char *tags[] = {
 	"vifm-:cm",
 	"vifm-:cmap",
 	"vifm-:cno",
+	"vifm-:cnorea",
+	"vifm-:cnoreabbrev",
 	"vifm-:cnoremap",
 	"vifm-:co",
 	"vifm-:colo",
@@ -143,7 +162,11 @@ const char *tags[] = {
 	"vifm-:comclear",
 	"vifm-:command",
 	"vifm-:copy",
+	"vifm-:cq",
+	"vifm-:cquit",
 	"vifm-:cu",
+	"vifm-:cuna",
+	"vifm-:cunabbrev",
 	"vifm-:cunmap",
 	"vifm-:d",
 	"vifm-:delc",
@@ -237,6 +260,8 @@ const char *tags[] = {
 	"vifm-:qun",
 	"vifm-:qunmap",
 	"vifm-:range",
+	"vifm-:redr",
+	"vifm-:redraw",
 	"vifm-:reg",
 	"vifm-:registers",
 	"vifm-:rename",
@@ -382,6 +407,7 @@ const char *tags[] = {
 	"vifm-c_CTRL-T",
 	"vifm-c_CTRL-U",
 	"vifm-c_CTRL-W",
+	"vifm-c_CTRL-X_/",
 	"vifm-c_CTRL-X_=",
 	"vifm-c_CTRL-X_CTRL-X_c",
 	"vifm-c_CTRL-X_CTRL-X_d",
@@ -395,6 +421,7 @@ const char *tags[] = {
 	"vifm-c_CTRL-X_m",
 	"vifm-c_CTRL-X_r",
 	"vifm-c_CTRL-X_t",
+	"vifm-c_CTRL-]",
 	"vifm-c_CTRL-_",
 	"vifm-c_Delete",
 	"vifm-c_Down",
@@ -426,6 +453,7 @@ const char *tags[] = {
 	"vifm-cpo-f",
 	"vifm-cpo-s",
 	"vifm-cpo-t",
+	"vifm-custom-views",
 	"vifm-cw",
 	"vifm-d",
 	"vifm-dd",
@@ -511,6 +539,7 @@ const char *tags[] = {
 	"vifm-m_N",
 	"vifm-m_ZQ",
 	"vifm-m_ZZ",
+	"vifm-m_b",
 	"vifm-m_e",
 	"vifm-m_gf",
 	"vifm-m_gg",
@@ -518,6 +547,7 @@ const char *tags[] = {
 	"vifm-m_k",
 	"vifm-m_l",
 	"vifm-m_n",
+	"vifm-m_q",
 	"vifm-m_zH",
 	"vifm-m_zL",
 	"vifm-m_zb",
@@ -532,6 +562,7 @@ const char *tags[] = {
 	"vifm-normal",
 	"vifm-options",
 	"vifm-p",
+	"vifm-patterns",
 	"vifm-plugin",
 	"vifm-q/",
 	"vifm-q:",
@@ -557,32 +588,9 @@ const char *tags[] = {
 	"vifm-q_CTRL-R",
 	"vifm-q_CTRL-U",
 	"vifm-q_CTRL-V",
-	"vifm-q_CTRL-W_+",
-	"vifm-q_CTRL-W_-",
-	"vifm-q_CTRL-W_<",
-	"vifm-q_CTRL-W_=",
-	"vifm-q_CTRL-W_>",
-	"vifm-q_CTRL-W_H",
-	"vifm-q_CTRL-W_J",
-	"vifm-q_CTRL-W_K",
-	"vifm-q_CTRL-W_L",
-	"vifm-q_CTRL-W__",
-	"vifm-q_CTRL-W_b",
-	"vifm-q_CTRL-W_bar",
-	"vifm-q_CTRL-W_h",
-	"vifm-q_CTRL-W_j",
-	"vifm-q_CTRL-W_k",
-	"vifm-q_CTRL-W_l",
-	"vifm-q_CTRL-W_o",
-	"vifm-q_CTRL-W_p",
-	"vifm-q_CTRL-W_s",
-	"vifm-q_CTRL-W_t",
-	"vifm-q_CTRL-W_v",
-	"vifm-q_CTRL-W_w",
-	"vifm-q_CTRL-W_x",
-	"vifm-q_CTRL-W_z",
 	"vifm-q_CTRL-Y",
 	"vifm-q_Enter",
+	"vifm-q_F",
 	"vifm-q_G",
 	"vifm-q_N",
 	"vifm-q_Q",
@@ -616,6 +624,7 @@ const char *tags[] = {
 	"vifm-selectors",
 	"vifm-set-options",
 	"vifm-startup",
+	"vifm-system()",
 	"vifm-t",
 	"vifm-to-p",
 	"vifm-to-s",
@@ -649,6 +658,7 @@ const char *tags[] = {
 	"vifm-zR",
 	"vifm-za",
 	"vifm-zb",
+	"vifm-zd",
 	"vifm-zf",
 	"vifm-zm",
 	"vifm-zo",
