@@ -20,7 +20,7 @@
 #ifndef VIFM__UTILS__UTILS_WIN_H__
 #define VIFM__UTILS__UTILS_WIN_H__
 
-#include <windef.h>
+#include <windows.h>
 
 #include <stddef.h> /* size_t */
 #include <wchar.h> /* wchar_t */
@@ -42,8 +42,6 @@
 
 int wcwidth(wchar_t c);
 
-int wcswidth(const wchar_t str[], size_t max_len);
-
 /* Executes a command (cmd) using CreateProcess() API function.  Expects path
  * that contain spaces to be enclosed in double quotes.  On internal error
  * returns last error code and sets *returned_exit_code to zero, otherwise sets
@@ -65,6 +63,15 @@ const char * attr_str_long(DWORD attr);
 /* Returns pointer to a statically allocated buffer. */
 const char * escape_for_cd(const char str[]);
 
+/* Resolves path to its destination.  Returns pointer to a statically allocated
+ * buffer. */
+const char * win_resolve_mount_points(const char path[]);
+
+/* Checks whether contents of working directory of the view was externally
+ * modified.  Returns negative value on error, zero for unchanged directory and
+ * positive number if directory was modified. */
+int win_check_dir_changed(FileView *view);
+
 #endif /* VIFM__UTILS__UTILS_WIN_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
