@@ -20,21 +20,30 @@
 #define VIFM__VIFM_H__
 
 #include "utils/macros.h"
-#include "ui.h"
+#include "ui/ui.h"
+
+#ifndef _WIN32
+#define CONF_DIR "~/.vifm"
+#else
+#define CONF_DIR "(%HOME%/.vifm or %APPDATA%/Vifm)"
+#endif
 
 /* Resets internal state and reloads configuration files. */
 void vifm_restart(void);
 
 /* Tries to quit fully initialized vifm.  Might fail if background tasks are
  * present and user chooses not to stop them. */
-void vifm_try_leave(int write_info, int force);
+void vifm_try_leave(int write_info, int cquit, int force);
 
-/* Leaves vifm storing list of file names (taken from the view or the files) in
- * a special file for use by an external application. */
-void _gnuc_noreturn vifm_return_file_list(const FileView *view, int nfiles,
+/* Communicates chosen files to something external. */
+void _gnuc_noreturn vifm_choose_files(const FileView *view, int nfiles,
 		char *files[]);
 
-#endif // VIFM__VIFM_H__
+/* Quits vifm with error after deinitializing ncurses, saving state to vifminfo
+ * and displaying the message. */
+void _gnuc_noreturn vifm_finish(const char message[]);
+
+#endif /* VIFM__VIFM_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
