@@ -26,11 +26,12 @@
 #include <unistd.h>
 
 #include <ctype.h>
+#include <dirent.h> /* DIR dirent opendir() readdir() closedir() */
 #include <errno.h> /* ENOENT errno */
 #include <locale.h> /* setlocale() */
 #include <stddef.h> /* size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h>
+#include <stdlib.h> /* EXIT_FAILURE exit() */
 #include <string.h> /* strcpy() */
 
 #include "utils/fs_limits.h"
@@ -168,13 +169,14 @@ char *HI_GROUPS[] =
 	[TOP_LINE_COLOR]     = "TopLine",
 	[TOP_LINE_SEL_COLOR] = "TopLineSel",
 	[STATUS_LINE_COLOR]  = "StatusLine",
-	[MENU_COLOR]         = "WildMenu",
+	[WILD_MENU_COLOR]    = "WildMenu",
 	[CMD_LINE_COLOR]     = "CmdLine",
 	[ERROR_MSG_COLOR]    = "ErrorMsg",
 	[BORDER_COLOR]       = "Border",
 	[OTHER_LINE_COLOR]   = "OtherLine",
+	[JOB_LINE_COLOR]     = "JobLine",
 };
-ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR - 2);
+ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR);
 
 char *COLOR_NAMES[8] = {
 	[COLOR_BLACK]   = "black",
@@ -202,22 +204,30 @@ static const int default_colors[][3] = {
 	[TOP_LINE_COLOR]     = { COLOR_BLACK,   COLOR_WHITE, 0                       },
 	[TOP_LINE_SEL_COLOR] = { COLOR_BLACK,   -1,          A_BOLD                  },
 	[STATUS_LINE_COLOR]  = { COLOR_BLACK,   COLOR_WHITE, A_BOLD                  },
-	[MENU_COLOR]         = { COLOR_WHITE,   COLOR_BLACK, A_UNDERLINE | A_REVERSE },
+	[WILD_MENU_COLOR]    = { COLOR_WHITE,   COLOR_BLACK, A_UNDERLINE | A_REVERSE },
 	[CMD_LINE_COLOR]     = { COLOR_WHITE,   COLOR_BLACK, 0                       },
 	[ERROR_MSG_COLOR]    = { COLOR_RED,     COLOR_BLACK, 0                       },
 	[BORDER_COLOR]       = { COLOR_BLACK,   COLOR_WHITE, 0                       },
 	[OTHER_LINE_COLOR]   = { -1,            -1,          -1                      },
+	[JOB_LINE_COLOR]     = { COLOR_BLACK,   COLOR_WHITE, A_BOLD | A_REVERSE      },
 };
-ARRAY_GUARD(default_colors, MAXNUM_COLOR - 2);
+ARRAY_GUARD(default_colors, MAXNUM_COLOR);
 
 static struct
 {
 	int count;
 	col_scheme_t array[MAX_COLOR_SCHEMES];
-}cs = {
+} cs = {
 	.count = 0,
 };
 
+/* Help message. */
+static const char *const USAGE =
+	"Usage: vifmrc-converter 0|1|2|3 [vifmrc_file vifminfo_file]\n\n"
+	"1 means comment commands in vifmrc and put more things to vifminfo\n"
+	"2 means convert colorscheme file only\n"
+	"3 means rename colorschemes only (add .vifm extension)";
+
 #ifdef _WIN32
 static int is_dir(const char *file);
 #endif
@@ -240,6 +250,8 @@ static void add_color(char s1[], char s2[], char s3[]);
 static int colname2int(char col[]);
 static void write_color_schemes(const char *colors_dir);
 static const char * conv_attrs_to_str(int attrs);
+static void rename_color_schemes(const char colors_dir[]);
+static int is_regular_file(const char path[]);
 
 static void
 chomp(char *text)
@@ -264,13 +276,13 @@ read_config_file(const char *config_file)
 	char *s2 = NULL;
 	char *s3 = NULL;
 	char *s4 = NULL;
-	char *sx = NULL;
 
 	if((fp = fopen(config_file, "r")) == NULL)
 		return 0;
 
 	while(fgets(line, sizeof(line), fp))
 	{
+		char *sx;
 		int args;
 
 		if(line[0] == '#')
@@ -294,7 +306,12 @@ read_config_file(const char *config_file)
 		}
 		/* COMMAND is handled here so that the command can have an '=' */
 		if(!strcmp(line, "COMMAND"))
-			add_command(s1, s2);
+		{
+			if(args == 2)
+			{
+				add_command(s1, s2);
+			}
+		}
 		else
 		{
 			if(args == 2 && (sx = s3 = strchr(s2, '=')) != NULL)
@@ -496,7 +513,7 @@ main(int argc, char **argv)
 	char startup_file[PATH_MAX];
 	char vifminfo_file[PATH_MAX], vifminfo_file_bak[PATH_MAX];
 	char colorschemes_file[PATH_MAX], colors_dir[PATH_MAX];
-	int vifm_like;
+	int mode;
 	char *vifmrc_arg, *vifminfo_arg;
 	int err;
 
@@ -504,21 +521,17 @@ main(int argc, char **argv)
 
 	if(argc != 2 && argc != 4)
 	{
-		puts("Usage: vifmrc-converter 0|1|2 [vifmrc_file vifminfo_file]\n\n"
-				"1 means comment commands in vifmrc and put more things to vifminfo\n"
-				"2 means convert colorscheme file only");
+		puts(USAGE);
 		return 1;
 	}
 	if(argv[1][1] != '\0' || (argv[1][0] != '0' && argv[1][0] != '1' &&
-			argv[1][0] != '2'))
+			argv[1][0] != '2' && argv[1][0] != '3'))
 	{
-		puts("Usage: vifmrc-converter 0|1|2 [vifmrc_file vifminfo_file]\n\n"
-				"1 means comment commands in vifmrc and put more things to vifminfo\n"
-				"2 means convert colorscheme file only");
+		puts(USAGE);
 		return 1;
 	}
 
-	vifm_like = atoi(argv[1]);
+	mode = atoi(argv[1]);
 
 	home_dir = getenv("HOME");
 #ifdef _WIN32
@@ -557,12 +570,18 @@ main(int argc, char **argv)
 	change_slashes(colors_dir);
 #endif
 
-	if(vifm_like == 2)
+	if(mode == 2)
 	{
 		convert_color_schemes(colorschemes_file, colors_dir);
 		return 0;
 	}
 
+	if(mode == 3)
+	{
+		rename_color_schemes(colors_dir);
+		return 0;
+	}
+
 	if(argc == 4)
 	{
 		vifmrc_arg = argv[2];
@@ -574,8 +593,10 @@ main(int argc, char **argv)
 		vifminfo_arg = vifminfo_file;
 	}
 
-	if(!vifm_like)
+	if(!mode)
+	{
 		vifminfo = VIFMINFO_BOOKMARKS;
+	}
 
 	read_config_file(config_file);
 	read_config_file(vifminfo_file);
@@ -598,12 +619,12 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	write_vifmrc(vifmrc_arg, vifm_like);
-	append_vifmrc(vifmrc_arg, vifm_like);
-	append_vifminfo_option(vifmrc_arg, vifm_like);
+	write_vifmrc(vifmrc_arg, mode);
+	append_vifmrc(vifmrc_arg, mode);
+	append_vifminfo_option(vifmrc_arg, mode);
 	append_startup(vifmrc_arg, startup_file);
 
-	write_vifminfo(vifminfo_arg, vifm_like);
+	write_vifminfo(vifminfo_arg, mode);
 
 	convert_color_schemes(colorschemes_file, colors_dir);
 
@@ -627,10 +648,17 @@ is_dir(const char *file)
 static void
 add_command(const char *name, const char *cmd)
 {
-	commands = realloc(commands, sizeof(command)*(ncommands + 1));
+	void *p = realloc(commands, sizeof(command)*(ncommands + 1));
+	if(p == NULL)
+	{
+		fprintf(stderr, "%s\n", "Memory allocation error.");
+		exit(EXIT_FAILURE);
+	}
+	commands = p;
+
 	commands[ncommands].name = strdup(conv_udf_name(name));
 	commands[ncommands].cmd = strdup(cmd);
-	ncommands++;
+	++ncommands;
 }
 
 static const char *
@@ -662,33 +690,50 @@ conv_udf_name(const char *cmd)
 static void
 add_bookmark(char name, const char *dir, const char *file)
 {
-	bookmarks = realloc(bookmarks, sizeof(bookmark)*(nbookmarks + 1));
+	void *p = realloc(bookmarks, sizeof(bookmark)*(nbookmarks + 1));
+	if(p == NULL)
+	{
+		fprintf(stderr, "%s\n", "Memory allocation error.");
+		exit(EXIT_FAILURE);
+	}
+	bookmarks = p;
+
 	bookmarks[nbookmarks].name = name;
 	bookmarks[nbookmarks].dir = strdup(dir);
 	bookmarks[nbookmarks].file = strdup(file);
-	nbookmarks++;
+	++nbookmarks;
 }
 
 static void
 add_filetype(const char *desc, const char *exts, const char *viewer,
 		const char *programs)
 {
-	char *p, *ex_copy, *free_this, *exptr2;
+	void *p;
+	char *e, *ex_copy, *free_this, *exptr2;
 	size_t len;
 
-	filetypes = realloc(filetypes, sizeof(filetype)*(nfiletypes + 1));
+	p = realloc(filetypes, sizeof(filetype)*(nfiletypes + 1));
+	if(p == NULL)
+	{
+		fprintf(stderr, "%s\n", "Memory allocation error.");
+		exit(EXIT_FAILURE);
+	}
+	filetypes = p;
+
 	filetypes[nfiletypes].description = strdup(desc);
 	filetypes[nfiletypes].viewer = strdup(viewer);
 	filetypes[nfiletypes].programs = strdup(programs);
 
-	p = NULL;
+	e = NULL;
 	len = 0;
 	ex_copy = strdup(exts);
 	free_this = ex_copy;
 	exptr2 = NULL;
 	while(*ex_copy != '\0')
 	{
+		char *p;
 		size_t new_len;
+
 		if((exptr2 = strchr(ex_copy, ',')) == NULL)
 			exptr2 = ex_copy + strlen(ex_copy);
 		else
@@ -697,18 +742,25 @@ add_filetype(const char *desc, const char *exts, const char *viewer,
 		new_len = len + 2 + strlen(ex_copy);
 		if(len > 0)
 			new_len += 1;
-		p = realloc(p, new_len + 1);
+		p = realloc(e, new_len + 1);
+		if(p == NULL)
+		{
+			fprintf(stderr, "%s\n", "Memory allocation error.");
+			exit(EXIT_FAILURE);
+		}
+		e = p;
+
 		if(len > 0)
-			strcpy(p + len, ",*.");
+			strcpy(e + len, ",*.");
 		else
-			strcpy(p + len, "*.");
-		strcat(p + len, ex_copy);
+			strcpy(e + len, "*.");
+		strcat(e + len, ex_copy);
 		len = new_len;
 
 		ex_copy = exptr2;
 	}
 	free(free_this);
-	filetypes[nfiletypes].extensions = p;
+	filetypes[nfiletypes].extensions = e;
 
 	nfiletypes++;
 }
@@ -1038,12 +1090,12 @@ write_vifminfo(const char *config_file, int vifm_like)
 	if(vifminfo & VIFMINFO_DHISTORY)
 	{
 		fputs("\n# Left window history (oldest to newest):\n", fp);
-		fprintf(fp, "d%s\n", (lwin_dir == NULL) ? "" : lwin_dir);
+		fprintf(fp, "d%s\n", lwin_dir);
 		if(vifminfo & VIFMINFO_SAVEDIRS)
 			fprintf(fp, "d\n");
 
 		fputs("\n# Right window history (oldest to newest):\n", fp);
-		fprintf(fp, "D%s\n", (rwin_dir == NULL) ? "" : rwin_dir);
+		fprintf(fp, "D%s\n", rwin_dir);
 		if(vifminfo & VIFMINFO_SAVEDIRS)
 			fprintf(fp, "D\n");
 	}
@@ -1082,7 +1134,6 @@ read_color_scheme_file(const char *config_file)
 	char *s1 = NULL;
 	char *s2 = NULL;
 	char *s3 = NULL;
-	char *sx = NULL;
 
 	if((fp = fopen(config_file, "r")) == NULL)
 	{
@@ -1094,6 +1145,7 @@ read_color_scheme_file(const char *config_file)
 	while(fgets(line, sizeof(line), fp))
 	{
 		int args;
+		char *sx;
 
 		if(line[0] == '#')
 			continue;
@@ -1164,8 +1216,8 @@ load_default_colors(void)
 static void
 init_color_scheme(col_scheme_t *cs)
 {
-	int i;
-	for(i = 0; i < ARRAY_LEN(default_colors); i++)
+	size_t i;
+	for(i = 0U; i < ARRAY_LEN(default_colors); ++i)
 	{
 		cs->color[i].fg = default_colors[i][0];
 		cs->color[i].bg = default_colors[i][1];
@@ -1187,7 +1239,7 @@ add_color(char s1[], char s2[], char s3[])
 	bg = colname2int(s3);
 
 	if(!strcmp(s1, "MENU"))
-		y = MENU_COLOR;
+		y = WILD_MENU_COLOR;
 	else if(!strcmp(s1, "BORDER"))
 		y = BORDER_COLOR;
 	else if(!strcmp(s1, "WIN"))
@@ -1256,7 +1308,8 @@ colname2int(char col[])
 	return -1;
 }
 
-int make_dir(const char *dir_name, mode_t mode)
+static int
+make_dir(const char dir_name[], mode_t mode)
 {
 #ifndef _WIN32
 	return mkdir(dir_name, mode);
@@ -1285,7 +1338,7 @@ write_color_schemes(const char *colors_dir)
 		char buf[PATH_MAX];
 		int y;
 
-		snprintf(buf, sizeof(buf), "%s/%s", colors_dir, cs.array[x].name);
+		snprintf(buf, sizeof(buf), "%s/%s.vifm", colors_dir, cs.array[x].name);
 		if((fp = fopen(buf, "w")) == NULL)
 		{
 			fprintf(stderr, "Can't create color scheme file at \"%s\"\n", buf);
@@ -1328,21 +1381,21 @@ write_color_schemes(const char *colors_dir)
 
 		fprintf(fp, "\" highlight group cterm=style ctermfg=foreground_color ctermbg=background_color\n\n");
 
-		for(y = 0; y < MAXNUM_COLOR - 2; y++)
+		for(y = 0; y < MAXNUM_COLOR; y++)
 		{
 			char fg_buf[16], bg_buf[16];
 			int fg = cs.array[x].color[y].fg;
 			int bg = cs.array[x].color[y].bg;
 			if(fg == -1)
 				strcpy(fg_buf, "none");
-			else if(fg < ARRAY_LEN(COLOR_NAMES))
+			else if(fg < (int)ARRAY_LEN(COLOR_NAMES))
 				snprintf(fg_buf, sizeof(fg_buf), "%s", COLOR_NAMES[fg]);
 			else
 				snprintf(fg_buf, sizeof(fg_buf), "%d", fg);
 
 			if(bg == -1)
 				strcpy(bg_buf, "none");
-			else if(bg < ARRAY_LEN(COLOR_NAMES))
+			else if(bg < (int)ARRAY_LEN(COLOR_NAMES))
 				snprintf(bg_buf, sizeof(bg_buf), "%s", COLOR_NAMES[bg]);
 			else
 				snprintf(bg_buf, sizeof(bg_buf), "%d", bg);
@@ -1374,5 +1427,57 @@ conv_attrs_to_str(int attrs)
 	return result;
 }
 
+/* Adds .vifm extension to all regular files in colors_dir directory. */
+static void
+rename_color_schemes(const char colors_dir[])
+{
+	DIR *dir;
+	struct dirent *d;
+
+	dir = opendir(colors_dir);
+	if(dir == NULL)
+	{
+		return;
+	}
+
+	while((d = readdir(dir)) != NULL)
+	{
+		char full_old_path[PATH_MAX];
+		char full_new_path[PATH_MAX];
+		snprintf(full_old_path, sizeof(full_old_path), "%s/%s", colors_dir,
+				d->d_name);
+		snprintf(full_new_path, sizeof(full_new_path), "%s/%s.vifm", colors_dir,
+				d->d_name);
+#ifndef _WIN32
+		if(d->d_type == DT_REG ||
+				(d->d_type == DT_UNKNOWN && is_regular_file(full_old_path)))
+#else
+		if(is_regular_file(full_old_path))
+#endif
+		{
+			(void)rename(full_old_path, full_new_path);
+		}
+	}
+	closedir(dir);
+}
+
+/* Checks whether file is the regular one (not a directory, pipe, etc.).
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+is_regular_file(const char path[])
+{
+#ifndef _WIN32
+	struct stat s;
+	return stat(path, &s) == 0 && (s.st_mode & S_IFMT) == S_IFREG;
+#else
+	const DWORD attrs = GetFileAttributesA(path);
+	if(attrs == INVALID_FILE_ATTRIBUTES)
+	{
+		return 0;
+	}
+	return (attrs & FILE_ATTRIBUTE_DIRECTORY) == 0UL;
+#endif
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
