@@ -1,7 +1,12 @@
 #ifndef VIFM_TESTS__COLMGR__TEST_H__
 #define VIFM_TESTS__COLMGR__TEST_H__
 
-#define MAX_COLOR_PAIRS 64
+#define RESERVED_COLOR_PAIRS 1
+#define CUSTOM_COLOR_PAIRS 127
+#define TOTAL_COLOR_PAIRS (RESERVED_COLOR_PAIRS + CUSTOM_COLOR_PAIRS)
+
+#define UNUSED_SEED 99
+#define INUSE_SEED 999
 
 #endif /* VIFM_TESTS__COLMGR__TEST_H__ */
 
