@@ -1,6 +1,8 @@
-#include <string.h>
+#include <stic.h>
 
-#include "seatest.h"
+#include <stddef.h> /* NULL size_t */
+#include <stdio.h> /* snprintf() */
+#include <string.h>
 
 #include "../../src/utils/macros.h"
 #include "../../src/column_view.h"
@@ -10,8 +12,29 @@
 static const size_t MAX_WIDTH = 40;
 static char print_buffer[40 + 1];
 
+static void column_line_print(const void *data, int column_id, const char buf[],
+		size_t offset);
+static void column1_func(int id, const void *data, size_t buf_len, char *buf);
+static void column2_func(int id, const void *data, size_t buf_len, char *buf);
+static void column2_short_func(int id, const void *data, size_t buf_len,
+		char *buf);
+
+SETUP()
+{
+	print_next = &column_line_print;
+	col1_next = &column1_func;
+	col2_next = &column2_func;
+}
+
+TEARDOWN()
+{
+	print_next = NULL;
+	col1_next = NULL;
+	col2_next = NULL;
+}
+
 static void
-column_line_print(const void *data, int column_id, const char *buf,
+column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset)
 {
 	strncpy(print_buffer + offset, buf, strlen(buf));
@@ -52,27 +75,9 @@ perform_test(column_info_t column_infos[], int n)
 	columns_free(cols);
 }
 
-static void
-setup(void)
+TEST(none_align_left)
 {
-	print_next = column_line_print;
-	col1_next = column1_func;
-	col2_next = column2_func;
-}
-
-static void
-teardown(void)
-{
-	print_next = NULL;
-	col1_next = NULL;
-	col2_next = NULL;
-}
-
-static void
-test_none_align_left(void)
-{
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_LEFT,     .sizing = ST_ABSOLUTE, .cropping = CT_NONE, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -85,11 +90,9 @@ test_none_align_left(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_ellipsis_align_left(void)
+TEST(ellipsis_align_left)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_LEFT,     .sizing = ST_ABSOLUTE, .cropping = CT_ELLIPSIS, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -102,11 +105,9 @@ test_ellipsis_align_left(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_truncating_align_left(void)
+TEST(truncating_align_left)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_LEFT,     .sizing = ST_ABSOLUTE, .cropping = CT_NONE, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -119,11 +120,9 @@ test_truncating_align_left(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_none_align_right(void)
+TEST(none_align_right)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_LEFT,     .sizing = ST_ABSOLUTE, .cropping = CT_NONE, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -136,11 +135,9 @@ test_none_align_right(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_none_align_right_overlapping(void)
+TEST(none_align_right_overlapping)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 0UL, .text_width = 0UL,
 		  .align = AT_LEFT,     .sizing = ST_AUTO, .cropping = CT_NONE, },
 		{ .column_id = COL2_ID, .full_width = 0UL, .text_width = 0UL,
@@ -154,11 +151,9 @@ test_none_align_right_overlapping(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_no_overlapping(void)
+TEST(no_overlapping)
 {
-	static column_info_t column_infos[3] =
-	{
+	static column_info_t column_infos[3] = {
 		{ .column_id = COL2_ID, .full_width = 0UL,     .text_width = 0UL,
 		  .align = AT_LEFT,     .sizing = ST_AUTO,     .cropping = CT_ELLIPSIS, },
 		{ .column_id = COL2_ID, .full_width = 8UL,     .text_width = 8UL,
@@ -174,11 +169,9 @@ test_no_overlapping(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_ellipsis_align_right(void)
+TEST(ellipsis_align_right)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_RIGHT,    .sizing = ST_ABSOLUTE, .cropping = CT_ELLIPSIS, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -191,11 +184,9 @@ test_ellipsis_align_right(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_truncating_align_right(void)
+TEST(truncating_align_right)
 {
-	static column_info_t column_infos[2] =
-	{
+	static column_info_t column_infos[2] = {
 		{ .column_id = COL1_ID, .full_width = 15UL,    .text_width = 15UL,
 		  .align = AT_LEFT,     .sizing = ST_ABSOLUTE, .cropping = CT_NONE, },
 		{ .column_id = COL2_ID, .full_width = 35UL,    .text_width = 35UL,
@@ -208,11 +199,9 @@ test_truncating_align_right(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-static void
-test_ellipsis_less_space(void)
+TEST(ellipsis_less_space)
 {
-	static column_info_t column_info =
-	{
+	static column_info_t column_info = {
 		.column_id = COL1_ID, .full_width = 0UL, .text_width = 0UL,
 		.align = AT_LEFT,     .sizing = ST_AUTO, .cropping = CT_ELLIPSIS,
 	};
@@ -230,28 +219,5 @@ test_ellipsis_less_space(void)
 	assert_string_equal(expected, print_buffer);
 }
 
-void
-cropping_tests(void)
-{
-	test_fixture_start();
-
-	fixture_setup(setup);
-	fixture_teardown(teardown);
-
-	run_test(test_none_align_left);
-	run_test(test_ellipsis_align_left);
-	run_test(test_truncating_align_left);
-
-	run_test(test_none_align_right);
-	run_test(test_none_align_right_overlapping);
-	run_test(test_no_overlapping);
-	run_test(test_ellipsis_align_right);
-	run_test(test_truncating_align_right);
-
-	run_test(test_ellipsis_less_space);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
