@@ -1,9 +1,8 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/cmds.h"
 
-static void
-test_empty_string_untouched(void)
+TEST(empty_string_untouched)
 {
 	char escaped[] = "";
 	const char *expected = "";
@@ -11,8 +10,7 @@ test_empty_string_untouched(void)
 	assert_string_equal(expected, escaped);
 }
 
-static void
-test_stops_at_the_end(void)
+TEST(stops_at_the_end)
 {
 	char escaped[] = "a\\\0b";
 	const char *expected = "a";
@@ -21,8 +19,7 @@ test_stops_at_the_end(void)
 	assert_int_equal('\0', escaped[2]);
 }
 
-static void
-test_incomplete_escape_sequence_truncated(void)
+TEST(incomplete_escape_sequence_truncated)
 {
 	char escaped[] = "a\\";
 	const char *expected = "a";
@@ -30,17 +27,5 @@ test_incomplete_escape_sequence_truncated(void)
 	assert_string_equal(expected, escaped);
 }
 
-void
-unescape_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_empty_string_untouched);
-	run_test(test_stops_at_the_end);
-	run_test(test_incomplete_escape_sequence_truncated);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
