@@ -1,11 +1,10 @@
-#include <stdlib.h>
+#include <stic.h>
 
-#include "seatest.h"
+#include <stdlib.h>
 
 #include "../../src/engine/completion.h"
 
-static void
-test_general(void)
+TEST(general)
 {
 	char *buf;
 
@@ -32,8 +31,7 @@ test_general(void)
 	free(buf);
 }
 
-static void
-test_sorting(void)
+TEST(sorting)
 {
 	char *buf;
 
@@ -57,8 +55,7 @@ test_sorting(void)
 	free(buf);
 }
 
-static void
-test_one_element_completion(void)
+TEST(one_element_completion)
 {
 	char *buf;
 
@@ -80,8 +77,7 @@ test_one_element_completion(void)
 	free(buf);
 }
 
-static void
-test_one_unambiguous_completion(void)
+TEST(one_unambiguous_completion)
 {
 	char *buf;
 
@@ -98,8 +94,7 @@ test_one_unambiguous_completion(void)
 	free(buf);
 }
 
-static void
-test_rewind_one_unambiguous_completion(void)
+TEST(rewind_one_unambiguous_completion)
 {
 	char *buf;
 
@@ -125,8 +120,7 @@ test_rewind_one_unambiguous_completion(void)
 	free(buf);
 }
 
-static void
-test_rewind(void)
+TEST(rewind_completion)
 {
 	char *buf;
 
@@ -171,8 +165,7 @@ test_rewind(void)
 	free(buf);
 }
 
-static void
-test_order(void)
+TEST(order)
 {
 	char *buf;
 
@@ -197,8 +190,7 @@ test_order(void)
 	free(buf);
 }
 
-static void
-test_umbiguous_begin(void)
+TEST(umbiguous_begin)
 {
 	char *buf;
 
@@ -230,8 +222,7 @@ test_umbiguous_begin(void)
 	free(buf);
 }
 
-static void
-test_two_matches(void)
+TEST(two_matches)
 {
 	char *buf;
 
@@ -259,8 +250,7 @@ test_two_matches(void)
 	free(buf);
 }
 
-static void
-test_removes_duplicates(void)
+TEST(removes_duplicates)
 {
 	char *buf;
 
@@ -288,24 +278,5 @@ test_removes_duplicates(void)
 	free(buf);
 }
 
-void
-completion_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_general);
-	run_test(test_sorting);
-	run_test(test_one_element_completion);
-	run_test(test_one_unambiguous_completion);
-	run_test(test_rewind_one_unambiguous_completion);
-	run_test(test_rewind);
-	run_test(test_order);
-	run_test(test_umbiguous_begin);
-	run_test(test_two_matches);
-	run_test(test_removes_duplicates);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
