@@ -1,9 +1,8 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/utils/filter.h"
 
-static void
-test_filter_created_empty(void)
+TEST(filter_created_empty)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 0));
@@ -11,8 +10,7 @@ test_filter_created_empty(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_case_insensitivity(void)
+TEST(case_insensitivity)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 0));
@@ -21,8 +19,7 @@ test_case_insensitivity(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_case_sensitivity(void)
+TEST(case_sensitivity)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -32,17 +29,5 @@ test_case_sensitivity(void)
 	filter_dispose(&filter);
 }
 
-void
-lifetime_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_filter_created_empty);
-	run_test(test_case_insensitivity);
-	run_test(test_case_sensitivity);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
