@@ -1,9 +1,8 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/utils/filter.h"
 
-static void
-test_returns_positive_number_on_match(void)
+TEST(returns_positive_number_on_match)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -14,8 +13,7 @@ test_returns_positive_number_on_match(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_returns_zero_on_no_match(void)
+TEST(returns_zero_on_no_match)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -26,8 +24,7 @@ test_returns_zero_on_no_match(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_returns_negative_number_for_empty_filter(void)
+TEST(returns_negative_number_for_empty_filter)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -37,8 +34,7 @@ test_returns_negative_number_for_empty_filter(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_returns_negative_number_for_invalid_regexp(void)
+TEST(returns_negative_number_for_invalid_regexp)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -49,18 +45,5 @@ test_returns_negative_number_for_invalid_regexp(void)
 	filter_dispose(&filter);
 }
 
-void
-matches_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_returns_positive_number_on_match);
-	run_test(test_returns_zero_on_no_match);
-	run_test(test_returns_negative_number_for_empty_filter);
-	run_test(test_returns_negative_number_for_invalid_regexp);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
