@@ -1,9 +1,8 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/utils/filter.h"
 
-static void
-test_filter_matches_differently_after_set(void)
+TEST(filter_matches_differently_after_set)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -18,8 +17,7 @@ test_filter_matches_differently_after_set(void)
 	filter_dispose(&filter);
 }
 
-static void
-test_set_to_empty_is_like_clear(void)
+TEST(set_to_empty_is_like_clear)
 {
 	filter_t filter;
 	assert_int_equal(0, filter_init(&filter, 1));
@@ -30,16 +28,5 @@ test_set_to_empty_is_like_clear(void)
 	filter_dispose(&filter);
 }
 
-void
-set_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_filter_matches_differently_after_set);
-	run_test(test_set_to_empty_is_like_clear);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
