@@ -1,11 +1,26 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 #include "../../src/engine/mode.h"
 #include "../../src/modes/modes.h"
 
+static int handler(wchar_t key);
+
 static int counter;
 
+SETUP()
+{
+	set_def_handler(CMDLINE_MODE, &handler);
+	add_user_keys(L"s", L":shell", NORMAL_MODE, 0);
+	add_user_keys(L"q", L"toto", CMDLINE_MODE, 0);
+}
+
+TEARDOWN()
+{
+	set_def_handler(CMDLINE_MODE, NULL);
+	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
+}
+
 static int
 handler(wchar_t key)
 {
@@ -18,23 +33,7 @@ handler(wchar_t key)
 	return 0;
 }
 
-static void
-add_custom_keys(void)
-{
-	set_def_handler(CMDLINE_MODE, &handler);
-	add_user_keys(L"s", L":shell", NORMAL_MODE, 0);
-	add_user_keys(L"q", L"toto", CMDLINE_MODE, 0);
-}
-
-static void
-teardown(void)
-{
-	set_def_handler(CMDLINE_MODE, NULL);
-	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
-}
-
-static void
-cmd_line_tst(void)
+TEST(cmd_line)
 {
 	counter = 0;
 
@@ -44,17 +43,5 @@ cmd_line_tst(void)
 	assert_true(counter == 9);
 }
 
-void
-def_keys_and_user_mappings(void)
-{
-	test_fixture_start();
-
-	fixture_setup(add_custom_keys);
-	fixture_teardown(teardown);
-
-	run_test(cmd_line_tst);
-
-	test_fixture_end();
-}
-
-/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab : */
+/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
+/* vim: set cinoptions+=t0 : */
