@@ -1,17 +1,12 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 #include "../../src/engine/mode.h"
 #include "../../src/modes/modes.h"
 
-static int
-def_handler(const wchar_t keys)
-{
-	return 0;
-}
+static int def_handler(const wchar_t keys);
 
-static void
-setup(void)
+SETUP()
 {
 	static int mode_flags[]= {
 		MF_USES_REGS | MF_USES_COUNT,
@@ -26,8 +21,13 @@ setup(void)
 	add_user_keys(L"asdf", L"ddd", CMDLINE_MODE, 0);
 }
 
-static void
-test_asdf(void)
+static int
+def_handler(const wchar_t keys)
+{
+	return 0;
+}
+
+TEST(asdf)
 {
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"a"));
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"as"));
@@ -35,16 +35,5 @@ test_asdf(void)
 	assert_false(IS_KEYS_RET_CODE(execute_keys(L"asdf")));
 }
 
-void
-no_regs_long_key(void)
-{
-	test_fixture_start();
-
-	fixture_setup(setup);
-
-	run_test(test_asdf);
-
-	test_fixture_end();
-}
-
-/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab : */
+/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
+/* vim: set cinoptions+=t0 : */
