@@ -1,10 +1,9 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 #include "../../src/modes/modes.h"
 
-static void
-setup(void)
+SETUP()
 {
 	add_user_keys(L"a", L"z", NORMAL_MODE, 0);
 	add_user_keys(L"ab", L"x", NORMAL_MODE, 0);
@@ -15,33 +14,19 @@ setup(void)
 	add_user_keys(L"qbc", L"k", NORMAL_MODE, 0);
 }
 
-static void
-test_when_previous_unknown(void)
+TEST(when_previous_unknown)
 {
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"a"));
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"ab"));
 	assert_false(IS_KEYS_RET_CODE(execute_keys(L"abc")));
 }
 
-static void
-test_when_previous_known(void)
+TEST(when_previous_known)
 {
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"q"));
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"qb"));
 	assert_false(IS_KEYS_RET_CODE(execute_keys(L"qbc")));
 }
 
-void
-pick_longest(void)
-{
-	test_fixture_start();
-
-	fixture_setup(setup);
-
-	run_test(test_when_previous_unknown);
-	run_test(test_when_previous_known);
-
-	test_fixture_end();
-}
-
-/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab : */
+/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
+/* vim: set cinoptions+=t0 : */
