@@ -1,16 +1,14 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 #include "../../src/modes/modes.h"
 
-static void
-add_custom_keys(void)
+SETUP()
 {
 	add_user_keys(L"abcdef", L"k", NORMAL_MODE, 0);
 }
 
-static void
-test_abcdef(void)
+TEST(abcdef)
 {
 	assert_int_equal(KEYS_WAIT, execute_keys(L"a"));
 	assert_int_equal(KEYS_WAIT, execute_keys(L"ab"));
@@ -20,16 +18,5 @@ test_abcdef(void)
 	assert_false(IS_KEYS_RET_CODE(execute_keys(L"abcdef")));
 }
 
-void
-put_wait_points(void)
-{
-	test_fixture_start();
-
-	fixture_setup(add_custom_keys);
-
-	run_test(test_abcdef);
-
-	test_fixture_end();
-}
-
-/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab : */
+/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
+/* vim: set cinoptions+=t0 : */
