@@ -1,4 +1,4 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 
@@ -14,22 +14,19 @@ extern int last_selector_count;
 		assert_int_equal((cmd_count), last_command_count); \
 	}
 
-static void
-test_no_number_ok(void)
+TEST(no_number_ok)
 {
 	check(L"d", L"dk", NO_COUNT_GIVEN, NO_COUNT_GIVEN);
 }
 
-static void
-test_with_number_ok(void)
+TEST(with_number_ok)
 {
 	check(L"d1",   L"d1k",   1*1,   NO_COUNT_GIVEN);
 	check(L"d12",  L"d12k",  1*12,  NO_COUNT_GIVEN);
 	check(L"d123", L"d123k", 1*123, NO_COUNT_GIVEN);
 }
 
-static void
-test_with_zero_number_fail(void)
+TEST(with_zero_number_fail)
 {
 	assert_int_equal(KEYS_UNKNOWN, execute_keys(L"d0"));
 	assert_int_equal(KEYS_UNKNOWN, execute_keys(L"d0k"));
@@ -39,26 +36,12 @@ test_with_zero_number_fail(void)
 	assert_int_equal(KEYS_UNKNOWN, execute_keys(L"d012k"));
 }
 
-static void
-test_with_number_before_and_in_the_middle_ok(void)
+TEST(with_number_before_and_in_the_middle_ok)
 {
 	check(L"2d1",   L"2d1k",   2*1,   NO_COUNT_GIVEN);
 	check(L"3d12",  L"3d12k",  3*12,  NO_COUNT_GIVEN);
 	check(L"2d123", L"2d123k", 2*123, NO_COUNT_GIVEN);
 }
 
-void
-selectors_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_no_number_ok);
-	run_test(test_with_number_ok);
-	run_test(test_with_zero_number_fail);
-	run_test(test_with_number_before_and_in_the_middle_ok);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
 /* vim: set cinoptions+=t0 : */
