@@ -1,10 +1,9 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/engine/keys.h"
 #include "../../src/modes/modes.h"
 
-static void
-add_custom_keys(void)
+SETUP()
 {
 	add_user_keys(L"hi", L"j", NORMAL_MODE, 0);
 	add_user_keys(L"hi2", L"hi", NORMAL_MODE, 0);
@@ -13,30 +12,15 @@ add_custom_keys(void)
 	add_user_keys(L"ha2", L"ho", NORMAL_MODE, 0);
 }
 
-static void
-user_key_chain(void)
+TEST(user_key_chain)
 {
 	assert_false(IS_KEYS_RET_CODE(execute_keys(L"ha2")));
 }
 
-static void
-user_key_chain_wait(void)
+TEST(user_key_chain_wait)
 {
 	assert_int_equal(KEYS_WAIT_SHORT, execute_keys(L"hi2"));
 }
 
-void
-users_key_to_key(void)
-{
-	test_fixture_start();
-
-	fixture_setup(add_custom_keys);
-
-	run_test(user_key_chain);
-	run_test(user_key_chain_wait);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
 /* vim: set cinoptions+=t0 : */
