@@ -1,9 +1,8 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include "../../src/commands.h"
 
-static void
-test_pipe(void)
+TEST(pipe)
 {
 	const char *buf;
 
@@ -23,8 +22,7 @@ test_pipe(void)
 	assert_int_equal(4, line_pos(buf, buf + 9, ' ', 1));
 }
 
-static void
-test_two_commands(void)
+TEST(two_commands)
 {
 	const char buf[] = "apropos|locate";
 
@@ -33,8 +31,7 @@ test_two_commands(void)
 	assert_int_equal(0, line_pos(buf, buf + 7, ' ', 0));
 }
 
-static void
-test_set_command(void)
+TEST(set_command)
 {
 	const char *buf;
 
@@ -49,8 +46,7 @@ test_set_command(void)
 	assert_int_equal(3, line_pos(buf, buf + 16, ' ', 0));
 }
 
-static void
-test_skip(void)
+TEST(skip)
 {
 	const char *buf;
 
@@ -60,8 +56,7 @@ test_skip(void)
 	assert_int_equal(1, line_pos(buf, buf + 15, ' ', 0));
 }
 
-static void
-test_custom_separator(void)
+TEST(custom_separator)
 {
 	const char *buf;
 
@@ -76,8 +71,7 @@ test_custom_separator(void)
 	assert_int_equal(0, line_pos(buf, buf + 14, '/', 1));
 }
 
-static void
-test_space_amp_before_bar(void)
+TEST(space_amp_before_bar)
 {
 	const char buf[] = "apropos &|locate";
 
@@ -87,20 +81,5 @@ test_space_amp_before_bar(void)
 	assert_int_equal(0, line_pos(buf, buf + 9, ' ', 0));
 }
 
-void
-test_command_separation(void)
-{
-	test_fixture_start();
-
-	run_test(test_pipe);
-	run_test(test_two_commands);
-	run_test(test_set_command);
-	run_test(test_skip);
-	run_test(test_custom_separator);
-	run_test(test_space_amp_before_bar);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
