@@ -1,15 +1,13 @@
-#include <stdlib.h>
+#include <stic.h>
 
-#include "seatest.h"
+#include <stdlib.h>
 
 #include "../../src/utils/macros.h"
 #include "../../src/macros.h"
 
-static void
-test_empty_string_ok(void)
+TEST(empty_string_ok)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'y', .value = "xx", .uses_left = 0, .group = -1, },
 	};
 
@@ -19,11 +17,9 @@ test_empty_string_ok(void)
 	free(expanded);
 }
 
-static void
-test_no_macros_ok(void)
+TEST(no_macros_ok)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -33,11 +29,9 @@ test_no_macros_ok(void)
 	free(expanded);
 }
 
-static void
-test_macro_substitution_works(void)
+TEST(macro_substitution_works)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -47,11 +41,9 @@ test_macro_substitution_works(void)
 	free(expanded);
 }
 
-static void
-test_use_negative_count_macro_not_added_implicitly(void)
+TEST(use_negative_count_macro_not_added_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = -1, .group = -1, },
 	};
 
@@ -61,11 +53,9 @@ test_use_negative_count_macro_not_added_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_use_count_0_macro_not_added_implicitly(void)
+TEST(use_count_0_macro_not_added_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -75,11 +65,9 @@ test_use_count_0_macro_not_added_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_use_count_1_macro_added_once_implicitly(void)
+TEST(use_count_1_macro_added_once_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 1, .group = -1, },
 	};
 
@@ -89,11 +77,9 @@ test_use_count_1_macro_added_once_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_use_count_1_macro_not_added_implicitly(void)
+TEST(use_count_1_macro_not_added_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 1, .group = -1, },
 	};
 
@@ -103,11 +89,9 @@ test_use_count_1_macro_not_added_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_use_count_2_macro_added_once_implicitly(void)
+TEST(use_count_2_macro_added_once_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 2, .group = -1, },
 	};
 
@@ -117,11 +101,9 @@ test_use_count_2_macro_added_once_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_use_count_2_macro_added_twice_implicitly(void)
+TEST(use_count_2_macro_added_twice_implicitly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 2, .group = -1, },
 	};
 
@@ -131,11 +113,9 @@ test_use_count_2_macro_added_twice_implicitly(void)
 	free(expanded);
 }
 
-static void
-test_unknown_macro_removed(void)
+TEST(unknown_macro_removed)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -145,11 +125,9 @@ test_unknown_macro_removed(void)
 	free(expanded);
 }
 
-static void
-test_double_percent_handled_correctly(void)
+TEST(double_percent_handled_correctly)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -159,11 +137,9 @@ test_double_percent_handled_correctly(void)
 	free(expanded);
 }
 
-static void
-test_ends_with_percent_ok(void)
+TEST(ends_with_percent_ok)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 0, .group = -1, },
 	};
 
@@ -173,11 +149,9 @@ test_ends_with_percent_ok(void)
 	free(expanded);
 }
 
-static void
-test_first_group_member_are_not_added_when_should_not(void)
+TEST(first_group_member_are_not_added_when_should_not)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 1, .group = 0, },
 		{ .letter = 'j', .value = "abc", .uses_left = 0, .group = 0, },
 	};
@@ -188,11 +162,9 @@ test_first_group_member_are_not_added_when_should_not(void)
 	free(expanded);
 }
 
-static void
-test_second_group_member_are_not_added_when_should_not(void)
+TEST(second_group_member_are_not_added_when_should_not)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 1, .group = 0, },
 		{ .letter = 'j', .value = "abc", .uses_left = 0, .group = 0, },
 	};
@@ -203,11 +175,9 @@ test_second_group_member_are_not_added_when_should_not(void)
 	free(expanded);
 }
 
-static void
-test_first_item_of_group_is_repeated_when_needed(void)
+TEST(first_item_of_group_is_repeated_when_needed)
 {
-	custom_macro_t macros[] =
-	{
+	custom_macro_t macros[] = {
 		{ .letter = 'i', .value = "xyz", .uses_left = 1, .group = 0, },
 		{ .letter = 'j', .value = "abc", .uses_left = 0, .group = 0, },
 	};
@@ -218,29 +188,5 @@ test_first_item_of_group_is_repeated_when_needed(void)
 	free(expanded);
 }
 
-void
-expand_custom_macros_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_empty_string_ok);
-	run_test(test_no_macros_ok);
-	run_test(test_macro_substitution_works);
-	run_test(test_use_negative_count_macro_not_added_implicitly);
-	run_test(test_use_count_0_macro_not_added_implicitly);
-	run_test(test_use_count_1_macro_added_once_implicitly);
-	run_test(test_use_count_1_macro_not_added_implicitly);
-	run_test(test_use_count_2_macro_added_once_implicitly);
-	run_test(test_use_count_2_macro_added_twice_implicitly);
-	run_test(test_unknown_macro_removed);
-	run_test(test_double_percent_handled_correctly);
-	run_test(test_ends_with_percent_ok);
-	run_test(test_first_group_member_are_not_added_when_should_not);
-	run_test(test_second_group_member_are_not_added_when_should_not);
-	run_test(test_first_item_of_group_is_repeated_when_needed);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
