@@ -1,11 +1,11 @@
-#include <stdlib.h>
-#include <string.h>
+#include <stic.h>
 
-#include "seatest.h"
+#include <stdlib.h> /* free() */
+#include <string.h> /* strdup() */
 
 #include "../../src/cfg/config.h"
-#include "../../src/filelist.h"
-#include "../../src/ui.h"
+#include "../../src/ui/ui.h"
+#include "../../src/filtering.h"
 
 #define assert_hidden(view, name, dir) \
 	assert_false(file_is_visible(&view, name, dir))
@@ -13,27 +13,29 @@
 #define assert_visible(view, name, dir) \
 	assert_true(file_is_visible(&view, name, dir))
 
-#ifdef _WIN32
-#define CASE_SENSATIVE_FILTER 0
-#else
-#define CASE_SENSATIVE_FILTER 1
-#endif
-
-static void
-setup(void)
+SETUP()
 {
+	cfg.slow_fs_list = strdup("");
+
 	cfg.filter_inverted_by_default = 1;
 
 	lwin.list_rows = 7;
 	lwin.list_pos = 2;
 	lwin.dir_entry = calloc(lwin.list_rows, sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("with(round)");
+	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[1].name = strdup("with[square]");
+	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[2].name = strdup("with{curly}");
+	lwin.dir_entry[2].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[3].name = strdup("with<angle>");
+	lwin.dir_entry[3].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[4].name = strdup("withSPECS+*^$?|\\");
+	lwin.dir_entry[4].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[5].name = strdup("with....dots");
+	lwin.dir_entry[5].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[6].name = strdup("withnonodots");
+	lwin.dir_entry[6].origin = &lwin.curr_dir[0];
 
 	lwin.dir_entry[0].selected = 1;
 	lwin.dir_entry[1].selected = 1;
@@ -44,8 +46,8 @@ setup(void)
 	lwin.dir_entry[6].selected = 0;
 	lwin.selected_files = 6;
 
-	filter_init(&lwin.manual_filter, CASE_SENSATIVE_FILTER);
-	filter_init(&lwin.auto_filter, CASE_SENSATIVE_FILTER);
+	filter_init(&lwin.manual_filter, FILTER_DEF_CASE_SENSITIVITY);
+	filter_init(&lwin.auto_filter, FILTER_DEF_CASE_SENSITIVITY);
 	lwin.invert = cfg.filter_inverted_by_default;
 
 	lwin.column_count = 1;
@@ -54,13 +56,21 @@ setup(void)
 	rwin.list_pos = 2;
 	rwin.dir_entry = calloc(rwin.list_rows, sizeof(*rwin.dir_entry));
 	rwin.dir_entry[0].name = strdup("dir1.d");
+	rwin.dir_entry[0].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[1].name = strdup("dir2.d");
+	rwin.dir_entry[1].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[2].name = strdup("dir3.d");
+	rwin.dir_entry[2].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[3].name = strdup("file1.d");
+	rwin.dir_entry[3].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[4].name = strdup("file2.d");
+	rwin.dir_entry[4].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[5].name = strdup("file3.d");
+	rwin.dir_entry[5].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[6].name = strdup("withnonodots");
-	rwin.dir_entry[7].name = strdup("withnonodots/");
+	rwin.dir_entry[6].origin = &rwin.curr_dir[0];
+	rwin.dir_entry[7].name = strdup("somedir");
+	rwin.dir_entry[7].origin = &rwin.curr_dir[0];
 
 	rwin.dir_entry[0].selected = 0;
 	rwin.dir_entry[1].selected = 0;
@@ -69,16 +79,18 @@ setup(void)
 	rwin.dir_entry[4].selected = 0;
 	rwin.dir_entry[5].selected = 0;
 	rwin.dir_entry[6].selected = 0;
+	rwin.dir_entry[7].selected = 0;
 	rwin.selected_files = 0;
 
-	filter_init(&rwin.manual_filter, CASE_SENSATIVE_FILTER);
-	filter_init(&rwin.auto_filter, CASE_SENSATIVE_FILTER);
+	filter_init(&rwin.manual_filter, FILTER_DEF_CASE_SENSITIVITY);
+	filter_init(&rwin.auto_filter, FILTER_DEF_CASE_SENSITIVITY);
 	rwin.invert = cfg.filter_inverted_by_default;
 
 	rwin.column_count = 1;
 }
 
-static void cleanup_view(FileView *view)
+static void
+cleanup_view(FileView *view)
 {
 	int i;
 
@@ -89,53 +101,61 @@ static void cleanup_view(FileView *view)
 	filter_dispose(&view->auto_filter);
 }
 
-static void
-teardown(void)
+TEARDOWN()
 {
+	free(cfg.slow_fs_list);
+	cfg.slow_fs_list = NULL;
+
 	cleanup_view(&lwin);
 	cleanup_view(&rwin);
 }
 
-static void
-test_filtering(void)
+TEST(filtering)
 {
-	int i;
-
+	assert_int_equal(7, lwin.list_rows);
 	filter_selected_files(&lwin);
+	assert_int_equal(1, lwin.list_rows);
 
-	for(i = 0; i < lwin.list_rows - 1; i++)
-		assert_hidden(lwin, lwin.dir_entry[i].name, 0);
-	assert_visible(lwin, lwin.dir_entry[i].name, 0);
+	assert_string_equal("withnonodots", lwin.dir_entry[0].name);
+	assert_visible(lwin, lwin.dir_entry[0].name, 0);
 }
 
-static void
-test_filtering_file_does_not_filter_dir(void)
+TEST(filtering_file_does_not_filter_dir)
 {
+	char *const name = strdup(rwin.dir_entry[6].name);
+
 	rwin.dir_entry[6].selected = 1;
 	rwin.selected_files = 1;
 
+	assert_int_equal(8, rwin.list_rows);
 	filter_selected_files(&rwin);
+	assert_int_equal(7, rwin.list_rows);
 
-	assert_hidden(rwin, rwin.dir_entry[6].name, 0);
-	/* Pass 0 as if for file, because we already have trailing slash there. */
-	assert_visible(rwin, rwin.dir_entry[7].name, 0);
+	assert_hidden(rwin, name, 0);
+	assert_visible(rwin, name, 1);
+
+	free(name);
 }
 
-static void
-test_filtering_dir_does_not_filter_file(void)
+TEST(filtering_dir_does_not_filter_file)
 {
-	rwin.dir_entry[7].selected = 1;
+	char *const name = strdup(rwin.dir_entry[6].name);
+
+	rwin.dir_entry[6].selected = 1;
+	rwin.dir_entry[6].type = FT_DIR;
 	rwin.selected_files = 1;
 
+	assert_int_equal(8, rwin.list_rows);
 	filter_selected_files(&rwin);
+	assert_int_equal(7, rwin.list_rows);
 
-	/* Pass 0 as if for file, because we already have trailing slash there. */
-	assert_hidden(rwin, rwin.dir_entry[7].name, 0);
-	assert_visible(rwin, rwin.dir_entry[6].name, 0);
+	assert_hidden(rwin, name, 1);
+	assert_visible(rwin, name, 0);
+
+	free(name);
 }
 
-static void
-test_filtering_files_does_not_filter_dirs(void)
+TEST(filtering_files_does_not_filter_dirs)
 {
 	(void)filter_set(&rwin.manual_filter, "^.*\\.d$");
 
@@ -145,10 +165,12 @@ test_filtering_files_does_not_filter_dirs(void)
 	assert_hidden(rwin, rwin.dir_entry[3].name, 0);
 	assert_hidden(rwin, rwin.dir_entry[4].name, 0);
 	assert_hidden(rwin, rwin.dir_entry[5].name, 0);
+	assert_visible(rwin, rwin.dir_entry[6].name, 0);
+	assert_visible(rwin, rwin.dir_entry[7].name, 1);
+	assert_int_equal(8, rwin.list_rows);
 }
 
-static void
-test_filtering_dirs_does_not_filter_files(void)
+TEST(filtering_dirs_does_not_filter_files)
 {
 	(void)filter_set(&rwin.manual_filter, "^.*\\.d/$");
 
@@ -158,10 +180,12 @@ test_filtering_dirs_does_not_filter_files(void)
 	assert_visible(rwin, rwin.dir_entry[3].name, 0);
 	assert_visible(rwin, rwin.dir_entry[4].name, 0);
 	assert_visible(rwin, rwin.dir_entry[5].name, 0);
+	assert_visible(rwin, rwin.dir_entry[6].name, 0);
+	assert_visible(rwin, rwin.dir_entry[7].name, 1);
+	assert_int_equal(8, rwin.list_rows);
 }
 
-static void
-test_filtering_files_and_dirs(void)
+TEST(filtering_files_and_dirs)
 {
 	(void)filter_set(&rwin.manual_filter, "^.*\\.d/?$");
 
@@ -171,24 +195,9 @@ test_filtering_files_and_dirs(void)
 	assert_hidden(rwin, rwin.dir_entry[3].name, 0);
 	assert_hidden(rwin, rwin.dir_entry[4].name, 0);
 	assert_hidden(rwin, rwin.dir_entry[5].name, 0);
-}
-
-void
-filtering_tests(void)
-{
-	test_fixture_start();
-
-	fixture_setup(setup);
-	fixture_teardown(teardown);
-
-	run_test(test_filtering);
-	run_test(test_filtering_file_does_not_filter_dir);
-	run_test(test_filtering_dir_does_not_filter_file);
-	run_test(test_filtering_files_does_not_filter_dirs);
-	run_test(test_filtering_dirs_does_not_filter_files);
-	run_test(test_filtering_files_and_dirs);
-
-	test_fixture_end();
+	assert_visible(rwin, rwin.dir_entry[6].name, 0);
+	assert_visible(rwin, rwin.dir_entry[7].name, 1);
+	assert_int_equal(8, rwin.list_rows);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
