@@ -1,11 +1,11 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include <stddef.h> /* NULL */
 #include <string.h> /* strcmp() */
 
+#include "../../src/ui/ui.h"
 #include "../../src/utils/env.h"
 #include "../../src/commands.h"
-#include "../../src/ui.h"
 
 #define VAR_A "VAR_A"
 #define VAR_B "VAR_B"
@@ -21,62 +21,56 @@ remove_tmp_vars(void)
 	env_remove(VAR_D);
 }
 
-static void
-setup(void)
+SETUP()
 {
 	init_commands();
 	lwin.selected_files = 0;
 	remove_tmp_vars();
 }
 
-static void
-test_if_without_else_true_condition(void)
+TEST(if_without_else_true_condition)
 {
 	const char *const COMMANDS = " | if 1 == 1"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(VAR_A, env_get(VAR_A));
 }
 
-static void
-test_if_without_else_false_condition(void)
+TEST(if_without_else_false_condition)
 {
 	const char *const COMMANDS = " | if 1 == 0"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 }
 
-static void
-test_if_with_else_true_condition(void)
+TEST(if_with_else_true_condition)
 {
 	const char *const COMMANDS = " | if 1 == 1"
 	                             " | else"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 }
 
-static void
-test_if_with_else_false_condition(void)
+TEST(if_with_else_false_condition)
 {
 	const char *const COMMANDS = " | if 1 == 0"
 	                             " | else"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(VAR_A, env_get(VAR_A));
 }
 
-static void
-test_if_true_if_true_condition(void)
+TEST(if_true_if_true_condition)
 {
 	const char *const COMMANDS = " | if 1 == 1"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
@@ -86,14 +80,13 @@ test_if_true_if_true_condition(void)
 	                             " |     let $"VAR_C" = '"VAR_C"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(VAR_A, env_get(VAR_A));
 	assert_string_equal(VAR_B, env_get(VAR_B));
 	assert_string_equal(VAR_C, env_get(VAR_C));
 }
 
-static void
-test_if_true_if_false_condition(void)
+TEST(if_true_if_false_condition)
 {
 	const char *const COMMANDS = " | if 1 == 1"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
@@ -103,14 +96,13 @@ test_if_true_if_false_condition(void)
 	                             " |     let $"VAR_C" = '"VAR_C"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(VAR_A, env_get(VAR_A));
 	assert_string_equal(NULL, env_get(VAR_B));
 	assert_string_equal(VAR_C, env_get(VAR_C));
 }
 
-static void
-test_if_false_if_true_condition(void)
+TEST(if_false_if_true_condition)
 {
 	const char *const COMMANDS = " | if 1 == 0"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
@@ -120,14 +112,13 @@ test_if_false_if_true_condition(void)
 	                             " |     let $"VAR_C" = '"VAR_C"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 	assert_string_equal(NULL, env_get(VAR_B));
 	assert_string_equal(NULL, env_get(VAR_C));
 }
 
-static void
-test_if_false_else_if_true_condition(void)
+TEST(if_false_else_if_true_condition)
 {
 	const char *const COMMANDS = " | if 1 == 0"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
@@ -139,15 +130,14 @@ test_if_false_else_if_true_condition(void)
 	                             " |     let $"VAR_D" = '"VAR_D"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 	assert_string_equal(VAR_B, env_get(VAR_B));
 	assert_string_equal(VAR_C, env_get(VAR_C));
 	assert_string_equal(VAR_D, env_get(VAR_D));
 }
 
-static void
-test_if_false_if_else_condition(void)
+TEST(if_false_if_else_condition)
 {
 	const char *const COMMANDS = " | if 1 == 0"
 	                             " |     let $"VAR_A" = '"VAR_A"'"
@@ -159,15 +149,14 @@ test_if_false_if_else_condition(void)
 	                             " |     let $"VAR_D" = '"VAR_D"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 	assert_string_equal(NULL, env_get(VAR_B));
 	assert_string_equal(NULL, env_get(VAR_C));
 	assert_string_equal(NULL, env_get(VAR_D));
 }
 
-static void
-test_if_true_else_if_else_condition(void)
+TEST(if_true_else_if_else_condition)
 {
 	const char *const COMMANDS = " | if 1 == 1"
 	                             " | else"
@@ -180,33 +169,35 @@ test_if_true_else_if_else_condition(void)
 	                             " |     let $"VAR_D" = '"VAR_D"'"
 	                             " | endif";
 
-	assert_int_equal(0, exec_commands(COMMANDS, &lwin, GET_COMMAND));
+	assert_int_equal(0, exec_commands(COMMANDS, &lwin, CIT_COMMAND));
 	assert_string_equal(NULL, env_get(VAR_A));
 	assert_string_equal(NULL, env_get(VAR_B));
 	assert_string_equal(NULL, env_get(VAR_C));
 	assert_string_equal(NULL, env_get(VAR_D));
 }
 
-void
-if_else_tests(void)
+TEST(sourcing_in_body)
 {
-	test_fixture_start();
+	const char *const CMDS = " | if 1 == 1"
+	                         " |     source test-data/scripts/set-env.vifm"
+	                         " | endif";
 
-	fixture_setup(setup);
-	fixture_teardown(remove_tmp_vars);
+	assert_int_equal(0, exec_commands(CMDS, &lwin, CIT_COMMAND));
+}
 
-	run_test(test_if_without_else_true_condition);
-	run_test(test_if_without_else_false_condition);
-	run_test(test_if_with_else_true_condition);
-	run_test(test_if_with_else_false_condition);
-	run_test(test_if_true_if_true_condition);
-	run_test(test_if_true_if_false_condition);
-	run_test(test_if_false_if_true_condition);
-	run_test(test_if_false_else_if_true_condition);
-	run_test(test_if_false_if_else_condition);
-	run_test(test_if_true_else_if_else_condition);
+TEST(wrong_expr_causes_hard_error)
+{
+	assert_true(exec_commands("if $USER == root", &lwin, CIT_COMMAND) < 0);
+}
 
-	test_fixture_end();
+TEST(misplaced_else_causes_hard_error)
+{
+	assert_true(exec_commands("else", &lwin, CIT_COMMAND) < 0);
+}
+
+TEST(misplaced_endif_causes_hard_error)
+{
+	assert_true(exec_commands("endif", &lwin, CIT_COMMAND) < 0);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
