@@ -1,29 +1,26 @@
-#include "seatest.h"
+#include <stic.h>
 
 #include <unistd.h> /* chdir() */
 
+#include "../../src/ui/ui.h"
 #include "../../src/utils/macros.h"
 #include "../../src/fileops.h"
-#include "../../src/ui.h"
 
-static void
-test_names_less_than_files(void)
+TEST(names_less_than_files)
 {
 	char *src[] = { "a", "b" };
 	char *dst[] = { "a" };
 	assert_false(is_name_list_ok(ARRAY_LEN(src), ARRAY_LEN(dst), src, dst));
 }
 
-static void
-test_names_greater_that_files_fail(void)
+TEST(names_greater_that_files_fail)
 {
 	char *src[] = { "a" };
 	char *dst[] = { "a", "b" };
 	assert_false(is_name_list_ok(ARRAY_LEN(src), ARRAY_LEN(dst), src, dst));
 }
 
-static void
-test_move_fail(void)
+TEST(move_fail)
 {
 	char *src[] = { "a", "b" };
 	char *dst[] = { "../a", "b" };
@@ -35,8 +32,7 @@ test_move_fail(void)
 #endif
 }
 
-static void
-test_rename_inside_subdir_ok(void)
+TEST(rename_inside_subdir_ok)
 {
 	char *src[] = { "../a", "b" };
 	char *dst[] = { "../a_a", "b" };
@@ -49,26 +45,24 @@ test_rename_inside_subdir_ok(void)
 #endif
 }
 
-static void
-test_incdec_leaves_zeros(void)
+TEST(incdec_leaves_zeros)
 {
-	assert_string_equal("1", add_to_name("0", 1));
-	assert_string_equal("01", add_to_name("00", 1));
-	assert_string_equal("00", add_to_name("01", -1));
-	assert_string_equal("-01", add_to_name("00", -1));
-	assert_string_equal("002", add_to_name("001", 1));
-	assert_string_equal("012", add_to_name("005", 7));
-	assert_string_equal("008", add_to_name("009", -1));
-	assert_string_equal("010", add_to_name("009", 1));
-	assert_string_equal("100", add_to_name("099", 1));
-	assert_string_equal("-08", add_to_name("-09", 1));
-	assert_string_equal("-10", add_to_name("-09", -1));
-	assert_string_equal("-14", add_to_name("-09", -5));
-	assert_string_equal("a01.", add_to_name("a00.", 1));
+	assert_string_equal("1", incdec_name("0", 1));
+	assert_string_equal("01", incdec_name("00", 1));
+	assert_string_equal("00", incdec_name("01", -1));
+	assert_string_equal("-01", incdec_name("00", -1));
+	assert_string_equal("002", incdec_name("001", 1));
+	assert_string_equal("012", incdec_name("005", 7));
+	assert_string_equal("008", incdec_name("009", -1));
+	assert_string_equal("010", incdec_name("009", 1));
+	assert_string_equal("100", incdec_name("099", 1));
+	assert_string_equal("-08", incdec_name("-09", 1));
+	assert_string_equal("-10", incdec_name("-09", -1));
+	assert_string_equal("-14", incdec_name("-09", -5));
+	assert_string_equal("a01.", incdec_name("a00.", 1));
 }
 
-static void
-test_single_file_rename(void)
+TEST(single_file_rename)
 {
 	chdir("test-data/rename");
 	assert_true(check_file_rename(".", "a", "a", ST_NONE) < 0);
@@ -81,8 +75,7 @@ test_single_file_rename(void)
 	chdir("../..");
 }
 
-static void
-test_rename_list_checks(void)
+TEST(rename_list_checks)
 {
 	int i;
 	char *list[] = { "a", "aa", "aaa" };
@@ -97,21 +90,5 @@ test_rename_list_checks(void)
 	}
 }
 
-void
-rename_tests(void)
-{
-	test_fixture_start();
-
-	run_test(test_names_less_than_files);
-	run_test(test_names_greater_that_files_fail);
-	run_test(test_move_fail);
-	run_test(test_rename_inside_subdir_ok);
-	run_test(test_incdec_leaves_zeros);
-	run_test(test_single_file_rename);
-	run_test(test_rename_list_checks);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
