@@ -1,16 +1,15 @@
+#include <stic.h>
+
 #include <stddef.h> /* NULL */
 #include <string.h>
 
-#include "seatest.h"
-
 #include "../../src/engine/cmds.h"
 #include "../../src/engine/completion.h"
 #include "../../src/commands.h"
 
 #include "asserts.h"
 
-static void
-vim_like_completion(void)
+TEST(vim_like_completion)
 {
 	init_commands();
 
@@ -38,15 +37,5 @@ vim_like_completion(void)
 	reset_cmds();
 }
 
-void
-test_reserved_commands(void)
-{
-	test_fixture_start();
-
-	run_test(vim_like_completion);
-
-	test_fixture_end();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
