@@ -1,12 +1,11 @@
-#include <stdlib.h>
+#include <stic.h>
 
-#include "seatest.h"
+#include <stdlib.h>
 
 #include "../../src/engine/completion.h"
 #include "../../src/engine/options.h"
 
-static void
-test_space_at_the_end(void)
+TEST(space_at_the_end)
 {
 	const char *start;
 	char *completed;
@@ -24,6 +23,10 @@ test_space_at_the_end(void)
 	assert_string_equal("all", completed);
 	free(completed);
 
+	completed = vle_compl_next();
+	assert_string_equal("cdpath", completed);
+	free(completed);
+
 	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
@@ -33,8 +36,7 @@ test_space_at_the_end(void)
 	free(completed);
 }
 
-static void
-test_one_choice_opt(void)
+TEST(one_choice_opt)
 {
 	const char *start;
 	char *completed;
@@ -55,8 +57,7 @@ test_one_choice_opt(void)
 	free(completed);
 }
 
-static void
-test_one_choice_val(void)
+TEST(one_choice_val)
 {
 	char buf[] = "sort=n";
 	const char *start;
@@ -79,8 +80,7 @@ test_one_choice_val(void)
 	free(completed);
 }
 
-static void
-test_invalid_input(void)
+TEST(invalid_input)
 {
 	const char *start;
 	char *completed;
@@ -92,8 +92,7 @@ test_invalid_input(void)
 	free(completed);
 }
 
-static void
-test_skip_abbreviations(void)
+TEST(skip_abbreviations)
 {
 	const char *start;
 	char *completed;
@@ -105,6 +104,10 @@ test_skip_abbreviations(void)
 	assert_string_equal("all", completed);
 	free(completed);
 
+	completed = vle_compl_next();
+	assert_string_equal("cdpath", completed);
+	free(completed);
+
 	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
@@ -138,8 +141,7 @@ test_skip_abbreviations(void)
 	free(completed);
 }
 
-static void
-test_expand_abbreviations(void)
+TEST(expand_abbreviations)
 {
 	const char *start;
 	char *completed;
@@ -156,8 +158,7 @@ test_expand_abbreviations(void)
 	free(completed);
 }
 
-static void
-test_after_eq_value_completion(void)
+TEST(after_eq_value_completion)
 {
 	char buf[] = "vifminfo=op";
 	const char *start;
@@ -176,8 +177,7 @@ test_after_eq_value_completion(void)
 	free(completed);
 }
 
-static void
-test_after_meq_value_completion(void)
+TEST(after_meq_value_completion)
 {
 	char buf[] = "vifminfo-=op";
 	const char *start;
@@ -196,8 +196,7 @@ test_after_meq_value_completion(void)
 	free(completed);
 }
 
-static void
-test_after_peq_value_completion(void)
+TEST(after_peq_value_completion)
 {
 	char buf[] = "vifminfo+=op";
 	const char *start;
@@ -216,8 +215,7 @@ test_after_peq_value_completion(void)
 	free(completed);
 }
 
-static void
-test_set_values_completion(void)
+TEST(set_values_completion)
 {
 	char buf[] = "vifminfo=tui,c";
 	const char *start;
@@ -240,8 +238,7 @@ test_set_values_completion(void)
 	free(completed);
 }
 
-static void
-test_colon(void)
+TEST(colon)
 {
 	const char *start;
 	char *completed;
@@ -259,6 +256,10 @@ test_colon(void)
 	assert_string_equal("all", completed);
 	free(completed);
 
+	completed = vle_compl_next();
+	assert_string_equal("cdpath", completed);
+	free(completed);
+
 	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
@@ -268,8 +269,7 @@ test_colon(void)
 	free(completed);
 }
 
-static void
-test_umbiguous_beginning(void)
+TEST(umbiguous_beginning)
 {
 	const char *start;
 	char *completed;
@@ -286,8 +286,7 @@ test_umbiguous_beginning(void)
 	free(completed);
 }
 
-static void
-test_matching_short_full(void)
+TEST(matching_short_full)
 {
 	const char *start;
 	char *completed;
@@ -308,8 +307,7 @@ test_matching_short_full(void)
 	free(completed);
 }
 
-static void
-test_after_equal_sign_completion_ok(void)
+TEST(after_equal_sign_completion_ok)
 {
 	const char *start;
 	char *completed;
@@ -325,8 +323,7 @@ test_after_equal_sign_completion_ok(void)
 	free(completed);
 }
 
-static void
-test_after_equal_sign_completion_spaces_ok(void)
+TEST(after_equal_sign_completion_spaces_ok)
 {
 	const char *start;
 	char *completed;
@@ -342,8 +339,7 @@ test_after_equal_sign_completion_spaces_ok(void)
 	free(completed);
 }
 
-static void
-test_after_fake_equal_sign_completion_fail(void)
+TEST(after_fake_equal_sign_completion_fail)
 {
 	const char *start;
 	char *completed;
@@ -359,8 +355,7 @@ test_after_fake_equal_sign_completion_fail(void)
 	free(completed);
 }
 
-static void
-test_all_completion_ok(void)
+TEST(all_completion_ok)
 {
 	const char *start;
 	char *completed;
@@ -373,8 +368,7 @@ test_all_completion_ok(void)
 	free(completed);
 }
 
-static void
-test_charset_completion_skips_entered_elements(void)
+TEST(charset_completion_skips_entered_elements)
 {
 	const char *start;
 	char *completed;
@@ -399,31 +393,60 @@ test_charset_completion_skips_entered_elements(void)
 	free(completed);
 }
 
-void
-opt_completion(void)
+TEST(charset_hat_completion_works)
 {
-	test_fixture_start();
+	const char *start;
+	char *completed;
 
-	run_test(test_space_at_the_end);
-	run_test(test_one_choice_opt);
-	run_test(test_one_choice_val);
-	run_test(test_invalid_input);
-	run_test(test_skip_abbreviations);
-	run_test(test_expand_abbreviations);
-	run_test(test_after_eq_value_completion);
-	run_test(test_after_meq_value_completion);
-	run_test(test_after_peq_value_completion);
-	run_test(test_set_values_completion);
-	run_test(test_colon);
-	run_test(test_umbiguous_beginning);
-	run_test(test_matching_short_full);
-	run_test(test_after_equal_sign_completion_ok);
-	run_test(test_after_equal_sign_completion_spaces_ok);
-	run_test(test_after_fake_equal_sign_completion_fail);
-	run_test(test_all_completion_ok);
-	run_test(test_charset_completion_skips_entered_elements);
+	vle_compl_reset();
+	complete_options("cpo^=a", &start);
 
-	test_fixture_end();
+	completed = vle_compl_next();
+	assert_string_equal("b", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("c", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("b", completed);
+	free(completed);
 }
 
-/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab : */
+TEST(no_no_or_inv_completion_for_all_pseudo_option)
+{
+	const char *start;
+	char *completed;
+
+	vle_compl_reset();
+	complete_options("noa", &start);
+	assert_string_equal("a", start);
+
+	completed = vle_compl_next();
+	assert_string_equal("a", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("a", completed);
+	free(completed);
+
+	vle_compl_reset();
+	complete_options("inva", &start);
+	assert_string_equal("a", start);
+
+	completed = vle_compl_next();
+	assert_string_equal("a", completed);
+	free(completed);
+
+	completed = vle_compl_next();
+	assert_string_equal("a", completed);
+	free(completed);
+}
+
+/* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
+/* vim: set cinoptions+=t0 : */
