@@ -1,5 +1,5 @@
+#include "../src/ui/ui.h"
 #include "../src/utils/macros.h"
-#include "../src/ui.h"
 
 void
 vifm_restart(void)
@@ -8,13 +8,19 @@ vifm_restart(void)
 }
 
 void
-vifm_try_leave(int write_info, int force)
+vifm_try_leave(int write_info, int cquit, int force)
 {
 	/* Do nothing. */
 }
 
 void
-vifm_return_file_list(const FileView *view, int nfiles, char *files[])
+vifm_choose_files(const FileView *view, int nfiles, char *files[])
+{
+	/* Do nothing. */
+}
+
+void
+vifm_finish(const FileView *view, int nfiles, char *files[])
 {
 	/* Do nothing. */
 }
